function P = PhysProps(Name)
%PhysProps returns physical properties of selected materials.
% 
% P = PhysProps(Name) returns physical properties of selected materials.
%  Input:
%  - Name - name (string) or names (cell array) identifying the material.
%    Possible name formats:
%    * Chemical name, like Carbon, Benzine, or Blood 
%    * Chemical formula, like C or H2O (caution in case of compound
%      formulas there can be a single formula for different chemicals with
%      unique properties - only the first one will be returned).
%    * In case of elements input of atomic numbers in 1-100 is also allowed.
%
%  Output:
%  - P - 2D cell array: each row corresponds to one 'name', columns are as
%    follows:
%    * column 1 - Z/A - mean ratio of atomic number to mass
%    * column 2 - density in g/cm^3
%    * column 3 - material formula guaranteed to be recognized by
%    'PhotonAttenuation' and 'ParseChemicalFormula' functions
%    * column 4 - name of the material (name used in NIST tables)
% 
% P = PhysProps('Compound Names') will return a list of about 200 compounds
% and mixtures.
%
% P = PhysProps('Element Data') will return a list of element symbols and
% Z/A's. 

% Sources:
% - elements:  http://physics.nist.gov/PhysRefData/XrayMassCoef/tab1.html
% - materials: http://physics.nist.gov/cgi-bin/Star/compos.pl
%
% See Also:
%   ParseChemicalFormula, PhotonAttenuation, PhotonAttenuationQ
% 
% Written by Jarek Tuszynski (SAIC), 2006
%
% Examples:
%   PhysProps({'H', 'Water', 'H2O', 82, 'lead', 'skin'})  
%   PhysProps('Compound Names') % list of aviable names 
%   PhysProps('Element Data')   % list of element symbols and Z/A
%   PhysProps('All Data')       % returns the database variable
%
% % get properties of concrete
% X = PhysProps('Concrete');
% [Z R] = ParseChemicalFormula(X{3});
% MFP = PhotonAttenuation(X{3}, 0.662, 'mean free path');
% Concrete.Density      = X{2};
% Concrete.Z_A          = X{1};
% Concrete.ElementZ     = Z';
% Concrete.ElementRatio = R';
% Concrete.MeaFreePath  = MFP; % of gammas from Cesium-137 source
% Concrete

%% -----------------------------------------------------------------------
persistent Props
% Z/A 	 Density     Names
Props = {...
0.99212, 8.375e-005, 'H', 'Hydrogen', '' ;...
0.49968, 1.663e-004, 'He', 'Helium', '' ;...
0.43221, 5.340e-001, 'Li', 'Lithium', '' ;...
0.44384, 1.848e+000, 'Be', 'Beryllium', '' ;...
0.46245, 2.370e+000, 'B', 'Boron', '' ;...
0.49954, 1.700e+000, 'C', 'Carbon', '' ;...
0.49976, 1.165e-003, 'N', 'Nitrogen', '' ;...
0.50002, 1.332e-003, 'O', 'Oxygen', '' ;...
0.47372, 1.580e-003, 'F', 'Fluorine', '' ;...
0.49555, 8.385e-004, 'Ne', 'Neon', '' ;...
0.47847, 9.710e-001, 'Na', 'Sodium', '' ;...
0.49373, 1.740e+000, 'Mg', 'Magnesium', '' ;...
0.48181, 2.699e+000, 'Al', 'Aluminum', '' ;...
0.49848, 2.330e+000, 'Si', 'Silicon', '' ;...
0.48428, 2.200e+000, 'P', 'Phosphorus', '' ;...
0.49897, 2.000e+000, 'S', 'Sulfur', '' ;...
0.47951, 2.995e-003, 'Cl', 'Chlorine', '' ;...
0.45059, 1.662e-003, 'Ar', 'Argon', '' ;...
0.48595, 8.620e-001, 'K', 'Potassium', '' ;...
0.49903, 1.550e+000, 'Ca', 'Calcium', '' ;...
0.46712, 2.989e+000, 'Sc', 'Scandium', '' ;...
0.45948, 4.540e+000, 'Ti', 'Titanium', '' ;...
0.45150, 6.110e+000, 'V', 'Vanadium', '' ;...
0.46157, 7.180e+000, 'Cr', 'Chromium', '' ;...
0.45506, 7.440e+000, 'Mn', 'Manganese', '' ;...
0.46556, 7.874e+000, 'Fe', 'Iron', '' ;...
0.45815, 8.900e+000, 'Co', 'Cobalt', '' ;...
0.47708, 8.902e+000, 'Ni', 'Nickel', '' ;...
0.45636, 8.960e+000, 'Cu', 'Copper', '' ;...
0.45879, 7.133e+000, 'Zn', 'Zinc', '' ;...
0.44462, 5.904e+000, 'Ga', 'Gallium', '' ;...
0.44071, 5.323e+000, 'Ge', 'Germanium', '' ;...
0.44046, 5.730e+000, 'As', 'Arsenic', '' ;...
0.43060, 4.500e+000, 'Se', 'Selenium', '' ;...
0.43803, 7.072e-003, 'Br', 'Bromine', '' ;...
0.42959, 3.478e-003, 'Kr', 'Krypton', '' ;...
0.43291, 1.532e+000, 'Rb', 'Rubidium', '' ;...
0.43369, 2.540e+000, 'Sr', 'Strontium', '' ;...
0.43867, 4.469e+000, 'Y', 'Yttrium', '' ;...
0.43848, 6.506e+000, 'Zr', 'Zirconium', '' ;...
0.44130, 8.570e+000, 'Nb', 'Niobium', '' ;...
0.43777, 1.022e+001, 'Mo', 'Molybdenum', '' ;...
0.43919, 1.150e+001, 'Tc', 'Technetium', '' ;...
0.43534, 1.241e+001, 'Ru', 'Ruthenium', '' ;...
0.43729, 1.241e+001, 'Rh', 'Rhodium', '' ;...
0.43225, 1.202e+001, 'Pd', 'Palladium', '' ;...
0.43572, 1.050e+001, 'Ag', 'Silver', '' ;...
0.42700, 8.650e+000, 'Cd', 'Cadmium', '' ;...
0.42676, 7.310e+000, 'In', 'Indium', '' ;...
0.42120, 7.310e+000, 'Sn', 'Tin', '' ;...
0.41889, 6.691e+000, 'Sb', 'Antimony', '' ;...
0.40752, 6.240e+000, 'Te', 'Tellurium', '' ;...
0.41764, 4.930e+000, 'I', 'Iodine', '' ;...
0.41130, 5.485e-003, 'Xe', 'Xenon', '' ;...
0.41383, 1.873e+000, 'Cs', 'Cesium', '' ;...
0.40779, 3.500e+000, 'Ba', 'Barium', '' ;...
0.41035, 6.154e+000, 'La', 'Lanthanum', '' ;...
0.41395, 6.657e+000, 'Ce', 'Cerium', '' ;...
0.41871, 6.710e+000, 'Pr', 'Praseodymium', '' ;...
0.41597, 6.900e+000, 'Nd', 'Neodymium', '' ;...
0.42094, 7.220e+000, 'Pm', 'Promethium', '' ;...
0.41234, 7.460e+000, 'Sm', 'Samarium', '' ;...
0.41457, 5.243e+000, 'Eu', 'Europium', '' ;...
0.40699, 7.900e+000, 'Gd', 'Gadolinium', '' ;...
0.40900, 8.229e+000, 'Tb', 'Terbium', '' ;...
0.40615, 8.550e+000, 'Dy', 'Dysprosium', '' ;...
0.40623, 8.795e+000, 'Ho', 'Holmium', '' ;...
0.40655, 9.066e+000, 'Er', 'Erbium', '' ;...
0.40844, 9.321e+000, 'Tm', 'Thulium', '' ;...
0.40453, 6.730e+000, 'Yb', 'Ytterbium', '' ;...
0.40579, 9.840e+000, 'Lu', 'Lutetium', '' ;...
0.40338, 1.331e+001, 'Hf', 'Hafnium', '' ;...
0.40343, 1.665e+001, 'Ta', 'Tantalum', '' ;...
0.40250, 1.930e+001, 'W', 'Tungsten', '' ;...
0.40278, 2.102e+001, 'Re', 'Rhenium', '' ;...
0.39958, 2.257e+001, 'Os', 'Osmium', '' ;...
0.40058, 2.242e+001, 'Ir', 'Iridium', '' ;...
0.39984, 2.145e+001, 'Pt', 'Platinum', '' ;...
0.40108, 1.932e+001, 'Au', 'Gold', '' ;...
0.39882, 1.355e+001, 'Hg', 'Mercury', '' ;...
0.39631, 1.172e+001, 'Tl', 'Thallium', '' ;...
0.39575, 1.135e+001, 'Pb', 'Lead', '' ;...
0.39717, 9.747e+000, 'Bi', 'Bismuth', '' ;...
0.40195, 9.320e+000, 'Po', 'Polonium', '' ;...
0.40479, 1.000e+001, 'At', 'Astatine', '' ;...
0.38736, 9.066e-003, 'Rn', 'Radon', '' ;...
0.39010, 1.000e+001, 'Fr', 'Francium', '' ;...
0.38934, 5.000e+000, 'Ra', 'Radium', '' ;...
0.39202, 1.007e+001, 'Ac', 'Actinium', '' ;...
0.38787, 1.172e+001, 'Th', 'Thorium', '' ;...
0.39388, 1.537e+001, 'Pa', 'Protactinium', '' ;...
0.38651, 1.895e+001, 'U', 'Uranium', '' ;...
0.39233, 2.030e+001, 'Np', 'Neptunium', '' ;...
0.38514, 1.984e+001, 'Pu', 'Plutonium', '' ;...
0.39085, 1.370e+001, 'Am', 'Americium', '' ;...
0.38855, 1.350e+001, 'Cm', 'Curium', '' ;...
0.39260, 1.400e+001, 'Bk', 'Berkelium', '' ;...
0.39031, 1.000e+001, 'Cf', 'Californium', '' ;...
0.39273, 8.840e+000, 'Es', 'Einsteinium', '' ;...
0.38896, NaN       , 'Fm', 'Fermium', '' ;... % last element
0.48181, 2.699e+000, 'Al', 'Aluminium', 'Al' ;... % Alternative spelling
0.49897, 2.000e+000, 'S', 'Sulphur'   , 'S' ;... % Alternative spelling
0.41383, 1.873e+000, 'Cs', 'Caesium'  , 'Cs' ;... % Alternative spelling
0.49954, 1.700e+000, 'C', 'Graphite'  , 'C' ;... % Alternative form
0.49954, 3.513e+000, 'C', 'Diamond'   , 'C' ;... % Alternative form
0.49954, 2.000e+001, 'C', 'Carbon, Amorphous', 'C' ;... % Alternative form
0.54903, 1.127e+000, 'A-150', 'A-150 TISSUE-EQUIVALENT PLASTIC', 'H(0.101327)C(0.775501)N(0.035057)O(0.052316)F(0.017422)Ca(0.018378)' ;...
0.55097, 7.899e-001, '', 'ACETONE', 'C3H6O' ;...
0.47410, 7.662e-000, '', 'CEMENTITE', 'Fe3C' ;...
0.53768, 1.097e-003, '', 'ACETYLENE', 'C2H2' ;...
0.51803, 1.350e+000, '', 'ADENINE', 'C5H5N5' ;...
0.55847, 9.200e-001, 'ADIPOSE TISSUE', 'ADIPOSE TISSUE (ICRP)', 'H(0.119477)C(0.637240)N(0.007970)O(0.232333)Na(0.000500)Mg(0.000020)P(0.000160)S(0.000730)Cl(0.001190)K(0.000320)Ca(0.000020)Fe(0.000020)Zn(0.000020)' ;...
0.49919, 1.205e-003, 'Air', 'AIR, DRY (NEAR SEA LEVEL)', 'C(0.000124)N(0.755267)O(0.231781)Ar(0.012827)' ;...
0.53876, 1.420e+000, '', 'ALANINE', 'C3H7NO2' ;...
0.49038, 3.970e+000, '', 'ALUMINUM OXIDE', 'Al2O3' ;...
0.55178, 1.100e+000, '', 'AMBER', 'C10H16O' ;...
0.51129, 8.260e-004, '', 'AMMONIA', 'NH3' ;...
0.53690, 1.024e+000, '', 'ANILINE', 'C6H7N' ;...
0.52740, 1.283e+000, '', 'ANTHRACENE', 'C14H10' ;...
0.52740, 1.450e+000, 'B-100', 'B-100 BONE-EQUIVALENT PLASTIC', 'H(0.065471)C(0.536945)N(0.021500)O(0.032085)F(0.167411)Ca(0.176589)' ;...
0.52792, 1.250e+000, '', 'BAKELITE', 'H(0.057441)C(0.774591)O(0.167968)' ;...
0.42207, 4.890e+000, '', 'BARIUM FLUORIDE', 'BaF2' ;...
0.44561, 4.500e+000, '', 'BARIUM SULFATE', 'BaSO4' ;...
0.53768, 8.787e-001, '', 'BENZENE', 'C6H6' ;...
0.47978, 3.010e+000, '', 'BERYLLIUM OXIDE', 'BeO' ;...
0.40961, 7.130e+000, 'BGO', 'BISMUTH GERMANIUM OXIDE', 'Bi12GeO20' ;...
0.54995, 1.060e+000, 'Blood', 'BLOOD (ICRP)', 'H(0.101866)C(0.100020)N(0.029640)O(0.759414)Na(0.001850)Mg(0.000040)Si(0.000030)P(0.000350)S(0.001850)Cl(0.002780)K(0.001630)Ca(0.000060)Fe(0.000460)Zn(0.000010)' ;...
0.53010, 1.850e+000, 'COMPACT BONE', 'BONE, COMPACT (ICRU)', 'H(0.063984)C(0.278000)N(0.027000)O(0.410016)Mg(0.002000)P(0.070000)S(0.002000)Ca(0.147000)' ;...
0.52130, 1.850e+000, 'CORTICAL BONE', 'BONE, CORTICAL (ICRP)', 'H(0.047234)C(0.144330)N(0.041990)O(0.446096)Mg(0.002200)P(0.104970)S(0.003150)Ca(0.209930)Zn(0.000100)' ;...
0.47058, 2.520e+000, '', 'BORON CARBIDE', 'B4C' ;...
0.48838, 1.812e+000, '', 'BORON OXIDE', 'B2O3' ;...
0.55423, 1.030e+000, 'Brain', 'BRAIN (ICRP)', 'H(0.110667)C(0.125420)N(0.013280)O(0.737723)Na(0.001840)Mg(0.000150)P(0.003540)S(0.001770)Cl(0.002360)K(0.003100)Ca(0.000090)Fe(0.000050)Zn(0.000010)' ;...
0.55196, 1.020e+000, 'Breast', 'Breast Tissue (ICRU)', 'H(0.106)C(0.332)N(0.03)O(0.527)Na(0.001)P(0.001)S(0.002)Cl(0.001)' ;...
0.58497, 2.493e-003, '', 'BUTANE', 'C4H10' ;...
0.56663, 8.098e-001, 'n-BUTANOL', 'N-BUTYL ALCOHOL', 'C4H10O' ;...
0.49969, 1.760e+000, 'C-552', 'C-552 AIR-EQUIVALENT PLASTIC', 'H(0.024680)C(0.501610)O(0.004527)F(0.465209)Si(0.003973)' ;...
0.41665, 6.200e+000, '', 'CADMIUM TELLURIDE', 'CdTe' ;...
0.40749, 7.900e+000, 'CWO', 'CADMIUM TUNGSTATE', 'CdWO4' ;...
0.49955, 2.800e+000, '', 'CALCIUM CARBONATE', 'CaCO3' ;...
0.48670, 3.180e+000, '', 'CALCIUM FLUORIDE', 'CaF2' ;...
0.49929, 3.300e+000, '', 'CALCIUM OXIDE', 'O(0.285299)Ca(0.714701)' ;...
0.49950, 2.960e+000, '', 'CALCIUM SULFATE', 'CaSO4' ;...
0.40936, 6.062e+000, '', 'CALCIUM TUNGSTATE', 'CaWO4' ;...
0.49989, 1.842e-003, '', 'CARBON DIOXIDE', 'CO2' ;...
0.48107, 1.594e+000, '', 'CARBON TETRACHLORIDE', 'CCl4' ;...
0.53040, 1.420e+000, 'CELLOPHANE', 'CELLULOSE ACETATE, CELLOPHANE', 'C6H10O5' ;...
0.53279, 1.200e+000, 'CAB', 'CELLULOSE ACETATE BUTYRATE', 'C15H22O8' ;...
0.51424, 1.490e+000, 'Nitrocellulose', 'CELLULOSE NITRATE', 'C6H8N2O9' ;...
0.55278, 1.030e+000, '', 'CERIC SULFATE DOSIMETER SOLUTION', 'H(0.107596)N(0.000800)O(0.874976)S(0.014627)Ce(0.002001)' ;...
0.42132, 4.115e+000, '', 'CESIUM FLUORIDE', 'CsF' ;...
0.41569, 4.510e+000, '', 'CESIUM IODIDE', 'CsI' ;...
0.48734, 1.106e+000, '', 'CHLOROBENZENE', 'C6H5Cl' ;...
0.51498, 1.483e+000, '', 'CHLOROFORM', 'CHCl3' ;...
0.50274, 2.300e+000, 'Portland Concrete', 'CONCRETE, PORTLAND', 'H(0.010000)C(0.001000)O(0.529107)Na(0.016000)Mg(0.002000)Al(0.033872)Si(0.337021)K(0.013000)Ca(0.044000)Fe(0.014000)' ;...
0.50932, 2.300e+000, 'Concrete', 'Concrete, Ordinary', 'H(0.0221)C(0.002484)O(0.57493)Na(0.015208)Mg(0.001266)Al(0.019953)Si(0.304627)K(0.010045)Ca(0.042951)Fe(0.006435)' ;...
0.45714, 3.350e+000, 'Barite Concrete', 'Concrete, Barite', 'H(0.003585)O(0.311622)Mg(0.001195)Al(0.004183)Si(0.010457)S(0.107858)Ca(0.050194)Fe(0.047505)Ba(0.4634)' ;...
0.57034, 7.790e-001, '', 'CYCLOHEXANE', 'C6H12' ;...
0.49098, 1.305e+000, 'ortho-dichlorobenzene', '1, 2-DICHLOROBENZENE', 'C6H4Cl2' ;...
0.48616, 1.220e+000, 'DCDE', 'DICHLORODIETHYL ETHER', 'C4H8Cl2O' ;...
0.48853, 1.235e+000, 'EDC', '1, 2-DICHLOROETHANE', 'C2H4Cl2' ;...
0.56663, 7.138e-001, '', 'DIETHYL ETHER', 'C4H10O' ;...
0.54724, 9.487e-001, '', 'N,N-DIMETHYL FORMAMIDE', 'C3H7NO' ;...
0.53757, 1.101e+000, 'DMSO', 'DIMETHYL SULFOXIDE', 'C2H6OS' ;...
0.59862, 1.253e-003, '', 'ETHANE', 'C2H6' ;...
0.56437, 7.893e-001, 'Ethanol', 'ETHYL ALCOHOL', 'C2H6O' ;...
0.54405, 1.130e+000, '', 'ETHYL CELLULOSE', 'C12H22O5' ;...
0.57034, 1.175e-003, '', 'ETHYLENE', 'C2H4' ;...
0.54877, 1.100e+000, 'EYE LENS', 'EYE LENS (ICRP)', 'H(0.099269)C(0.193710)N(0.053270)O(0.653751)' ;...
0.47592, 5.200e+000, 'Iron(III) oxide', 'FERRIC OXIDE', 'Fe2O3' ;...
0.46507, 7.150e+000, '', 'FERROBORIDE', 'FeB' ;...
0.47323, 5.700e+000, 'Iron(II) oxide', 'FERROUS OXIDE', 'FeO' ;...
0.55328, 1.024e+000, '', 'FERROUS SULFATE DOSIMETER SOLUTION', 'H(0.108259)N(0.000027)O(0.878636)Na(0.000022)S(0.012968)Cl(0.000034)Fe(0.000054)' ;...
0.47968, 1.120e+000, 'Dichlorodifluoromethane', 'FREON-12', 'CCl2F2' ;...
0.44801, 1.800e+000, 'Dibromodifluoromethane', 'FREON-12B2', 'CBr2F2' ;...
0.47866, 9.500e-001, '', 'FREON-13', 'CClF3' ;...
0.45665, 1.500e+000, '', 'FREON-13B1', 'CBrF3' ;...
0.42262, 1.800e+000, '', 'FREON-13I1', 'CIF3' ;...
0.42266, 7.440e+000, '', 'GADOLINIUM OXYSULFIDE', 'Gd2O2S' ;...
0.44247, 5.310e+000, '', 'GALLIUM ARSENIDE', 'GaAs' ;...
0.53973, 1.291e+000, '', 'GEL IN PHOTOGRAPHIC EMULSION', 'H(0.081180)C(0.416060)N(0.111240)O(0.380640)S(0.010880)' ;...
0.49707, 2.230e+000, 'Pyrex Glass', 'Glass, Borosilicate', 'B(0.040064)O(0.539562)Na(0.028191)Al(0.011644)Si(0.377220)K(0.003321)' ;...
0.42101, 6.220e+000, 'Lead Glass', 'GLASS, LEAD', 'O(0.156453)Si(0.080866)Ti(0.008092)As(0.002651)Pb(0.751938)' ;...
0.49731, 2.400e+000, 'Plate Glass', 'GLASS, PLATE', 'O(0.459800)Na(0.096441)Si(0.336553)Ca(0.107205)' ;...
0.53489, 1.540e+000, 'GLc', 'GLUCOSE', 'C6H14O7' ;...
0.53371, 1.460e+000, '', 'GLUTAMINE', 'C5H10N2O3' ;...
0.54292, 1.261e+000, 'Glycerin', 'GLYCEROL', 'C3H8O3' ;...
0.51612, 1.580e+000, '', 'GUANINE', 'C5H5N5O' ;...
0.50039, 2.320e+000, 'GYPSUM', 'GYPSUM, PLASTER OF PARIS', 'H4CaSO6' ;...
0.57882, 6.838e-001, 'HEPTANE', 'N-HEPTANE', 'C7H16' ;...
0.58020, 6.603e-001, 'HEXANE', 'N-HEXANE', 'C6H14' ;...
0.51264, 1.420e+000, '', 'KAPTON POLYIMIDE FILM', 'C22H10N2O5' ;...
0.42588, 6.280e+000, '', 'LANTHANUM OXYBROMIDE', 'LaOBr' ;...
0.42706, 5.860e+000, '', 'LANTHANUM OXYSULFIDE', 'La2OS' ;...
0.40323, 9.530e+000, '', 'LEAD OXIDE', 'PbO' ;...
0.50052, 1.178e+000, '', 'LITHIUM AMIDE', 'LiNH2' ;...
0.48720, 2.110e+000, '', 'LITHIUM CARBONATE', 'Li2CO3' ;...
0.46262, 2.635e+000, '', 'LITHIUM FLUORIDE', 'LiF' ;...
0.50321, 8.200e-001, '', 'LITHIUM HYDRIDE', 'LiH' ;...
0.41839, 3.494e+000, '', 'LITHIUM IODIDE', 'LiI' ;...
0.46852, 2.013e+000, '', 'LITHIUM OXIDE', 'Li2O' ;...
0.48487, 2.440e+000, '', 'LITHIUM TETRABORATE', 'Li2B4O7' ;...
0.54965, 1.050e+000, 'Lung', 'LUNG (ICRP)', 'H(0.101278)C(0.102310)N(0.028650)O(0.757072)Na(0.001840)Mg(0.000730)P(0.000800)S(0.002250)Cl(0.002660)K(0.001940)Ca(0.000090)Fe(0.000370)Zn(0.000010)' ;...
0.55512, 1.050e+000, '', 'M3 WAX', 'H(0.114318)C(0.655823)O(0.092183)Mg(0.134792)Ca(0.002883)' ;...
0.49814, 2.958e+000, '', 'MAGNESIUM CARBONATE', 'MgCO3' ;...
0.48153, 3.000e+000, '', 'MAGNESIUM FLUORIDE', 'MgF2' ;...
0.49622, 3.580e+000, '', 'MAGNESIUM OXIDE', 'MgO' ;...
0.49014, 2.530e+000, '', 'MAGNESIUM TETRABORATE', 'MgB4O7' ;...
0.40933, 6.360e+000, '', 'MERCURIC IODIDE', 'HgI2' ;...
0.62334, 6.672e-004, '', 'METHANE', 'CH4' ;...
0.56176, 7.914e-001, '', 'METHANOL', 'CH4O' ;...
0.56479, 9.900e-001, '', 'MIX D WAX', 'H(0.134040)C(0.777960)O(0.035020)Mg(0.038594)Ti(0.014386)' ;...
0.53886, 1.000e+000, '', 'MS20 TISSUE SUBSTITUTE', 'H(0.081192)C(0.583442)N(0.017798)O(0.186381)Mg(0.130287)Cl(0.000900)' ;...
0.54938, 1.040e+000, 'SKELETAL MUSCLE', 'MUSCLE, SKELETAL (ICRP)', 'H(0.100637)C(0.107830)N(0.027680)O(0.754773)Na(0.000750)Mg(0.000190)P(0.001800)S(0.002410)Cl(0.000790)K(0.003020)Ca(0.000030)Fe(0.000040)Zn(0.000050)' ;...
0.55005, 1.040e+000, 'STRIATED MUSCLE', 'MUSCLE, STRIATED (ICRU)', 'H(0.101997)C(0.123000)N(0.035000)O(0.729003)Na(0.000800)Mg(0.000200)P(0.002000)S(0.005000)K(0.003000)' ;...
0.54828, 1.110e+000, '', 'MUSCLE-EQUIVALENT LIQUID, WITH SUCROSE', 'H(0.098234)C(0.156214)N(0.035451)O(0.710100)' ;...
0.55014, 1.070e+000, '', 'MUSCLE-EQUIVALENT LIQUID, WITHOUT SUCROSE', 'H(0.101969)C(0.120058)N(0.035451)O(0.742522)' ;...
0.53053, 1.145e+000, '', 'NAPHTHALENE', 'C10H8' ;...
0.51986, 1.199e+000, '', 'NITROBENZENE', 'C6H5NO2' ;...
0.49985, 1.831e-003, '', 'NITROUS OXIDE', 'N2O' ;...
0.55063, 1.080e+000, '', 'NYLON, DU PONT ELVAMIDE 8062', 'H(0.103509)C(0.648415)N(0.099536)O(0.148539)' ;...
0.54790, 1.140e+000, '', 'NYLON, TYPE 6 AND TYPE 6/6', 'C12H22O2N2' ;...
0.55236, 1.140e+000, '', 'NYLON, TYPE 6/10', 'C8H15ON' ;...
0.55649, 1.425e+000, '', 'NYLON, TYPE 11 (RILSAN)', 'C11H21ON' ;...
0.57778, 7.026e-001, 'Liquid Octane', 'OCTANE, LIQUID', 'C8H18' ;...
0.55149, 2.200e+000, '', 'Ovary', 'H(0.105)C(0.093)N(0.024)O(0.768)Na(0.002)P(0.002)S(0.002)Cl(0.002)K(0.002)' ;...
0.57275, 9.300e-001, 'PARAFFIN', 'PARAFFIN WAX', 'C25H52' ;...
0.58212, 6.262e-001, 'PENTANE', 'N-PENTANE', 'C5H12' ;...
0.48176, 3.820e+000, 'Kodak Photo Emulsion', 'Photographic Emulsion (Kodak Type AA)', 'H(0.0305)C(0.2107)N(0.0721)O(0.1632)Br(0.2228)Ag(0.3007)' ;...
0.45453, 1.030e+000, 'Nuclear Photo Emulsion', 'Photographic Emulsion (Standard Nuclear)', 'H(0.0141)C(0.072261)N(0.01932)O(0.066101)S(0.00189)Br(0.349104)Ag(0.474105)I(0.00312)' ;...
0.45453, 3.815e+000, '', 'PHOTOGRAPHIC EMULSION', 'H(0.0141)C(0.072261)N(0.01932)O(0.066101)S(0.00189)Br(0.349103)Ag(0.474105)I(0.00312)' ;...
0.54141, 1.032e+000, 'VINYLTOLUENE', 'PLASTIC SCINTILLATOR (VINYLTOLUENE BASED)', 'C9H10' ;...
0.38879, 1.146e+001, '', 'PLUTONIUM DIOXIDE', 'PuO2' ;...
0.52767, 1.170e+000, 'PAN', 'POLYACRYLONITRILE', 'C3H3N' ;...
0.52697, 1.200e+000, 'MAKROLON', 'POLYCARBONATE (MAKROLON, LEXAN)', 'C16H14O3' ;...
0.48558, 1.300e+000, '', 'POLYCHLOROSTYRENE', 'C17H18Cl2' ;...
0.57034, 9.400e-001, '', 'POLYETHYLENE', 'C2H4' ;...
0.52037, 1.400e+000, '', 'POLYETHYLENE TEREPHTHALATE (MYLAR)', 'C10H8O4' ;...
0.53937, 1.190e+000, 'PLEXIGLASS', 'POLYMETHYL METHACRALATE (LUCITE, PERSPEX, PLEXIGLASS)', 'C5H8O2' ;...
0.51183, 1.425e+000, '', 'POLYOXYMETHYLENE', 'H2CO' ;...
0.57034, 9.000e-001, '', 'POLYPROPYLENE', 'C3H6' ;...
0.53768, 1.060e+000, '', 'POLYSTYRENE', 'CH' ;...
0.47992, 2.200e+000, 'Teflon', 'POLYTETRAFLUOROETHYLENE (TEFLON)', 'C2F4' ;...
0.48081, 2.100e+000, '', 'POLYTRIFLUOROCHLOROETHYLENE', 'C2F3Cl' ;...
0.53432, 1.190e+000, 'PVA', 'POLYVINYL ACETATE', 'C4H6O2' ;...
0.54480, 1.300e+000, 'PVOH', 'POLYVINYL ALCOHOL', 'C2H4O' ;...
0.54537, 1.120e+000, 'PVB', 'POLYVINYL BUTYRAL', 'C8H13O2' ;...
0.48710, 1.300e+000, 'PVC', 'POLYVINYL CHLORIDE', 'C2H3Cl' ;...
0.49513, 1.700e+000, 'SARAN', 'POLYVINYLIDENE CHLORIDE', 'CHCl' ;...
0.49973, 1.760e+000, 'Kynar', 'POLYVINYLIDENE FLUORIDE', 'C2H2F2' ;...
0.53984, 1.250e+000, 'PVP', 'POLYVINYL PYRROLIDONE', 'C6H9NO' ;...
0.43373, 3.130e+000, '', 'POTASSIUM IODIDE', 'KI' ;...
0.48834, 2.320e+000, '', 'POTASSIUM OXIDE', 'K2O' ;...
0.58962, 1.879e-003, '', 'PROPANE', 'C3H8' ;...
0.58962, 4.300e-001, 'LIQUID PROPANE', 'PROPANE, LIQUID', 'C3H8' ;...
0.56576, 8.035e-001, '1-Propanol', 'N-PROPYL ALCOHOL', 'C3H8O' ;...
0.53097, 9.819e-001, 'Azine', 'PYRIDINE', 'C5H5N' ;...
0.57034, 9.200e-001, 'BUTYL', 'RUBBER, BUTYL', 'C4H8' ;...
0.55785, 9.200e-001, 'Latex', 'RUBBER, NATURAL', 'C5H8' ;...
0.48605, 1.230e+000, 'NEOPRENE', 'RUBBER, NEOPRENE', 'C4H5Cl' ;...
0.49930, 2.320e+000, '', 'SILICON DIOXIDE', 'SiO2' ;...
0.43670, 6.473e+000, '', 'SILVER BROMIDE', 'AgBr' ;...
0.44655, 5.560e+000, '', 'SILVER CHLORIDE', 'AgCl' ;...
0.43663, 6.470e+000, '', 'SILVER HALIDES IN PHOTOGRAPHIC EMULSION', 'Br(0.422895)Ag(0.573748)I(0.003357)' ;...
0.42594, 6.010e+000, '', 'SILVER IODIDE', 'AgI' ;...
0.54932, 1.100e+000, 'Skin', 'SKIN (ICRP)', 'H(0.100588)C(0.228250)N(0.046420)O(0.619002)Na(0.000070)Mg(0.000060)P(0.000330)S(0.001590)Cl(0.002670)K(0.000850)Ca(0.000150)Fe(0.000010)Zn(0.000010)' ;...
0.49062, 2.532e+000, 'Washing soda', 'SODIUM CARBONATE', 'Na2CO3' ;...
0.42697, 3.667e+000, '', 'SODIUM IODIDE', 'NaI' ;...
0.48403, 2.270e+000, '', 'SODIUM MONOXIDE', 'Na2O' ;...
0.49415, 2.261e+000, '', 'SODIUM NITRATE', 'NaNO3' ;...
0.53260, 9.707e-001, 'trans-1, 2-diphenylethylene', 'STILBENE', 'C14H12' ;...
0.53170, 1.581e+000, 'Sugar', 'SUCROSE', 'C12H22O11' ;...
0.52148, 1.234e+000, 'p-Terphenyl', 'TERPHENYL', 'C18H10' ;...
0.55108, 1.040e+000, 'TESTES', 'TESTES (ICRP)', 'H(0.104166)C(0.092270)N(0.019940)O(0.773884)Na(0.002260)Mg(0.000110)P(0.001250)S(0.001460)Cl(0.002440)K(0.002080)Ca(0.000100)Fe(0.000020)Zn(0.000020)' ;...
0.48241, 1.625e+000, 'PCE', 'TETRACHLOROETHYLENE', 'C2Cl4' ;...
0.40861, 7.004e+000, '', 'THALLIUM CHLORIDE', 'TlCl' ;...
0.55121, 1.000e+000, 'Soft Tissue', 'TISSUE, SOFT (ICRP)', 'H(0.104472)C(0.232190)N(0.024880)O(0.630238)Na(0.001130)Mg(0.000130)P(0.001330)S(0.001990)Cl(0.001340)K(0.001990)Ca(0.000230)Fe(0.000050)Zn(0.000030)' ;...
0.54975, 1.000e+000, '4 component soft tissue', 'TISSUE, SOFT (ICRP four components)', 'H(0.101172)C(0.111000)N(0.026000)O(0.761828)' ;...
0.54993, 1.064e-003, '', 'TISSUE-EQUIVALENT GAS (METHANE BASED)', 'H(0.101869)C(0.456179)N(0.035172)O(0.406780)' ;...
0.55027, 1.826e-003, '', 'TISSUE-EQUIVALENT GAS (PROPANE BASED)', 'H(0.102672)C(0.568940)N(0.035022)O(0.293366)' ;...
0.46528, 4.260e+000, '', 'TITANIUM DIOXIDE', 'TiO2' ;...
0.54265, 8.669e-001, 'methylbenzene', 'TOLUENE', 'C7H8' ;...
0.48710, 1.460e+000, 'TCE', 'TRICHLOROETHYLENE', 'Cl3C2H' ;...
0.52404, 1.070e+000, 'phosphoric acid', 'TRIETHYL PHOSPHATE', 'C6H15PO4' ;...
0.40371, 2.400e+000, 'Tungsten(VI) fluoride', 'TUNGSTEN HEXAFLUORIDE', 'WF6' ;...
0.39687, 1.128e+001, '', 'URANIUM DICARBIDE', 'UC2' ;...
0.39194, 1.363e+001, 'Uranium carbide', 'URANIUM MONOCARBIDE', 'UC' ;...
0.39996, 1.096e+001, 'Uranium dioxide', 'URANIUM OXIDE', 'UO2' ;...
0.53284, 1.323e+000, 'Diaminomethanal', 'UREA', 'N2H4CO' ;...
0.54632, 1.230e+000, 'Val', 'VALINE', 'C5H11NO2' ;...
0.52772, 1.800e+000, 'Viton', 'VITON FLUOROELASTOMER', 'C5H2F8' ;...
0.55509, 1.000e+000, 'Water', 'WATER, LIQUID', 'H2O' ;...
0.55509, 7.562e-004, 'Steam', 'WATER VAPOR', 'H2O' ;...
0.55509, 0.897e+000, 'Ice'  , 'WATER, ICE', 'H2O' ;...
0.54631, 8.700e-001, '', 'XYLENE', 'C8H10' ;...
0.38930, 2.055     , '', 'Yellowcake', 'U3O8';... % source www.cdc.gov/niosh/ocas/pdfs/abrwh/drreview/scadrr01.pdf
0.55493, 0.93      , 'BPE' ,'Borated Polyethylene, 5%', 'B(5)H(11.6)C(61.2)O(22.2)' ;...
0.55012, 1.03      , 'BPE-10','Borated Polyethylene, 10%', 'B(10)H(11)C(58)O(21)' ;...	
0.46556, 7.86      , 'Steel', 'Steel, ASTM A 366', 'C(8.5E-4)P(1.5E-4)S(1.75E-4)Mn(0.003)Fe(0.996)';...
NaN,     NaN,        '', '', '';...
};

%% Special cases: if Name is empty than just show aviable names
nElem = 100;
if (strcmp(Name, 'Compound Names'))
  P = Props(nElem+1:end, 3:5);
  return;
end
if (strcmp(Name, 'Element Data'))
  P = Props(1:nElem,[3, 1]);
  return;
end
if (strcmp(Name, 'All Data'))
  P = Props(:,[1 2 5 4 3]);
  idx = find(cellfun('isempty',P(:, 3)));
  P(idx, 3) = P(idx, 5);
  return;
end

%% Process Material Input Parameter
if (ischar   (Name)), Name = {Name}; end
if (isnumeric(Name)), Name = num2cell(Name); end

%% Initialize variables
nName = length(Name);
nData = length(Props);
P     = cell(nName,size(Props, 2));

%% Main Loop
for i = 1:nName
  name = Name{i};
  if (ischar(name)) 
    k = find(strcmpi( name, Props(:, 3:4)), 1);% which benchmark?
    if (isempty(k))
      k = find(strcmp( name, Props(:, 5)), 1);% which benchmark?
    end;
    name = nData;
    if (~isempty(k)), name=k; end;
    if (name>nData), name = name-nData; end
  end
  P(i,:) = Props(name,:);
  if (~isempty(P{i, 5})), P(i, 3) = P(i, 5); end
end
P(:, 5) =[];