#include <math.h>
#include "mex.h"
#include <cuda.h>
#include <cufft.h>

// simple math stuff:
// derivative function, direction is 'x' or 'y', non-transposed
__global__ void derivative_nt(float* out, const float* in, int imsize1, int imsize2, char direction) {
    int n = blockIdx.x * blockDim.x + threadIdx.x;
    int m = blockIdx.y * blockDim.y + threadIdx.y;

    if (n<imsize1 && m<imsize2) 
    {
        int index = n + imsize1 * m;

        if (direction == 'x') {    //horizontal derivative
            if (m < imsize2-1) out[index] = in[index+imsize1] - in[index];
            if (m == imsize2-1) out[index] = 0.0;
        }
        if (direction == 'y') {    //vertical derivative
            if (n < imsize1 - 1) out[index] = in[index+1] - in[index];
            if (n == imsize1 - 1) out[index] = 0.0;
        }
    }
}

// derivative function, direction is 'x' or 'y', TRANSPOSED
__global__ void derivative_t(float* out, const float* in, int imsize1, int imsize2, char direction) {
    int n = blockIdx.x * blockDim.x + threadIdx.x;
    int m = blockIdx.y * blockDim.y + threadIdx.y;

    if (n<imsize1 && m<imsize2) {
        int index = n + imsize1 * m;

        if (direction == 'x') {    //horizontal transposed derivative
            if (m > 0 && n < imsize2) out[index] = in[index] - in[index-imsize1];
            if (m == 0) out[index] = 0.0;
        }
        if (direction == 'y') {    //vertical transposed derivative
            if (n > 0 && m < imsize1) out[index] = in[index] - in[index-1];
            if (n == 0) out[index] = 0.0;        
        }
    }
}

// TV norm of the matrix
__global__ void tv_mag(float* out, const float* in, int imsize) {
    int n = blockIdx.x * blockDim.x + threadIdx.x;
    int m = blockIdx.y * blockDim.y + threadIdx.y;
    float dx, dy;

    if (n<imsize && m<imsize) {
        int index = m + imsize * n;

        //horizontal derivative
            if (n < imsize-1) dx = in[index+imsize] - in[index];
            if (n == imsize-1) dx = 0.0;

        //vertical derivative
            if (m < imsize - 1) dy = in[index+1] - in[index];
            if (m == imsize - 1) dy = 0.0;

        out[index] = sqrtf(dx*dx + dy*dy);
        
    }
}

// Calculate magnitude of a and b
__global__ void magnitude(float* out, const float* in_a, const float* in_b, int imsize) {
    int n = blockIdx.x * blockDim.x + threadIdx.x;
    int m = blockIdx.y * blockDim.y + threadIdx.y;

    if (n<imsize && m<imsize) {

        int index = n + imsize * m;
        float a, b;
        if (n < imsize && m < imsize) {
            a = in_a[index]; a *= a;
            b = in_b[index]; b *= b;
            out[index] = sqrtf(a+b);
        }
    }
}

// Calculate magnitude of a, b and c
__global__ void magnitude_abc(float* out, const float* in_a, const float* in_b, const float* in_c, int imsize) {
    int n = blockIdx.x * blockDim.x + threadIdx.x;
    int m = blockIdx.y * blockDim.y + threadIdx.y;

    if (n<imsize && m<imsize) {

        int index = n + imsize * m;
        float a, b, c;
        if (n < imsize && m < imsize) {
            a = in_a[index]; a *= a;
            b = in_b[index]; b *= b;
            c = in_c[index]; c *= c;
            out[index] = sqrtf(a+b+c);
        }
    }
}

// Calculate [a b] = [a b] ./ sqrt(a.^2+b.^2)
__global__ void normalize(float* a, float* b, int imsize, float eta) {
    int n = blockIdx.x * blockDim.x + threadIdx.x;
    int m = blockIdx.y * blockDim.y + threadIdx.y;

    if (n<imsize && m<imsize) {

        int index = n + imsize * m;

        if (n < imsize && m < imsize) {
            float a_ = a[index]; 
            float b_ = b[index]; 

            float nor = sqrtf(a_*a_ + b_*b_);
	    if (nor > eta) 
	    {
		a[index] = a_ / nor;
		b[index] = b_ / nor;
	    } else
	    {
		a[index] = a_ / eta;
		b[index] = b_ / eta;
	    }
        }
    }
}

// Calculate [a b c] = [a b c] ./ sqrt(a.^2+b.^2+c.^2)
__global__ void normalize_abc(float* a, float* b, float* c, int imsize, float eta) {
    int n = blockIdx.x * blockDim.x + threadIdx.x;
    int m = blockIdx.y * blockDim.y + threadIdx.y;

    if (n<imsize && m<imsize) {

        int index = n + imsize * m;

        if (n < imsize && m < imsize) {
            float a_ = a[index]; 
            float b_ = b[index]; 
            float c_ = c[index];

            float nor = sqrtf(a_*a_ + b_*b_ + c_*c_);

	    if (nor > eta) 
	    {
		a[index] = a_ / nor;
		b[index] = b_ / nor;
        c[index] = c_ / nor;
	    } else
	    {
		a[index] = a_ / eta;
		b[index] = b_ / eta;
        c[index] = c_ / eta;
	    }
        }
    }
}

// iRadon transform kernel
__global__ void backproject(const float* ProjData_gpu, float* outdata_gpu, int N, int M, int H, int L, const float* theta) 
{ 
	const float PI = 3.14159265;
	float centx = (float)(N-1)/2;
	float centy = (float)(M-1)/2;
	float centt = (float)(L-1)/2;

    	int idx = blockIdx.x * blockDim.x + threadIdx.x; 
    	int idy = blockIdx.y * blockDim.y + threadIdx.y;

	if ( idx < N && idy < M) {
						
			int index = (M-1-idy) + idx*M;

			float idxx = (idx-centx)/centx;			
			float idyy = (idy-centy)/centy;
			
			outdata_gpu[index] = 0.f;
			float tidx = 0.f;

            for (int anglenr = 0; anglenr < H; anglenr++){

		float ang = theta[anglenr];

                tidx = (idxx*cos(ang) + idyy*sin(ang));
 		
		float projidx = (tidx*centx+centt);

		if (projidx < 0) projidx = 0;
		if (projidx > N - 2) projidx = N -2;

                int projidxL = projidx;
                float weight = projidx - projidxL;
		
                float ApproxProjection = 0;

		ApproxProjection = ((1.f-weight)*ProjData_gpu[projidxL+anglenr*L] + (weight*ProjData_gpu[projidxL+anglenr*L + 1]))/2;

		outdata_gpu[index] = outdata_gpu[index] + ApproxProjection;

            }
			outdata_gpu[index] = outdata_gpu[index]*(PI/(H));
		}
}

// Radon transform kernel
__global__ void projection(const float* theta, const float* xhat_gpu, float* b_gpu, int N, int M, int numAngles, 
        float rFirst, int rSize, float xOrigin, float yOrigin) { 

    // Declare variables
    int index, x1, y1;
    float r, h, x, y, dx, dy, p;
    float delta = 0.9; // Spacing parameter
    
    // Declare parallel computing variables
    int n = blockIdx.x * blockDim.x + threadIdx.x; //n = theta
    int m = blockIdx.y * blockDim.y + threadIdx.y; //m = r1
      
    // Calculate location of points on rays, find value on points using
    // bilinear interpolation and update output with the sum of the points.

    if (n < numAngles && m < rSize) {
        index = m + rSize * n; //ok
        b_gpu[index] = 0.0;
        r = (float) (m + rFirst);

	float ang = theta[n];
	float ss = sin(ang);
	float cc = cos(ang);


        for (h = (float) rFirst; h < (float) (rFirst + rSize - 1); h += delta) {
            x = xOrigin + (r * cc - h * ss);
            y = yOrigin - (r * ss + h * cc);
            if ((x >= 0.0 && x < (float) (N - 1)) && (y >= 0.0 && y < (float) (M - 1))) {
                x1 = (int) (x); dx = x - x1;
                y1 = (int) (y); dy = y - y1;
                p = xhat_gpu[y1 + M * x1] * (1 - dx) * (1 - dy) + 
                    xhat_gpu[y1 + M * (x1 + 1)] * dx * (1 - dy) + 
                    xhat_gpu[(y1 + 1) + M * x1] * (1 - dx) * dy + 
                    xhat_gpu[(y1 + 1) + M * (x1 + 1)] * dx * dy;
                b_gpu[index] += p * delta ;
	    }
        }
    }
}