classdef ASTRA_wrapper
   properties (Access = public)
       
       memory_limit = 1024*1024*512*4;
       % This memory limit depends on the memory available on GPU. 
   end
   
   methods (Access = public)
       function vol = IRADON_3D(this, sino, theta)
           % Compute IRADON of a large volume using ASTRA
           
           sz = size(sino);
           
           % split the volume into chunks:
           s = whos('sino'); 
           unit_size = s.bytes ./ numel(sino);
           
           % total size of the sinogram and the volume:
           total_size = unit_size .* (numel(sino) + sz(1)*sz(1)*sz(3));
           
           % chunk size and number:
           n_chunks = ceil(total_size / this.memory_limit);
           chunk_height = floor(this.memory_limit / total_size * sz(3));
           
           % reconstructed volume:
           vol = zeros(sz(1), sz(1), sz(3), s.class);
           
           for ii = 0:(n_chunks-1)
               from = ii*chunk_height + 1;
               to = min((ii + 1)*chunk_height, sz(3));
               vol(:,:,from:to) = this.astra_iradon_3d(sino(:,:, from:to), theta);
               
           end
           
           % normalize:
           vol = vol ./ (sz(2).^2 * sz(1) * sqrt(2) / 2);
       end
       
       function sino = RADON_3D(this, vol, theta)
           % Compute RADON of a large volume using ASTRA
           sz = size(vol);
           
           % split the volume into chunks:
           s = whos('vol'); 
           unit_size = s.bytes ./ numel(vol);
           
           % total size of the sinogram and the volume:
           total_size = unit_size .* (numel(vol) + sz(1)*sz(3)*length(theta));
           
           % chunk size and number:
           n_chunks = ceil(total_size / this.memory_limit);
           chunk_height = floor(this.memory_limit / total_size * sz(3));
           
           % reconstructed volume:
           sino = zeros(sz(1), length(theta), sz(3), s.class);
           
           for ii = 0:(n_chunks-1)
               from = ii*chunk_height + 1;
               to = min((ii + 1)*chunk_height, sz(3));
               sino(:,:,from:to) = this.astra_radon_3d(vol(:,:,from:to), theta);
               
           end
       end
   end
   
   methods (Static, Access = private)
        function vol = astra_iradon_3d(sino, theta)
        % this function calculates 3D iradon transform of the volume using ASTRA tools

            sz = size(sino);

            % initialize ASTRA projector:
            proj_geom = astra_create_proj_geom('parallel3d', 1, 1, sz(3), sz(1), theta);
            vol_geom = astra_create_vol_geom(sz(1), sz(1), sz(3));
            %proj_id = astra_create_projector('linear3d', proj_geom, vol_geom);

            % copy data to GPU:  
            volume_id = astra_mex_data3d('create','-vol', vol_geom, 0);
            sino_id = astra_mex_data3d('create','-sino', proj_geom, sino);

            % initialize algorithm:
            cfg = astra_struct('BP3D_CUDA');
            %cfg.ProjectorId = proj_id;
            cfg.ProjectionDataId = sino_id;
            cfg.ReconstructionDataId = volume_id;
            %cfg.FilterType = 'none';

            % run:
            alg_id = astra_mex_algorithm('create', cfg);
            astra_mex_algorithm('run', alg_id);
            vol = astra_mex_data3d('get', volume_id) * 2 * size(proj_geom.ProjectionAngles,2) / pi;

            % kill stuff:
            astra_mex_data3d('delete', sino_id);
            astra_mex_data3d('delete', volume_id);
            astra_mex_algorithm('delete', alg_id);
            %astra_mex_projector('delete', proj_id);
        end
        
        function sino = astra_radon_3d(volume, theta)
        % this function calculates 3D radon transform of the volume using ASTRA tools

            sz = size(volume);

            % initialize ASTRA geometries:
            proj_geom = astra_create_proj_geom('parallel3d', 1, 1, sz(3), sz(1), theta);
            vol_geom = astra_create_vol_geom(sz(1), sz(2), sz(3));
            %proj_id = astra_create_projector('linear3d', proj_geom, vol_geom);
            %proj_id = astra_create_projector('cuda3d', proj_geom, vol_geom);

            % copy data to GPU:  
            volume_id = astra_mex_data3d('create','-vol', vol_geom, volume);
            sino_id = astra_mex_data3d('create','-sino', proj_geom, 0);

            % initialize algorithm:
            cfg = astra_struct('FP3D_CUDA');
            %cfg = astra_struct('FP_CUDA');
            %cfg.ProjectorId = proj_id;
            %cfg.ProjectionGeometry = proj_geom;
            %cfg.VolumeGeometry = vol_geom;	
            cfg.ProjectionDataId = sino_id;
            cfg.VolumeDataId = volume_id;

            % run algorithm:
            alg_id = astra_mex_algorithm('create', cfg);
            astra_mex_algorithm('iterate', alg_id);

            % get data:
            sino = astra_mex_data3d('get',sino_id);

            % kill stuff:
            astra_mex_data3d('delete', sino_id);
            astra_mex_data3d('delete', volume_id);
            astra_mex_algorithm('delete', alg_id);
           % astra_mex_projector('delete', proj_id);
        end
   end
end