classdef CPU_mat
    %CPU_MAT Math routines realized on CPU.
    %
    % Math routines used in TV-minimization. Class is fully static
    % and does not have to be initialized.
    % 
    % Example 1 (static use): 
    %
    % image_norm = CPU_mat.norm(image);
    %
    % Example 2 (instantiated use):   
    %
    % m = CPU_mat;
    % image_norm = m.norm(image);
    %
    % See also: GPU_mat
    
    methods(Static)
        
         function x = projection(x, low_bound, high_bound)
            % PROJECTION Projection of x within the bounds

            if isfinite(high_bound) || isfinite(low_bound)
                x = (((low_bound<x) .* (x<high_bound)) .* x);
            end

            if isfinite(high_bound)
                x = x + ((x>=high_bound) * high_bound);
            end

            if isfinite(low_bound)
                x = x + (low_bound * (x<=low_bound));
            end

        end

        function n = norm(x)
            % Calculates the L2 norm of x
            n = sqrt(sum( abs(x(:)).^2 ));
        end

        function [x, y] = normalize(x, y, epsilon)
            % Calculate x = x ./ sqrt(x^2 + y^2); y = y ./ sqrt(x^2 + y^2);

            nor = sqrt(x.^2 + y.^2);
            nor(nor < epsilon) = epsilon;

            x = x ./ nor;
            y = y ./ nor;
        end

        function x = dx(x)
            % Horizontal derivative of x

            x(1:end - 1, :) = x(1:end - 1, :) - x(2:end, :);
            x(end, :) = 0;
        end

        function x = dy(x)
            % Vertical derivative of x

            x(:, 1:end - 1) = x(:, 1:end - 1) - x(:, 2:end);
            x(:, end) = 0;
        end

        function x = dx_(x)
            % Horizontal(transposed) derivative of x

            x(2:end, :) = x(1:end - 1, :) - x(2:end, :);
            x(1, :) = 0;
        end

        function x = dy_(x)
            % Vertical(transposed) derivative of x

            x(:, 2:end) = x(:, 1:end - 1) - x(:, 2:end);
            x(:, 1) = 0;
        end

        function tv = TV(x)
            % Total Variation norm
            tv = sum(sum(sqrt( CPU_mat.dx(x).^2 + CPU_mat.dy(x).^2 )));
        end
        
        function sino = RADON(vol, theta)
            sino = radon(vol, theta);
            
            % even knippen!
            sz = size(vol, 1);
            sino = sino((end-sz)/2+1:(end+sz)/2,:); 
        end
        
        function vol = IRADON(sino, theta)
            vol = iradon(sino, theta, 'linear', 'none', 1, size(sino, 1));
        end
            
        function vol = FBP(sino, theta)
            vol = iradon(sino, theta, 'linear', 'Ram-Lak', 1, size(sino, 1));
        end
    end
end