classdef CPU_mat3d < CPU_mat
    %CPU_MAT Math routines realized on CPU.
    %
    % Math routines used in TV-minimization. Class is fully static
    % and does not have to be initialized.
    % 
    % Example 1 (static use): 
    %
    % image_norm = CPU_mat.norm(image);
    %
    % Example 2 (instantiated use):   
    %
    % m = CPU_mat;
    % image_norm = m.norm(image);
    %
    % See also: GPU_mat
    
    methods(Static)
        
         function x = projection(x, low_bound, high_bound)
            % PROJECTION Projection of x within the bounds

            if isfinite(high_bound) || isfinite(low_bound)
                x = (((low_bound<x) .* (x<high_bound)) .* x);
            end

            if isfinite(high_bound)
                x = x + ((x>=high_bound) * high_bound);
            end

            if isfinite(low_bound)
                x = x + (low_bound * (x<=low_bound));
            end

        end

        function n = norm(x)
            % Calculates the L2 norm of x
            n = sqrt(sum( abs(x(:)).^2 ));
        end

        function [x, y, z] = normalize(x, y, z, epsilon)
            % Calculate x = x ./ sqrt(x^2 + y^2); y = y ./ sqrt(x^2 + y^2);

            nor = sqrt(x.^2 + y.^2 + z.^2);
            nor(nor < epsilon) = epsilon;

            x = x ./ nor;
            y = y ./ nor;
            z = z ./ nor;
        end

        function x = dx(x)
            % Horizontal derivative of x

            x(1:end - 1, :, :) = x(1:end - 1, :, :) - x(2:end, :, :);
            %x(end, :, :) = 0;
            x(end, :, :)  = -x(end-1, :, :);
        end

        function x = dy(x)
            % Vertical derivative of x

            x(:, 1:end - 1, :) = x(:, 1:end - 1, :) - x(:, 2:end, :);
            %x(:, end, :) = 0;
            x(:, end, :) = -x(:, end-1, :);
        end
        
        function x = dz(x)
            % Vertical derivative of z

            x(:, :, 1:end - 1) = x(:, :, 1:end - 1) - x(:, :, 2:end);
            %x(:, :, end) = 0;
            x(:, :, end) = -x(:, :, end-1);
        end

        function x = dx_(x)
            % Horizontal(transposed) derivative of x

            x(2:end, :, :) = x(1:end - 1, :, :) - x(2:end, :, :);
            %x(1, :, :) = 0;
            x(1, :, :) = -x(2, :, :);
        end

        function x = dy_(x)
            % Vertical(transposed) derivative of x

            x(:, 2:end, :) = x(:, 1:end - 1, :) - x(:, 2:end, :);
            %x(:, 1, :) = 0;
            x(:, 1, :) = -x(:, 2, :);
        end

        function x = dz_(x)
            % Vertical(transposed) derivative of z

            x(:, :, 2:end) = x(:, :, 1:end - 1) - x(:, :, 2:end);
            %x(:, :, 1) = 0;
            x(:, :, 1) = -x(:, :, 2);
        end
        
        function tv = TV(x)
            % Total Variation norm
            tv = sum(sum(sum(sqrt( CPU_mat3d.dx(x).^2 + CPU_mat3d.dy(x).^2 + CPU_mat3d.dz(x).^2))));
        end
        
        function sino = RADON(vol, theta)
            % quick wrapper for radon (use lower case radon for speed)
            sino = zeros([size(vol, 1), size(vol, 3), length(theta)]);
            
            for ii = 1:size(vol, 3)
                slice = RADON@CPU_mat(squeeze(vol(:, :, ii)), theta);
                sino(:, ii, :) = slice;
            end
        end
            
        function vol = IRADON(sino, theta)
            % quick wrappers for iradon (use lower case radon for speed)           
            vol = zeros([size(sino, 1), size(sino, 1), size(sino, 2)]);
            
            parfor ii = 1:size(sino, 2)
                slice = IRADON@CPU_mat(squeeze(sino(:, ii, :)), theta);
                vol(:, :, ii) = slice;
            end
        end
        
        function vol = FBP(sino, theta)
            % IRADON with a simple ramp filter. Works only for even sizes
            % of the sinogram!
            
            vol = zeros([size(sino, 1), size(sino, 1), size(sino, 2)]);
            
            parfor ii = 1:size(sino, 2)
                slice = FBP@CPU_mat(squeeze(sino(:, ii, :)), theta);
                vol(:, :, ii) = slice;
            end
        end
    end
end
