classdef GPU_mat < CPU_mat
    %GPU_MAT Math routines realized on GPU.
    %
    % Math routines used in TV-minimization. Class is not static!
    % therefore it has to be initialized before use.
    %
    % GPU_MAT uses CUDA kernels from gpu_cernels.cu and gpu_kernels.ptx. 
    % The latter has to be recompiled on each machine using static
    % method <a href="matlab:help GPU_mat.recompile_kernels">GPU_mat.recompile_kernels</a> or running script <a href="matlab:help dependencies.m">dependencies.m</a>
    %
    % Example:   
    %
    % m = GPU_mat;
    % image_norm = m.norm(image);
    %
    % See also: CPU_mat, dependencies
    
    properties (Access = protected)
        nt_diff_kernel = [];
        t_diff_kernel = [];
        magnitude_kernel = [];
        normalize_kernel = [];
        tv_kernel = [];
        
        % radon kernels:
        iradon_kernel = [];
        radon_kernel = [];
    end
    
    methods(Static)
        function image = gpu_array(sz, val)
            %GPU_ARRAY Initiates an array of type 'single' in GPU memory.
            % sz    - size of the array
            % val   - initial value
            
            if nargin < 2 
                image = parallel.gpu.GPUArray.zeros(sz(1), sz(2), 'single');
            else
                image = val .* parallel.gpu.GPUArray.ones(sz(1), sz(2), 'single');
            end
        end
        
        function recompile_kernels(dir)
            % Recompiles CUDA code on current machine.
            % The 'dir' must be the root dir of the toolset. 
            % 
            % If everything went well it should produce file 'gpu_kernels.ptx'
            % in 'gpu_kernels/' directory.
            
            try            
            % produce the correct path (platform dependent):
            to = fullfile(dir, 'gpu_kernels', 'gpu_kernels.ptx');
            from = fullfile(dir, 'gpu_kernels', 'gpu_kernels.cu');
            include = fullfile(matlabroot, 'extern', 'include');
            
            % to avoid problems with spaces (at least in Windows):
            to = ['"', to, '"'];
            from = ['"', from, '"'];
            include = ['"', include, '"'];
            
            % Compile:
            comp = computer('arch');
            switch comp
                case 'win64'
                    [a, b] = system(['"%VS90COMNTOOLS%vsvars32.bat" & nvcc -m 64 -ptx ', from, ...
                        ' -o ', to, ' -I ', include]);
                case 'win32'
                    [a, b] = system(['"%VS90COMNTOOLS%vsvars32.bat" & nvcc -ptx ', from, ...
                        ' -o ', to, ' -I ', include]);
                otherwise
                    [a, b] = system(['nvcc -ptx ', from, ' -o ', to, ' -arch=sm_20 -Xcompiler -fPIC', ' -I ', include]); 
            end
            
            catch exception
                disp('wow-wow-wow! We couldn`t recompile CUDA kernels! You GPU code might not work :(');
                
                rethrow(exception);
            end
            
            if a ~= 0
                disp(b);
                disp('Couldn`t compile CUDA kernels!');    
                
                if ~isempty(strfind(b, 'VS90COMNTOOLS'))
                    disp('Probably couldnt find MS Visual Studio 9.0 on your machine...');
                end
            else
                disp('Yeah baby! CUDA Kernels were recompiled for your achitecture. Cross your fingers, GPU code might work...');
            end
        end
    end
    
    methods (Access = public)
        
        function this = GPU_mat()
            % Initialize CUDA kernels.
            % Uses gpu_kernels.ptx and gpu_kernels.cu.
            
            % image processing:
            this.nt_diff_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'derivative_nt');
            this.t_diff_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'derivative_t');
            this.magnitude_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'magnitude');
            this.normalize_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'normalize');
            this.tv_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'tv_mag');
            
            % radon:
            this.iradon_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'backproject');
            this.radon_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'projection');

        end
        
        function [imag_diff] = dx(this, image) 
            % Horizontal derivative of x
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.nt_diff_kernel.ThreadBlockSize = [8 32];
            this.nt_diff_kernel.GridSize = ceil([imsz(1) / 8, imsz(2) / 32]);
            imag_diff = feval(this.nt_diff_kernel, imag_diff, image, int32(imsz(1)), int32(imsz(2)), int8('x'));
            
        end
        
        function [imag_diff] = dy(this, image) 
            % Vertical derivative of x
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.nt_diff_kernel.ThreadBlockSize = [8 32];
            this.nt_diff_kernel.GridSize = ceil([imsz(1) / 8, imsz(2) / 32]);
            imag_diff = feval(this.nt_diff_kernel, imag_diff, image, int32(imsz(1)), int32(imsz(2)), int8('y'));
            
        end
        
        function [imag_diff] = dx_(this, image) 
            % Horizontal(transposed) derivative of x
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.t_diff_kernel.ThreadBlockSize = [8 32];
            this.t_diff_kernel.GridSize = ceil([imsz(1) / 8, imsz(2) / 32]);
            imag_diff = feval(this.t_diff_kernel, imag_diff, image, int32(imsz(1)), int32(imsz(2)), int8('x'));
            
        end
        
        function [imag_diff] = dy_(this, image)   
            % Vertical(transposed) derivative of x
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.t_diff_kernel.ThreadBlockSize = [8 32];
            this.t_diff_kernel.GridSize = ceil([imsz(1) / 8, imsz(2) / 32]);
            imag_diff = feval(this.t_diff_kernel, imag_diff, image, int32(imsz(1)), int32(imsz(2)), int8('y'));
            
        end
        
        function [ab_mag] = magnitude(this, a, b)  
            % sqrt(a.^2 + b.^2)
            imsz = size(a, 1);
            ab_mag = this.gpu_array([imsz, imsz]);
            
            this.magnitude_kernel.ThreadBlockSize = [8 32];
            this.magnitude_kernel.GridSize = ceil([imsz / 8, imsz / 32]);
            ab_mag = feval(this.magnitude_kernel, ab_mag, a, b, int32(imsz));
            
        end
        
        function [a, b] = normalize(this, a, b, eta) 
            % Calculate a = a ./ sqrt(a^2 + b^2); b = b ./ sqrt(a^2 + b^2);
            imsz = size(a, 1);
            
            this.normalize_kernel.ThreadBlockSize = [8 32];
            this.normalize_kernel.GridSize = ceil([imsz / 8, imsz / 32]);
            [a, b] = feval(this.normalize_kernel, a, b, int32(imsz), eta);
            
        end
        
        function deriv = TV_derivative(this, vol, eta)
            % Total Variation derivative
            
            vol_dx = this.dx(vol);
            vol_dy = this.dy(vol);
            
            [vol_dx, vol_dy] = this.normalize(vol_dx, vol_dy, eta); 

            vol_dx = this.dx_(vol_dx);
            vol_dy = this.dy_(vol_dy);

            deriv = vol_dx + vol_dy;
        end
        
        function tv = TV(this, vol)
            % Total Variation norm
            % Returns a number to CPU memory.
            
            imsz = size(vol, 1);
            tv = this.gpu_array([imsz, imsz]);
            
            this.tv_kernel.ThreadBlockSize = [8 32];
            this.tv_kernel.GridSize = ceil([imsz / 8, imsz / 32]);
            tv = gather(sum(sum(feval(this.tv_kernel, tv, vol, int32(imsz)))));
            
            %tv = sum(sum(this.magnitude(this.dx(vol), this.dy(vol))));
        end
        
        function d = norm(this, vol)
            % Calculate L2 norm of the matrix 
            % Returns a number to CPU memory.
            
            %d = sqrt(sum(gather(sum(abs(vol).^2))));
            
            % in matlab 8.0 GPU version works faster!
            d = gather(sqrt(sum(sum(abs(vol).^2))));
        end
        
        function x = max(this, vol)
            % overloaded Matlab max(vol) function for gpuArray objects
            %x = max(max(gather(vol)));
            
            % in matlab 8.0 GPU version works faster!
            x = gather(max(max(vol)));
        end
        
        function sino = RADON(this, vol, theta)
            % quick wrapper for radon (use lower case radon for speed)
            sino = this.gpu_array([size(vol, 1) length(theta)]);
            sino = this.radon(vol, sino, theta);
        end
            
        function vol = IRADON(this, sino, theta)
            % quick wrappers for iradon (use lower case radon for speed)
            vol = this.gpu_array([size(sino, 1) size(sino, 1)]);
            vol = this.iradon(sino, vol, theta);
        end
        
        function vol = FBP(this, sino, theta)
            % IRADON with a simple ramp filter. Works only for even sizes
            % of the sinogram!
            
            sz = size(sino,1);
            
            sino(sz*2, :) = 0;
            sino = circshift(sino, [sz/2, 0]);
            
            %filter:
            filt = gpuArray(1/sz *fftshift(abs((yyy(sino, [sz+1, 0])))));
           
            % apply filter:
            sino = real(ifft(fft(sino) .* filt));
            
            % reconstruct:
            vol = this.IRADON(sino, theta);
            vol = vol(floor((end-sz)/2):ceil((end+sz)/2-1), floor((end-sz)/2):ceil((end+sz)/2-1));
        end
        
        function vOL = iradon(this, sIno, vOL, theta)     
            % Calculates inverse Radon transform using GPU computing

            imsize = size(vOL, 1);
            tsize = length(theta);
            sinsize = size(sIno, 1);
            
            % kernel:
            this.iradon_kernel.ThreadBlockSize = [8 32];
            this.iradon_kernel.GridSize = ceil([imsize / 8, imsize / 32]);

            vOL = feval(this.iradon_kernel, sIno, vOL, int32(imsize), int32(imsize), int32(tsize), int32(sinsize), theta / 180 * pi);
        end

        function sIno = radon(this, vOL, sIno, theta)
            % Calculates forward Radon transform using GPU computing
            
            imsize = size(vOL, 1);
            tsize = length(theta);
            sinsize = size(sIno, 1);

            Origin = single(max(0, ((imsize-1)/2)));            
            rFirst = -single(imsize - 1) / 2;
            
            % kernel:
            this.radon_kernel.ThreadBlockSize = [8 32];
            this.radon_kernel.GridSize = ceil([tsize / 8, imsize / 32]);
                
            sIno = feval(this.radon_kernel, single(theta / 180 * pi), vOL, sIno, int32(imsize), int32(imsize), int32(tsize), rFirst, int32(sinsize), Origin, Origin);            
        end
    end

end
