classdef GPU_mat3d < GPU_mat
    %GPU_MAT Math routines realized on GPU.
    %
    % Math routines used in TV-minimization. Class is not static!
    % therefore it has to be initialized before use.
    %
    % GPU_MAT uses CUDA kernels from gpu_cernels.cu and gpu_kernels.ptx. 
    % The latter has to be recompiled on each machine using static
    % method <a href="matlab:help GPU_mat.recompile_kernels">GPU_mat.recompile_kernels</a> or running script <a href="matlab:help dependencies.m">dependencies.m</a>
    %
    % Example:   
    %
    % m = GPU_mat;
    % image_norm = m.norm(image);
    %
    % See also: CPU_mat, dependencies
    
    methods(Static)
        function image = gpu_array(sz, val)
            %GPU_ARRAY Initiates an array of type 'single' in GPU memory.
            % sz    - size of the array
            % val   - initial value
            
            if nargin < 2 
                image = gpuArray.zeros(sz, 'single');
            else
                image = val .* gpuArray.ones(sz, 'single');
            end
        end
    end
          
    methods(Access = public)
        
        function this = GPU_mat3d()
            % Initialize CUDA kernels.
            % Uses gpu_kernels.ptx and gpu_kernels.cu.
            
            % image processing:
            this.nt_diff_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'derivative_nt');
            this.t_diff_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'derivative_t');
            this.magnitude_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'magnitude_abc');
            this.normalize_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'normalize_abc');
            this.tv_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'tv_mag');
            
            % radon:
            this.iradon_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'backproject');
            this.radon_kernel = parallel.gpu.CUDAKernel('gpu_kernels.ptx', 'gpu_kernels.cu', 'projection');

        end
                
        function [imag_diff] = dx(this, image) 
            % Horizontal derivative of image
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.nt_diff_kernel.ThreadBlockSize = [8 32];
            this.nt_diff_kernel.GridSize = ceil([imsz(1) / 8, imsz(2) / 32]);
                        
            for ii = 1:imsz(3)
                x = this.gpu_array(imsz(1:2));
                x = feval(this.nt_diff_kernel, x, image(:,:,ii), int32(imsz(1)), int32(imsz(2)), int8('x'));
                
                imag_diff(:,:,ii) = x;
            end
            
        end
        
        function [imag_diff] = dy(this, image) 
            % Vertical derivative of image
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.nt_diff_kernel.ThreadBlockSize = [8 32];
            this.nt_diff_kernel.GridSize = ceil([imsz(1) / 8, imsz(2) / 32]);
            
            for ii = 1:imsz(3)
                x = this.gpu_array(imsz(1:2));
                x = feval(this.nt_diff_kernel, x, image(:,:,ii), int32(imsz(1)), int32(imsz(2)), int8('y'));
                
                imag_diff(:,:,ii) = x;
            end
            
        end
        
        function [imag_diff] = dz(this, image) 
            % dZ derivative of image
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.nt_diff_kernel.ThreadBlockSize = [32 8];
            this.nt_diff_kernel.GridSize = ceil([imsz(2) / 32, imsz(3) / 8]);
            
            for ii = 1:imsz(1)
                x = this.gpu_array(imsz(2:3));
                x = feval(this.nt_diff_kernel, x, image(ii,:,:), int32(imsz(2)), int32(imsz(3)), int8('x'));
                
                imag_diff(ii,:,:) = x;
            end
        end
        
        function [imag_diff] = dx_(this, image) 
            % Horizontal(transposed) derivative of image
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.t_diff_kernel.ThreadBlockSize = [8 32];
            this.t_diff_kernel.GridSize = ceil([imsz(1) / 8, imsz(2) / 32]);
            
            for ii = 1:imsz(3)
                x = this.gpu_array(imsz(1:2));
                x = feval(this.t_diff_kernel, x, image(:,:,ii), int32(imsz(1)), int32(imsz(2)), int8('x'));
                
                imag_diff(:,:,ii) = x;
            end
            
        end
        
        function [imag_diff] = dy_(this, image)   
            % Vertical(transposed) derivative of image
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.t_diff_kernel.ThreadBlockSize = [8 32];
            this.t_diff_kernel.GridSize = ceil([imsz(1) / 8, imsz(2) / 32]);
            
            for ii = 1:imsz(3)
                x = this.gpu_array(imsz(1:2));
                x = feval(this.t_diff_kernel, x, image(:,:,ii), int32(imsz(1)), int32(imsz(2)), int8('y'));
                
                imag_diff(:,:,ii) = x;
            end
            
        end
        
        function [imag_diff] = dz_(this, image) 
            % dZ (transposed) derivative of image
            imsz = size(image);
            imag_diff = this.gpu_array(imsz);
            
            this.t_diff_kernel.ThreadBlockSize = [32 8];
            this.t_diff_kernel.GridSize = ceil([imsz(2) / 32, imsz(3) / 8]);
            
            for ii = 1:imsz(1)
                x = this.gpu_array(imsz(2:3));
                x = feval(this.t_diff_kernel, x, image(ii,:,:), int32(imsz(2)), int32(imsz(3)), int8('x'));
                
                imag_diff(ii,:,:) = x;
            end
        end
        
        function [ab_mag] = magnitude(this, a, b, c)  
            % sqrt(a.^2 + b.^2)
            imsz = size(a, 1);
            ab_mag = this.gpu_array(size(a));
            
            this.magnitude_kernel.ThreadBlockSize = [8 32];
            this.magnitude_kernel.GridSize = ceil([imsz / 8, imsz / 32]);
            
            for ii = 1:size(a, 3)
                x = feval(this.magnitude_kernel, ab_mag, a(:,:,ii), b(:,:,ii), c(:,:,ii), int32(imsz));
                
                ab_mag(:,:,ii) = x;
            end
            
        end
        
        function [a, b, c] = normalize(this, a, b, c, eta) 
            % Calculate a = a ./ sqrt(a^2 + b^2); b = b ./ sqrt(a^2 + b^2);
            imsz = size(a, 1);
            
            this.normalize_kernel.ThreadBlockSize = [8 32];
            this.normalize_kernel.GridSize = ceil([imsz / 8, imsz / 32]);
            
            for ii = 1:size(a, 3)
                a_ = a(:,:,ii);
                b_ = b(:,:,ii);
                c_ = c(:,:,ii);
                
                [a_, b_, c_] = feval(this.normalize_kernel, a_, b_, c_, int32(imsz), eta);
                
                a(:,:,ii) = a_;
                b(:,:,ii) = b_;
                c(:,:,ii) = c_;
            end
            
        end
        
        function deriv = TV_derivative(this, vol, eta)
            % Total Variation derivative
            
            vol_dx = this.dx(vol);
            vol_dy = this.dy(vol);
            
            [vol_dx, vol_dy] = this.normalize(vol_dx, vol_dy, eta); 

            vol_dx = this.dx_(vol_dx);
            vol_dy = this.dy_(vol_dy);

            deriv = vol_dx + vol_dy;
        end
        
        function tv = TV(this, vol)
            % Total Variation norm
            % Returns a number to CPU memory.
            
            imsz = size(vol, 1);
            
            this.tv_kernel.ThreadBlockSize = [8 32];
            this.tv_kernel.GridSize = ceil([imsz / 8, imsz / 32]);
            
            tv = 0;
            
            for ii =1:size(vol, 3)
                tv = tv + gather(sum(sum(feval(this.tv_kernel, tv*0, vol(:,:,ii), int32(imsz)))));
            end
            
            %tv = sum(sum(this.magnitude(this.dx(vol), this.dy(vol))));
        end
        
        function d = norm(this, vol)
            % Calculate L2 norm of the matrix 
            % Returns a number to CPU memory.
            
            %d = sqrt(sum(gather(sum(abs(vol).^2))));
            
            % in matlab 8.0 GPU version works faster!
            d = gather(sqrt(sum(sum(sum(abs(vol).^2)))));
        end
        
        function x = max(this, vol)
            % overloaded Matlab max(vol) function for gpuArray objects
            %x = max(max(gather(vol)));
            
            % in matlab 8.0 GPU version works faster!
            x = gather(max(max(max(vol))));
        end
        
        function sino = RADON(this, vol, theta)
            % quick wrapper for radon (use lower case radon for speed)
            sino = this.gpu_array([size(vol, 1), size(vol, 3), length(theta)]);
            
            for ii = 1:size(vol, 3)
                slice = this.RADON@GPU_mat(vol(:, :, ii), theta);
                sino(:, ii, :) = slice;
            end
        end
            
        function vol = IRADON(this, sino, theta)
            % quick wrappers for iradon (use lower case radon for speed)           
            vol = this.gpu_array([size(sino, 1), size(sino, 1), size(sino, 2)]);
            
            for ii = 1:size(sino, 2)
                slice = this.IRADON@GPU_mat(sino(:, ii, :), theta);
                vol(:, :, ii) = slice;
            end
        end
        
        function vol = FBP(this, sino, theta)
            % IRADON with a simple ramp filter. Works only for even sizes
            % of the sinogram!
            
            vol = this.gpu_array([size(sino, 1), size(sino, 1), size(sino, 2)]);
            
            for ii = 1:size(sino, 2)
                slice = this.FBP@GPU_mat(sino(:, ii, :), theta);
                vol(:, :, ii) = slice;
            end
        end
        
        function vOL = iradon(this, sIno, vOL, theta)     
            % Calculates inverse Radon transform using GPU computing

            imsize = size(vOL, 1);
            tsize = length(theta);
            sinsize = size(sIno, 1);
            
            % kernel:
            this.iradon_kernel.ThreadBlockSize = [8 32];
            this.iradon_kernel.GridSize = ceil([imsize / 8, imsize / 32]);

            vOL = feval(this.iradon_kernel, sIno, vOL, int32(imsize), int32(imsize), int32(tsize), int32(sinsize), theta / 180 * pi);
        end

        function sIno = radon(this, vOL, sIno, theta)
            % Calculates forward Radon transform using GPU computing
            
            imsize = size(vOL, 1);
            tsize = length(theta);
            sinsize = size(sIno, 1);

            Origin = single(max(0, ((imsize-1)/2)));            
            rFirst = -single(imsize - 1) / 2;
            
            % kernel:
            this.radon_kernel.ThreadBlockSize = [8 32];
            this.radon_kernel.GridSize = ceil([tsize / 8, imsize / 32]);
                
            sIno = feval(this.radon_kernel, single(theta / 180 * pi), vOL, sIno, int32(imsize), int32(imsize), int32(tsize), rFirst, int32(sinsize), Origin, Origin);            
        end
    end

end
