classdef PCI_rec
    %Phase-Contrast Imaging routines.
    %
    % This class is designed for forward modelling and non-iterative image
    % reconstruction in Phase-Contrast Imaging. Data can be simulated using
    % Fresnel propagator or the various propagators based on linearized
    % models. Point Spread Function can be included in the forward model
    % in order account for the finite source size and limited detector
    % resoultuion.
        
    methods(Static)
        
        function w = frequency_space(params, pixelsize, energy, r1, r2)
            % Generates the lambda*w^2*R image that can be used to calculate phase propagator at distance R, wavelength lambda.
            % PCI_rec.frequency_space(params, pixelsize, energy, r1, r2)
            
            % Image dimentions:
            dims = size_gen(params);
            pix_sz = repmat(pixelsize, size(dims));
    
            % Frequency squared * lambda;
            w = geometry.w2_r(pix_sz, dims, r1, r2) / (2*unit_convert.e2k(energy));
                        
        end
        
        function p = fresnel_propagator(params, pixelsize, energy, r1, r2)
            % Fresnel propagator in frequency space.
            % PCI_rec.fresnel_propagator(params, pixelsize, energy, r1, r2)
                       
            p = exp(1i * PCI_rec.frequency_space(params, pixelsize, energy, r1, r2));   
        end
                
        function p = dual_tie_propagator(params, pixelsize, energy, r1, r2, alpha)
            % Dual TIE propagator in frequency space
            % PCI_rec.dual_tie_propagator(params, pixelsize, energy, r1, r2, alpha)
            
            p = 1 + (alpha) * PCI_rec.frequency_space(params, pixelsize, energy, r1, r2);
        end
        
        function [p, w] = dual_ctf_propagator(params, pixelsize, energy, r1, r2, alpha)
            % propagator in frequency space
            % PCI_rec.dual_ctf_propagator(params, pixelsize, energy, r1, r2, alpha)
            
            w = PCI_rec.frequency_space(params, pixelsize, energy, r1, r2);  
            p = -2*cos(w) + 2*(alpha) * sin(w);
        end
        
        function [p, w] = dual_mixed_propagator(params, pixelsize, energy, r1, r2, alpha)
            % propagator in frequency space
            % PCI_rec.dual_ctf_propagator(params, pixelsize, energy, r1, r2, alpha)
            
            w = PCI_rec.frequency_space(params, pixelsize, energy, r1, r2);  
            p = cos(w) - (alpha) * sin(w);
        end
        
        function [p, w] = dual_mixed_propagator_ext(params, pixelsize, energy, r1, r2, alpha)
            % propagator in frequency space
            % PCI_rec.dual_ctf_propagator(params, pixelsize, energy, r1, r2, alpha)
            
            w = PCI_rec.frequency_space(params, pixelsize, energy, r1, r2);  
            p = cos(w) + (w - alpha) .* sin(w);
        end
        
        function intensity_image = forward_propagation(field_image, pixelsize, energy, r1, r2)
            % Calculate propagated intensity image
            % PCI_rec.forward_propagation(field_image, pixelsize, energy, r1, r2)
            
            p = PCI_rec.fresnel_propagator(field_image, pixelsize, energy, r1, r2);
            
            intensity_image = abs(ifft2((fft2(field_image) .* p))).^2;
        end
        
        function intensity_image = sinogram_propagation(field_image, pixelsize, energy, r1, r2)
            % Calculate propagated sinogram (see forward_propagation).
            % PCI_rec.sinogram_propagation(field_image, pixelsize, energy, r1, r2)
            
            p = PCI_rec.fresnel_propagator([size(field_image, 1), 1], pixelsize, energy, r1, r2);
            p = repmat(p, [1, size(field_image, 2)]);
            
            intensity_image = abs(ifft((fft(field_image) .* p))).^2;
        end
        
        function image = add_ctf_noise(image, ctf, param, noise_type) 
            % Apply convolution with CTF and add noise. 
            % PCI_rec.add_ctf_noise(image, ctf, param, noise_type)
            %
            % Check help on imnoise for noise types.
            
            % Convolution:
            if ~isempty(ctf)
                image = real(ifft2(fft2(image) .* ctf));
            end
            
            % Noise:
            if (nargin > 2) && (param > 0)
                if nargin == 3 || strcmp(noise_type, 'gaussian')
                    % Gaussian (we don't like imnoise implementation):
                    image = image + param * (randn(size(image)));
                    
                else
                    % Poisson:
                    image = 1e12/param * imnoise(image * param/1e12, noise_type);
                    
                end
            end
        end
        
        function phi = back_propagation_tie(i0, ir, pixelsize, energy, r1, r2, epsilon)
            % Solves the inverse propagation problem using TIE model.
            % PCI_rec.back_propagation_tie(i0, ir, pixelsize, energy, r1, r2, epsilon)
            %
            % Input images are intensity image at zero distance i0 and the
            % phase-contrast image at distance r2

            % Generating stuff needed to calculate inverse:
            w = PCI_rec.frequency_space(ir, pixelsize, energy, r1, r2);
                
            % Inverse:
            phi = -(ir ./ i0 - 1) .* w.^2 ./ (w + epsilon);
        end
        
        function [i0, phi] = back_propagation_ctf(ir, pixelsize, energy, r1, r2, epsilon)
            % Solves the inverse propagation problem using least-squares solution of CTF approximation.
            % PCI_rec.back_propagation_ctf(ir, pixelsize, energy, r1, r2, epsilon)
            %
            % At least two input images must be provided in cell array ir.
            % All parameters must be in a form of cell array with
            % number of elements equal to the number of aquired images. 
            
            sz = size(ir{1});
            
            C = zeros(sz);
            CC = zeros(sz);
            A = zeros(sz);
            AA = zeros(sz);
            B = zeros(sz);
            
            % Generating stuff needed to calculate inverse:
            for ii = 1:length(ir)
                w = PCI_rec.frequency_space(ir{ii}, pixelsize{ii}, energy{ii}, r1{ii}, r2{ii});
                
                s = sin(w);
                c = cos(w);
                
                A = A + s .* c;
                B = B + s.^2;
                C = C + c.^2;

                ftir = fft2(ir{ii});
                
                AA = AA + ftir .* c;
                CC = CC + ftir .* s;
            end

            % Inverse:
            phi = (C .* CC - A .* AA) ./ (2 * (B.*C - A.^2) + epsilon(1));
            i0 =  (A .* CC - B .* AA) ./ (2 * (B.*C - A.^2) + epsilon(2));
            
            phi = real(ifft2(phi));
            i0 = real(ifft2(i0));
        end
        
        function phi = back_propagation_dual_ctf(ir, i0, pixelsize, energy, r1, r2, epsilon)
            % Solves the inverse propagation problem using least-squares solution of Mixed TIE-CTF approximation (non-iterative version).
            % PCI_rec.back_propagation_dual_ctf(ir, i0, pixelsize, energy, r1, r2, epsilon)
            %
            % At least one input image must be provided in the cell array IR.
            % i0 is an image measured at zero distance.
            % All parameters except i0 must be in a form of cell array with
            % number of elements equal to the number of aquired images. 
            
            sz = size(ir{1});
            
            C = zeros(sz);
            CC = zeros(sz);
            
            % Generating stuff needed to calculate inverse:
            for ii = 1:length(ir)
                w = PCI_rec.frequency_space(ir{ii}, pixelsize{ii}, energy{ii}, r1{ii}, r2{ii});
                
                C = C + (2*sin(w)).^2;   
                CC = CC + fft2(ir{ii} - i0) .* (2 * sin(w));
            end

            % Inverse:
            phi = CC ./ (C + epsilon);
            phi =  real(ifft2(phi));
        end
    end
end