classdef TV_min
    % Class contains functions needed for constrained L1 minimization using FISTA. 
    % Check the original implementation at: 
    % http://www.math.tau.ac.il/~teboulle/papers/tlv.pdf
    %
    % The code uses X, Y and Z to store the current guess of the solution
    % at different stages of the calculation. For instance Y is the image
    % after L2 minimization step, Z is the image after L1 minimization and
    % X is the resulting solution. To know the details - check out the code!!!
    %
    % See the code for description of all parameters of this class.
    %
    % EXAMPLES: 
    %
    % a = phantom(256);
    % b = radon(a);
    %
    % f = TV_min();
    %
    % % Define linear system A(X) = B with the initial guess X:
    % f.X = zeros(size(a));
    % f.Y = b;
    % f.A = @(x) radon(x);
    % f.A_trans = @(x) iradon(x, [], 'linear', 'none', 1, 256);
    %
    % % Parameters of iterative reconstruction:
    % f.L = 2*max(max(iradon(radon(ones(size(a))), [], 'linear', 'none')));
    % f.lambda = 0.01;
    % f.bounds = [0, inf];
    % f.X_true = a;
    %
    % [x, history] = f.run();
    %
    % figure(1); imshow(x, []);
    % figure(2); plot(history.err);
    
properties (Access = public)
    
    % Function handle for A(x)
    A = @(x) radon(x);
    
    % Function handle for A_transposed(x)
    A_trans = @(b) iradon(b, [], 'linear', 'none');
    
    % Lipschitz constant L = 2*max( A_trans * A )
    L = 2;
    
    % TV strength lambda
    lambda = 0;
    
    % Solution guess X
    X = [];
    
    % Observed data Y
    Y = [];
    
    % Number of iterations
    n_iter = 100;
    
    % Tolerance for stop condition:
    l2_tolerance = 0;
    upd_tolerance = 0;
    
    % Function handle for filter (can zero some pixels each iteration for instance)
    filter = [];
       
    % Lower and upper bounds for X
    bounds = [-inf, inf];
    
    % Fancy runtime display
    display_it = 1;    
    save_it = 0;
    save_file = '';
   
    % Glorious history
    history = [];    
    history_it = 1;
    
    % True solution, if known, to calculate the error
    X_true = [];
    
    % Calculator (GPU/CPU)
    calc = CPU_mat;
end

properties (Access = protected)
    
    % Some variables needed during the reconstruction:
    tau = 1;
    
    Z_ = [];
    Z = [];
    TV_res_x = [];
    TV_res_y = [];

    err_norm = [];
    init_norm = [];
    
    init_l2 = [];
   
end

methods (Access = public)
    
    function this = TV_min()
        
        % create an empty history:
        this.history.L2 = [];
        this.history.TV = [];
        this.history.err = [];
        this.history.upd = [];     
    end

    function [X, history] = run(this, X, n_iter)
        % Calculates X: min |A(X) - Y|^2 + lambda * TV
        % Some initialization:
        
        if nargin == 3
            this.n_iter = n_iter;
        end
        
        if nargin > 2
            this.X = X;
        end
        
        % Initial norm of X, used to normalize smth:
        this.init_norm = 0;%this.calc.norm(this.X);
        
        % Initial error norm, used to normalize smth: 
        if ~isempty(this.X_true)
            this.err_norm = this.calc.norm(this.X - this.X_true);
        end
        
        % Initial L2 norm:
        this.init_l2 = this.calc.norm(this.A(this.X) - this.Y)^2;
        
        % Z is the guess X with reducind L2 norm:
        this.Z = this.X;
        
        % tau is a number that determines step size in some complicated fashion:
        this.tau = 1;

        % Residual of TV norm, used for the next iteration of TV min.
        this.TV_res_x = [];
        this.TV_res_y = [];
        
        for i = 1:this.n_iter

            % Run one iteration:
            this = this.l2_iteration();
        
            % Update the display:
            if this.display_it
                
                if ~ismatrix(this.X)
                    show(this.X(:,:,end/2), [], 101); title('TV-min');
                else
                    show(this.X, [], 101); title('TV-min');
                end
                
                xlabel(sprintf('%i%%', round(i / this.n_iter * 100)));
                refresh(101);
            else
                %disp(i);
            end
            
            if this.save_it
                x = squeeze(this.X(:,:,end/2));
                save([this.save_file, num2str(i)], 'x');
                
                clear x;
            end
            
            % stop condition:
            if ~isempty(this.l2_tolerance)&&(i > 2)
                
                tol_number = abs(this.history.L2(end) - this.history.L2(end - 1)) ./ this.history.L2(end - 1);
                
                if tol_number < this.l2_tolerance
                    
                    fprintf('Convergence reached at %u\n', i);
                    break;
                end
            end
            
            % stop condition:
            if ~isempty(this.upd_tolerance)&&(i > 2)
                              
                if this.history.upd(end) < this.upd_tolerance
                    
                    fprintf('Convergence reached at %u\n', i);
                    break;
                end
            end
        end
        
        if this.display_it
            close(101);
        end
        
        X = this.X;
        history = this.history;
    end
    
    %
end

methods (Access = protected)
    
    function this = l2_iteration(this)
        % A single iteration of FISTA minimization:
                   
        % Subproblem (1), minimization of L2 = |A(X) - Y|^2:
        this.Z = this.Z - 2/this.L * this.A_trans( this.A(this.Z) - this.Y );
        
        % Subproblem (2), minimization of L1 = 2 * lambda * |TV(X)|:        
        if this.lambda > 0     
            
            % l1_iteration mainly calculates the image with lower TV and stores it in this.Z_:            
            this = this.l1_iteration();
        else
            
            % if l1_iteration is not necessary, then Z stores Y (the result of l2 minimization)
            this.Z_ = this.Z;
        end
        
        if ~isempty(this.filter)
            
            % Can apply any extra filter to the solution after each iteration:
            this.Z_ = this.filter(this.Z_);
        end

        % Compute the total variation and L2-norm:
        this.history.L2(end+1) = this.calc.norm(this.A(this.Z_) - this.Y)^2;       
        this.history.TV(end+1) = 2 * this.lambda * this.calc.TV(this.Z_);
            
        % Compute the error (except mean component):
        if ~isempty(this.X_true)
            err = this.X - this.X_true;
            err = err - mean(mean(err));      
            
            this.history.err(end+1) = this.calc.norm(err) / this.err_norm;
        else
            this.history.err(end+1) = 0;
        end

        % X before TV minimization:
        X_ = this.X;
        
        % X after TV minimization
        this.X = this.Z_;
        
        % Here the algorithm can be modified into 'monotonic' version.
        % Which was not tested yet. Something TODO in future...

        if this.init_norm == 0
            this.init_norm = this.calc.norm(this.X);
            this.history.upd(1) = this.init_norm;
        else
            this.history.upd(end+1) = this.calc.norm(this.X - X_) / this.init_norm;
        end
        
        % updating tau (needed for optimal step size)
        tau_ = this.tau;
        this.tau = (1 + sqrt(1+4*tau_^2))/2;

        % updating Z (needed for the next guess of X):
        this.Z = this.X;% + tau_ / this.tau * (this.Z - this.X) + (tau_ - 1) / this.tau * (this.X - X_);    
    end
    
    function this = l1_iteration(this)
        % Calculates image with lower TV from the image this.Y. Stores the
        % results in the image this.Z. It uses this.TV_res_x and this.TV_res_y
        % from the last time it was called.
        
        % Initialize TV residual:
        if isempty(this.TV_res_x)
            this.TV_res_x = zeros(size(this.Z));
            this.TV_res_y = zeros(size(this.Z));
        end

        % Modified TV residual:
        final_TV_res_x = this.TV_res_x;
        final_TV_res_y = this.TV_res_y;

        % these are some internal variables for this function:
        tau = 1;
        stop_count = 0;
        i = 0;
        la = 2 * this.lambda / this.L;

        % End result:
        this.Z_ = this.Z .* 0;

        while ((i < 10) && (stop_count < 5))
            i = i + 1;

            % old Z:
            Z_ = this.Z_;
    
            % new Xout:
            this.Z_ = this.Z - la * (this.calc.dx_(final_TV_res_x) + this.calc.dy_(final_TV_res_y));
            this.Z_ = this.calc.projection(this.Z_, this.bounds(1), this.bounds(2));

            % Taking a step towards minus of the gradient
            TV_res_x_ = this.TV_res_x;
            TV_res_y_ = this.TV_res_y;
            
            this.TV_res_x = final_TV_res_x - 1/(8*la) * this.calc.dx(this.Z_);
            this.TV_res_y = final_TV_res_y - 1/(8*la) * this.calc.dy(this.Z_);

            % this part can be changed to anisotropic, now it's L1 type:
            [this.TV_res_x, this.TV_res_y] = this.calc.normalize(this.TV_res_x, this.TV_res_y, 1);

            %Updating residual and tau:
            tau_ = tau;
            tau = (1 + sqrt(1 + 4*tau_^2)) / 2;

            final_TV_res_x = ...
                this.TV_res_x + (tau_ - 1) / (tau) * (this.TV_res_x - TV_res_x_);

            final_TV_res_y = ...
                this.TV_res_y + (tau_ - 1) / (tau) * (this.TV_res_y - TV_res_y_);

            % stop criterion:
            re = this.calc.norm(this.Z_ - Z_) / this.calc.norm(this.Z_);

            if (re < 1e-3)
                stop_count = stop_count + 1;
            else
                stop_count = 0;
            end
        end
    end
end
    
end