classdef TV_min_2
    % Modification of TV_min (check TV_min for details). TV_min_2 is
    % designed for support of system of equations with two unknown
    % variables mu and phi. 
properties (Access = public)
    
    % Function handle for A(x)
    A = struct('mu', @(x) radon(x), 'phi', @(x) radon(x));
    
    % Function handle for A_transposed(x)
    A_trans = struct('mu', @(b) iradon(b, [], 'linear', 'none'), ...
        'phi', @(b) iradon(b, [], 'linear', 'none'));
    
    % Lipschitz constant L = 2*max( A_trans * A )
    L = struct('mu', 2, 'phi', 2);
    
    % TV strength lambda
    lambda = struct('mu', 0, 'phi', 0);
    
    % Solution guess X
    X = struct('mu', [], 'phi', []);
    
    % Observed data Y
    Y = struct('mu', [], 'phi', []);
    
    % Number of iterations
    n_iter = 100;
       
    % Lower and upper bounds for X
    bounds = struct('mu', [-inf, inf], 'phi', [-inf, inf]);
    
    % Filter to apply to the solution at every iteration:
    filter = struct('mu', [], 'phi', []);
    
    % Fancy runtime display
    display_it = 1;    
    history_it = 1;
   
    % Glorious history
    history = [];    
    
    % True solution, if known, to calculate the error
    X_true = struct('mu', [], 'phi', []);
    
    % Calculator (GPU/CPU)
    calc = CPU_mat;
end

properties (Access = protected)
    
    % Some variables needed during the reconstruction:
    tau = 1;
    
    Z_ = struct('mu', [], 'phi', []);
    
    Z = struct('mu', [], 'phi', []);
    
    TV_res_x = struct('mu', [], 'phi', []);
    TV_res_y = struct('mu', [], 'phi', []);
    
    err_norm = struct('mu', [], 'phi', []);
    init_norm = struct('mu', [], 'phi', []);
end

methods (Access = public)
    
    function this = TV_min_2()
        
        % create an empty history:
        this.history = struct('L2', struct('mu', [], 'phi', []), ...
            'TV', struct('mu', [], 'phi', []),...
            'err', struct('mu', [], 'phi', []),...
            'upd', struct('mu', [], 'phi', []));  
    end

    function [X, history] = run(this, X, n_iter)
        % Calculates X: min |A(X) - Y|^2 + lambda * TV for two unknown
        % images phi and mu. Variables are defined using structures 
        % (X.mu and X.phi)
        
        % Matrix 'A' should be defined as struct (A.mu and A.phi) to have a
        % system of equations for two unknown images.
        if ~isstruct(this.A)
            error('Transformation matrix A is not defined for two independent variables phi and mu!'); 
        end
        
        % Initial norm of X, used to normalize smth:
        this.init_norm.mu = this.calc.norm(this.X.mu);
        this.init_norm.phi = this.calc.norm(this.X.phi);
        
        % Initial error norm, used to normalize smth: 
        if ~isempty(this.X_true)
            this.err_norm.mu = this.calc.norm(this.X.mu - this.X_true.mu);
            this.err_norm.phi = this.calc.norm(this.X.phi - this.X_true.phi);
        end
        
        % Z is the guess X with reducind L2 norm:
        this.Z = this.X;
        
        % tau is a number that determines step size in some complicated fashion:
        this.tau = 1;

        % Residual of TV norm, used for the next iteration of TV min.
        this.TV_res_x.mu = [];
        this.TV_res_y.mu = [];
        this.TV_res_x.phi = [];
        this.TV_res_y.phi = [];
        
        for i = 1:this.n_iter

            % Run one iteration (two variable version):
            this = this.l2_iteration();
        
            % Update the display:
            if this.display_it
                figure(101);
                subplot(1, 2, 1);
                show(this.X.mu, [], 101); title('TV-min: mu');
                subplot(1, 2, 2);
                show(this.X.phi, [], 101); title('TV-min: phi');
                %subplot(1, 1, 1);
                xlabel(sprintf('%i%%', round(i / this.n_iter * 100)));
                refresh(101);
            end

        end
        
        if this.display_it
            close(101);
        end
        
        X = this.X;
        history = this.history;
    end
end

methods (Access = protected)
      
    function D = multi_derivative(this)
        % Calculates the derivative when Z consists of mult. images (mu and phi)

            len = size(this.A.mu, 2);
            
            D.mu = zeros(size(this.Z.mu));
            D.phi = zeros(size(this.Z.phi));
            
            % loop over variables:
            for ii = 1:len
                S = this.A.phi{ii}; C = this.A.mu{ii};
                ST = this.A_trans.phi{ii}; CT = this.A_trans.mu{ii};
            
                residual = S(this.Z.phi) + C(this.Z.mu) - this.Y{ii};
            
                D.phi = D.phi + ST(residual) / len;
                D.mu = D.mu + CT(residual) / len;
            end
            
            clear residual;
    end
        
    function this = l2_iteration(this)
        % A single iteration of FISTA minimization in case of two
        % independent unknown images. 
                   
        % Subproblem (1), minimization of L2 = |A(X) - Y|^2:
        % First calculate total derivative of the problem:
        D = this.multi_derivative();
        
        this.Z.mu = this.Z.mu - 2/this.L.mu * D.mu;
        this.Z.phi = this.Z.phi - 2/this.L.phi * D.phi;
        
        % Subproblem (2), minimization of L1 = 2 * lambda * |TV(X)|:        
        if this.lambda.mu > 0     
            % l1_iteration mainly calculates the image with lower TV and stores it in this.Z_:            
            this = this.l1_iteration_mu();
        else
            this.Z_.mu = this.Z.mu;
        end
        
        if this.lambda.phi > 0     
            % l1_iteration mainly calculates the image with lower TV and stores it in this.Z_:            
            this = this.l1_iteration_phi();
        else
            this.Z_.phi = this.Z.phi;
        end
        
        if ~isempty(this.filter.mu)
            
            % Can apply any extra filter to the solution after each iteration:
            this.Z_.mu = this.filter.mu(this.Z_.mu);
        end

        if ~isempty(this.filter.phi)
            
            % Can apply any extra filter to the solution after each iteration:
            this.Z_.mu = this.filter.mu(this.Z_.phi);
        end
        
        if this.history_it
            D = this.multi_derivative();

            % Compute the total variation and L2-norm:
            this.history.L2.mu(end+1) = this.calc.norm(D.mu)^2;       
            this.history.TV.mu(end+1) = 2 * this.lambda.mu * this.calc.TV(this.Z_.mu);
            this.history.L2.phi(end+1) = this.calc.norm(D.phi)^2;       
            this.history.TV.phi(end+1) = 2 * this.lambda.phi * this.calc.TV(this.Z_.phi);

            % Compute the error (except mean component):
            err.mu = this.X.mu - this.X_true.mu;
            err.mu = err.mu - mean(mean(err.mu));
            err.phi = this.X.phi - this.X_true.phi;
            err.phi = err.phi - mean(mean(err.phi));

            if ~isempty(this.X_true.mu)
                this.history.err.mu(end+1) = this.calc.norm(err.mu) / this.err_norm.mu;
            else
                this.history.err.mu(end+1) = 0;
            end

            if ~isempty(this.X_true.phi)
                this.history.err.phi(end+1) = this.calc.norm(err.phi) / this.err_norm.phi;
            else
                this.history.err.phi(end+1) = 0;
            end

            % X before TV minimization:
            X_ = this.X;

            % X after TV minimization
            this.X = this.Z_;

            % Here the algorithm can be modified into 'monotonic' version.
            % Which was not tested yet. Something TODO in future...

            this.history.upd.mu(end+1) = this.calc.norm(this.X.mu - X_.mu) / this.init_norm.mu;
            this.history.upd.phi(end+1) = this.calc.norm(this.X.phi - X_.phi) / this.init_norm.phi;
        else
            % X before TV minimization:
            X_ = this.X;
            % X after TV minimization
            this.X = this.Z_;
        end
        
        % updating tau (needed for optimal step size)
        tau_ = this.tau;
        this.tau = (1 + sqrt(1+4*tau_^2))/2;

        % updating Z (needed for the next guess of X):
        this.Z.mu = this.X.mu + tau_ / this.tau * (this.Z.mu - this.X.mu)...
            + (tau_ - 1) / this.tau * (this.X.mu - X_.mu);  
        
        this.Z.phi = this.X.phi + tau_ / this.tau * (this.Z.phi - this.X.phi)...
            + (tau_ - 1) / this.tau * (this.X.phi - X_.phi);    
    end
    
     function this = l1_iteration_mu(this)
        % Calculates image with lower TV from the image this.Y.mu. Stores the
        % results in the image this.Z.mu. It uses this.TV_res_x.mu and
        % this.TV_res_y.mu from the last time it was called.
        
        % Initialize TV residual:
        if isempty(this.TV_res_x.mu)
            this.TV_res_x.mu = zeros(size(this.Z.mu));
            this.TV_res_y.mu = zeros(size(this.Z.mu));
        end

        % Modified TV residual:
        final_TV_res_x.mu = this.TV_res_x.mu;
        final_TV_res_y.mu = this.TV_res_y.mu;

        % these are some internal variables for this function:
        tau = 1;
        stop_count = 0;
        i = 0;
        la.mu = 2 * this.lambda.mu / this.L.mu;

        % End result:
        this.Z_.mu = this.Z.mu .* 0;

        while ((i < 10) && (stop_count < 5))
            i = i + 1;

            % old Z:
            Z_.mu = this.Z_.mu;
    
            % new Xout:
            this.Z_.mu = this.Z.mu - la.mu * (this.calc.dx_(final_TV_res_x.mu) + this.calc.dy_(final_TV_res_y.mu));
            this.Z_.mu = this.calc.projection(this.Z_.mu, this.bounds.mu(1), this.bounds.mu(2));

            % Taking a step towards minus of the gradient
            TV_res_x_.mu = this.TV_res_x.mu;
            TV_res_y_.mu = this.TV_res_y.mu;
            
            this.TV_res_x.mu = final_TV_res_x.mu - 1/(8*la.mu) * this.calc.dx(this.Z_.mu);
            this.TV_res_y.mu = final_TV_res_y.mu - 1/(8*la.mu) * this.calc.dy(this.Z_.mu);

            % this part can be changed to anisotropic, now it's L1 type:
            [this.TV_res_x.mu, this.TV_res_y.mu] = this.calc.normalize(this.TV_res_x.mu, this.TV_res_y.mu, 1);

            %Updating residual and tau:
            tau_ = tau;
            tau = (1 + sqrt(1 + 4*tau_^2)) / 2;

            final_TV_res_x.mu = ...
                this.TV_res_x.mu + (tau_ - 1) / (tau) * (this.TV_res_x.mu - TV_res_x_.mu);

            final_TV_res_y.mu = ...
                this.TV_res_y.mu + (tau_ - 1) / (tau) * (this.TV_res_y.mu - TV_res_y_.mu);

            % stop criterion:
            re = this.calc.norm(this.Z_.mu - Z_.mu) / this.calc.norm(this.Z_.mu);

            if (re < 1e-3)
                stop_count = stop_count + 1;
            else
                stop_count = 0;
            end
        end
     end
    
    function this = l1_iteration_phi(this)
        % Calculates image with lower TV from the image this.Y.mu. Stores the
        % results in the image this.Z.mu. It uses this.TV_res_x.mu and
        % this.TV_res_y.mu from the last time it was called.
        
        % Initialize TV residual:
        if isempty(this.TV_res_x.phi)
            this.TV_res_x.phi = zeros(size(this.Z.phi));
            this.TV_res_y.phi = zeros(size(this.Z.phi));
        end

        % Modified TV residual:
        final_TV_res_x.phi = this.TV_res_x.phi;
        final_TV_res_y.phi = this.TV_res_y.phi;

        % these are some internal variables for this function:
        tau = 1;
        stop_count = 0;
        i = 0;
        la.phi = 2 * this.lambda.phi / this.L.phi;

        % End result:
        this.Z_.phi = this.Z.phi .* 0;

        while ((i < 10) && (stop_count < 5))
            i = i + 1;

            % old Z:
            Z_.phi = this.Z_.phi;
    
            % new Xout:
            this.Z_.phi = this.Z.phi - la.phi * (this.calc.dx_(final_TV_res_x.phi) + this.calc.dy_(final_TV_res_y.phi));
            this.Z_.phi = this.calc.projection(this.Z_.phi, this.bounds.phi(1), this.bounds.phi(2));

            % Taking a step towards minus of the gradient
            TV_res_x_.phi = this.TV_res_x.phi;
            TV_res_y_.phi = this.TV_res_y.phi;
            
            this.TV_res_x.phi = final_TV_res_x.phi - 1/(8*la.phi) * this.calc.dx(this.Z_.phi);
            this.TV_res_y.phi = final_TV_res_y.phi - 1/(8*la.phi) * this.calc.dy(this.Z_.phi);

            % this part can be changed to anisotropic, now it's L1 type:
            [this.TV_res_x.phi, this.TV_res_y.phi] = this.calc.normalize(this.TV_res_x.phi, this.TV_res_y.phi, 1);

            %Updating residual and tau:
            tau_ = tau;
            tau = (1 + sqrt(1 + 4*tau_^2)) / 2;

            final_TV_res_x.phi = ...
                this.TV_res_x.phi + (tau_ - 1) / (tau) * (this.TV_res_x.phi - TV_res_x_.phi);

            final_TV_res_y.phi = ...
                this.TV_res_y.phi + (tau_ - 1) / (tau) * (this.TV_res_y.phi - TV_res_y_.phi);

            % stop criterion:
            re = this.calc.norm(this.Z_.phi - Z_.phi) / this.calc.norm(this.Z_.phi);

            if (re < 1e-3)
                stop_count = stop_count + 1;
            else
                stop_count = 0;
            end
        end
    end
end
    
end