classdef detection
    % Generates resolution and spectral properties (CTF, PSF, QDE) of the x-ray tube and
    % the detector.

    methods(Static)
        
        function sz = source_size_scaling(sz, r1, r2)
            % Scaling of the source size projected on the image plane depending on the position of the object. 
            %
            % sz = detection.source_size_scaling(sz, r1, r2)
            
            m = geometry.M(r1, r2);
            sz = ((m-1) / m) * sz;
        end
        
        function sz = detector_pixel_scaling(sz, r1, r2)
            % Scaling of the detector pixel size projected on the image plane depending on the position of the object. 
            %
            % sz = detection.detector_pixel_scaling(sz, r1, r2)
            
            m = geometry.M(r1, r2);
            sz = sz / m;
        end
        
        function ctf = gaussian_ctf(params, pix_sz, sigma)
            % Gaussian CTF in Fourier space (corresponds to PSF in space domain). 
            %
            % ctf = detection.gaussian_ctf(params, pix_sz, sigma)
            
            dims = size_gen(params);
            
            wx = geometry.wx_space(pix_sz, dims);
            wy = geometry.wy_space(pix_sz, dims);
            
            if numel(sigma) == 1
                sigma = [sigma, sigma, sigma];
            end
                
            ctf = exp(-((wx*sigma(2)).^2 + ...
                (wy*sigma(1)).^2)/2);

            if numel(dims) > 2
                wz = geometry.wz_space(pix_sz, dims);
                ctf = ctf .* exp(-((wz*sigma(3)).^2)/2);
            end
        end
        
        function ctf = sperical_ctf(params, pix_sz, radius)
            % Projected sphere CTF in Fourier space (corresponds to PSF in space domain).
            %
            % ctf = detection.sperical_ctf(params, pix_sz, radius)
            
            dims = size_gen(params);

			uvrR = sqrt(geometry.w2_space(pix_sz, dims)) * radius;
			
			ctf = real(-3 * (uvrR .* cos(uvrR) - sin(uvrR)) ./ (uvrR.^3));
			ctf(uvrR == 0) = 1;
        end
        
        function ctf = sinc_ctf(params, pix_sz, pitch)
            % Sinc function CTF. Corresponds to convolution with a block(pitch) in space domain. 
            %
            % ctf = detection.sinc_ctf(params, pix_sz, pitch)
            
            dims = size_gen(params);
            
            ctf = sinc(geometry.wx_space(pix_sz, dims) * pitch / (2 * pi)) .* ...
                sinc(geometry.wy_space(pix_sz, dims) * pitch / (2 * pi));
        end
        
        function spectrum = gaussian_spectrum(mean_e, sigma_e, energy)
            % Generates gaussian-like spectrum with given mean and STD.
            %
            % spectrum = detection.gaussian_spectrum(mean_e, sigma_e, energy)
            
            spectrum = exp(-(energy - mean_e).^2 / (2*sigma_e.^2));
        end
              
        function spectrum = detector_spectrum(material, thikness, energy)
            % Generates spectral sensitivity of scintillator (i.e. QDE) with particular composition and thikness(units: meters, eV). 
            %
            % spectrum = detection.detector_spectrum(material, thikness, energy)
            %
            % Default material is BaFBr
            
            % PhotonAttenuation is in [cm^2/g]. Thickess must be in meters.
            spectrum = 1 - exp(- thikness * 100 * PhotonAttenuation(material, energy * 1e-6, 'meac')');
            
            % Normalize:
            spectrum = spectrum ./ max(spectrum);
        end
        
        function spectrum = bremsstrahlung_spectrum(voltage, energy)
            % Normalized bremsstrahlung radiation spectrum for an X-ray tube using Kramer formula. 
            %
            % spectrum = detection.bremsstrahlung_spectrum(voltage, energy)
            %
            % Voltage should be in Volts. Energy - eV.
            %
                        
            % Cutoff energy (Duane-Hunt law):
            energy0 = 2*pi* unit_convert.h_bar_ev / unit_convert.h_ev * voltage;
                        
            % Kramer formula:
            spectrum = energy.^2 .* (energy0 ./ energy - 1);
            spectrum(spectrum < 0) = 0;
            
            % Normalized spectrum:
            spectrum = spectrum ./ max(spectrum);
            
        end
        
    end
end