classdef geometry
    % Class that helps to generate coordinate and frequency space in the
    % detector plane for the fan beam imaging geometry. In order to
    % simulate parallel beam geometry R1 (source-to-object distance) must
    % be set to infinity.
    
    methods(Static)
        
        function w = w2_r(pix_sz, dims, R1, R2) 
            % Calculates frequency squared multiplied with effective
            % propagation distance R that can be used as a parameter for
            % the propagators.
            
			w = geometry.w2_space(pix_sz, dims) * geometry.r_effective(R1, R2);
        end
        
        function w2 = w2_space(pix_sz, dims)
            % Frequency squared space in the detector plane.
            
            sz = geometry.plane_size(pix_sz, dims);
                                   
            u = 2 * pi * xxx(dims) / sz(2);
            v = 2 * pi * yyy(dims) / sz(1);
           
            % 2D / 3D case:
            if size(sz) < 3
                w2 = ifftshift(u.^2 + v.^2);
            else
                w = 2 * pi * zzz(dims) / sz(3);
                
                w2 = ifftshift(u.^2 + v.^2 + w.^2);
            end
            
        end
        
        function wx = wx_space(pix_sz, dims)
            % Frequency space along x dimention in the detector plane.
            
            sz = geometry.plane_size(pix_sz, dims);          
            wx = ifftshift(2 * pi * xxx(dims) / sz(2));
            
        end
        
        function wy = wy_space(pix_sz, dims)
            % Frequency space along y dimention in the detector plane.
            
            sz = geometry.plane_size(pix_sz, dims);          
            wy = ifftshift(2 * pi * yyy(dims) / sz(1));
            
        end
        
        function wz = wz_space(pix_sz, dims)
            % Frequency space along y dimention in the detector plane.
            
            sz = geometry.plane_size(pix_sz, dims);          
            wz = ifftshift(2 * pi * zzz(dims) / sz(3));
            
        end
        
        function sz = plane_size(pix_sz, dims)
            % Size of the image plane.            
            sz = pix_sz .* dims;
            
        end
       
        function r = r_effective(R1, R2)
            % Effective propagation distance (effective as in parallel beam)
            m = geometry.M(R1, R2);
            
            r = R2 / m;
        end
        
        function m = M(R1, R2)
            % Magnification of the system.
            
            if R1 == inf 
                m = 1;
            else
                m = (R1 + R2) / R1;
            end
        end
    end
end