classdef phantoms
    % Generator of projected phantom images for X-ray PCI simulations. 
    % Uses NIST database to find the optical(x-ray) properties of materials.
    
    methods(Static)
        
        function image = sphere(params, coord, radius)
            % Draw a sphere. <sphere(params, coord, radius)>
            
            image = real(sqrt((radius)^2 - rrr(params, coord).^2));
        end
        
        function image = circle(params, coord, radius)
            % Draw a full circle. <circle(params, coord, radius)>
            
            image = real(((rrr(params, coord) < radius)));
        end
        
        function image = edge(params, angle)
            % Draw an edge through the center of coordinates. <edge(params, angle)>
            
            image = yyy(params)*cos(angle) > xxx(params)*sin(angle);
        end
        
        function image = block(params, coord, width, length, angle)
            % Draw a block. <block(params, coord, width, length, angle)>
            
            shift_horz = width .* [sin(angle), cos(angle)] / 2;
            shift_vert = length .* [cos(angle), -sin(angle)] / 2;
            
            image = ...
                (yyy(params, coord + shift_horz)*cos(angle) < -xxx(params, coord + shift_horz)*sin(angle)) &...
                (yyy(params, coord - shift_horz)*cos(angle) > -xxx(params, coord - shift_horz)*sin(angle)) &...
                (-yyy(params, coord + shift_vert)*sin(angle) < -xxx(params, coord + shift_vert)*cos(angle)) &...
                (-yyy(params, coord - shift_vert)*sin(angle) > -xxx(params, coord - shift_vert)*cos(angle));     
        end
        
        function image = spheres_phantom(image, flat, radius, number, seed)
            % Generate phantom that consists of many cirlcles / spheres. <spheres_phantom(image, flat, radius, number, seed)>
            
            RandStream.setDefaultStream(RandStream('mt19937ar','seed',seed));

            sz = size(image);
            coords = (rand(2, number) - 0.5) * (min(sz) - radius * 5) + (sz(1) / 2);
            radii = abs(randn(1, number)) * radius;

            image = image .* 0;
            
            % Generate circles:
            for ii = 1:number
                if flat
                    image = image + phantoms.circle(image, coords(:,ii), radii(ii));
                else
                    image = image + phantoms.sphere(image, coords(:,ii), radii(ii));
                end
            end

            % Normalize:
            image = image / max(image(:));
            
        end
        
        function m = find_material(name)
            % Find the material properties. See Matlab code of PhysProps
            % for the list of appropriate material names. <find_material(name)>
            
			PP = PhysProps(name);
            
            if(isnan(PP{1}) || isnan(PP{2}))
				error('Material name not reconized, see help PhysProps')
            else
                % Z/A ratio and density:
                m.za = PP{1};
                m.rho = PP{2}; 
                
                % Chem. formula:
                m.formula = PP{3};
                m.name = PP{4};
                
                % average Z number:
                [z r] = ParseChemicalFormula(PP{3});
                m.z = sum(z .* r);
            end
        end
              
        function n = material_refraction(name, energy)
            % Calculates complex refrative index of the material taking
            % into account it's density. See Matlab code for PhysProps for
            % the list of appropriate material names. <material_refraction(name, energy)>
            
            % Find material:
            m = phantoms.find_material(name);
            
            % Electron density of the material:
            rho_e = m.rho .* m.za .* unit_convert.Na;
            
            % Attenuation:
			mu = permute(PhotonAttenuation(name, energy .* 1e-6, 'mac'), [2 1]);

            % Phase:
			phi = rho_e .* 100^2 .* unit_convert.re .* unit_convert.e2lambda(energy);
            
            % Refraction index (per meter)
            n = 100 * m.rho .* (mu/2 - 1i * phi);
        end
    end
end
