%Solution for multiple distances PCI based on CTF model.
%
% This script demonstrates the results of phase retrieval based on
% TV-minimization versus the standard noniterative inversion approach for
% MIXED model. 
% [M. Langer et al., Medical Physics, Vol. 35, No. 10, 2008]
% [De Caro et al., Medical Physics, Vol. 37, No. 7, 2010]
%
% Input phase-contrast images is simulated using Fresnel propagation in
% <a href="matlab:help monochromatic_simulation.m">monochromatic_simulation.m</a> script.
%
% In this script phase-retrieval is based on multiple phase-contrast images
% recorded at different distances. System of equations that represent 
% each of the recorded images is solved simultaneously by TV-minimization.
% Check <a href="matlab:help full_ctf_system_example.m">full_ctf_system_example.m</a> for more information.

close all
clear all
%% Load the input data:

% current dir:
dir = fileparts(mfilename('fullpath'));

% Load images:
pci_images = ...
    {load_binary([dir, '\monochromatic_simulation\plastic_phantom_0m_pci.dat'], [256 256]),...
    load_binary([dir, '\monochromatic_simulation\plastic_phantom_0.5m_pci.dat'], [256 256])}

% Round truth (for I0 image and I0*phi)
ground_truth.mu = load_binary([dir, '\monochromatic_simulation\plastic_phantom_intensity.dat'], [256 256]);
ground_truth.phi = ground_truth.mu .* load_binary([dir, '\monochromatic_simulation\plastic_phantom_phi.dat'], [256 256]);

% Show pci image and ground truth:
show(pci_images{2}, [], 1, 'Phase-contrast image');
show(ground_truth.phi, [], 2, 'Phase retrieved image (ground truth)');

%% Initialize linear propagator based on Dual CTF model:

% Experimental conditions:
energy = 20e3;          % KeV
pixelsize = 1e-6;       % microns
r1 = inf;               % parallel beam
r2 = {0, 0.5};          % meters

%% Construct the Least-squares solution for MIXED phase-propagator:
sz = size(pci_images{1});

% Construct the Least-squares solution for multiple distance:
numerator = zeros(sz);
denominator = zeros(sz);

% Fourier transform of the zero-distance image:
ft_i0 = fft2(pci_images{1});

% The first of the pci_images will be used as 'zero-distance' image I0:
for ii = 2:length(pci_images)
   
    w = PCI_rec.frequency_space(sz, pixelsize, energy, r1, r2{ii});  
        
    s = 2*sin(w);
    c = cos(w); 
    
    % Generate numerator and denominator of the equation:
    numerator = numerator + (fft2(pci_images{ii}) - ft_i0 .* c ) .* s;
    denominator = denominator + s.^2;
end

%% Solve the inverse problem using Wiener deconvolution:

% Wiener deconvolution:
epsilon = 1e-4; 
wiener_recon = numerator ./ (denominator + 1e-4);

% Set zero frequency to 1:
wiener_recon = real(ifft2(wiener_recon));

% Show the result:
show(wiener_recon, [], 3, 'Wiener deconvolution');

%clear C CC A B AA mu phi;
%% Construct the system of equations for multiple distances (to use it in TV):
% The solution is found by minimizaing the descripancy of the MIXED model
% with each mesured image including zero-distance image i0

% Initialize the TV_min:
f = TV_min_2();

% Initialize a cell-array for known variables:
f.Y = cell(1, length(pci_images));

% Initialize transformation matrixes for two variables system (phi and mu):
f.A.mu = cell(1, length(pci_images));
f.A_trans.mu = cell(1, length(pci_images));

f.A.phi = cell(1, length(pci_images));
f.A_trans.phi = cell(1, length(pci_images));

f.L.mu = 0;
f.L.phi = 0;

% Define the system of equations for MIXED aprroach:
% I_r = cos(w) * I_0 + 2sin(w) * (I_0 * phi)

for ii = 1:length(pci_images)

    % Frequency space:
    w = PCI_rec.frequency_space(sz, pixelsize, energy, r1, r2{ii});
    s = 2*sin(w);
    c = cos(w);
    
    % Intensity image propagator; we use A.mu to store it:
    f.A.mu{ii} = @(x) c .* fft2(x);
    
    % Propagator for (I0*phi) term; we use A.phi to store it:
    f.A.phi{ii} = @(x) s .* fft2(x);
    
    % transposed propagators:
    f.A_trans.mu{ii} = @(x) real(ifft2(c .* x ));
    f.A_trans.phi{ii} = @(x) real(ifft2(s .* x ));
    
    % Known images:
    f.Y{ii} = fft2(pci_images{ii});
       
    f.L.phi = 2 * max(max(abs(s(:)).^2), f.L.phi/2);
    f.L.mu = 2 * max(max(abs(c(:)).^2), f.L.mu/2);
end
%% Solve the system of equations using TV-minimization approach:

% Unknown variables (can be two: mu and phi), initial guess:
%f.X.mu = tv_recon.mu;zeros(sz);
%f.X.phi = tv_recon.phi;zeros(sz);
f.X.mu = zeros(sz);
f.X.phi = zeros(sz);

% Initialization parameters of the algorithm:
f.n_iter = 2000;

f.lambda.mu = f.L.mu / numel(wiener_recon) * 600;
f.lambda.phi = f.L.phi / numel(wiener_recon) * 600;

f.X_true = ground_truth;
 
f.bounds.mu = [-inf, inf];
f.bounds.phi = [-inf, inf];

% Calculate!
[tv_recon, history] = f.run();
 
% Set mean to zero:
tv_recon.mu = tv_recon.mu;
tv_recon.phi = tv_recon.phi;

% Display the result:
show(tv_recon.phi, [], 4 , 'TV-minimization: phi');
show(tv_recon.mu, [], 5 , 'TV-minimization: mu');

% Display the history of convergence:
figure(6); 
semilogy(history.L2.phi + history.TV.phi); title('Cost function: phi');

figure(7); 
semilogy(history.err.phi); title('Error: phi');

%% Compare the results in real and in frequency space:

% Real space error of the Wiener reconstruction:
wiener_err = wiener_recon - ground_truth.phi;
wiener_err = wiener_err - mean(wiener_err(:));

% Real space error of the Wiener reconstruction:
tv_err = tv_recon.phi - ground_truth.phi;
tv_err = tv_err - mean(tv_err(:));

% Show error in real space:
show(wiener_err, [], 7, 'Real space error: Wiener deconvolution'); 
colorbar;
show(tv_err, [], 8, 'Real space error: TV-mimimization deconvolution'); 
colorbar;

% Calculate average Fourier spectrum of the images and the associated
% errors:
ft_image = ifftshift(fft2(ground_truth.phi));
image_spectrum = radial_average(abs(ft_image));

ft_image = ifftshift(fft2(wiener_err));
wiener_error_spectrum = radial_average(abs(ft_image));

ft_image = ifftshift(fft2(tv_err));
tv_error_spectrum = radial_average(abs(ft_image));

% 1D version of the propagator (to overlay it with the image spectra):
p_1d = sqrt(radial_average(abs(ifftshift(denominator))));
p_1d = p_1d ./ max(p_1d);

% Show error in frequency space:
show(wiener_error_spectrum ./ image_spectrum, 'b', 9); hold on;
show(tv_error_spectrum ./ image_spectrum, 'r',9); 
show(p_1d, 'k:', 9); hold off;
ylim([0, 3]);

legend('Wiener error', 'TV error', 'Propagator envelope');
title('Error in frequency space');

%% Calculate the descripancy between Wiener and TV:
change_sp = radial_average(abs(ifftshift(fft2(wiener_recon - tv_recon.phi))));
wiener_sp = radial_average(abs(ifftshift(fft2(wiener_recon))));

%Show:
show(change_sp ./ wiener_sp, 'b', 10);hold on;
show(p_1d ./ max(p_1d), 'k:', 10); hold off;
xlim([2, 128]);
title('Difference with Wiener deconvolution');