% Propagation-based PCI forward modelling.
% This script contains an example of simulation of the phase-contrast image
% (monochromatic) of a simple digital phantom. The digital phantom used in
% simulation is randomly generated scatter of polyethelen spheres submerged 
% in water. Image is simulated for single x-ray energy (20KeV). The Point
% Sperad Functions of the x-ray source and the detector are not taken into
% account for simplicity. 
%
% The result of the simulation is saved on harddisk and can be uploaded
% later on to test the reconstruction routines, for instance in
% <a href="matlab:help dual_ctf_example">dual_ctf_example.m</a>

%% Initialize:

% Coordinate system grid and spacing:
pci_image = zeros(256);
pixelsize = 1e-6;           % 1 micron
energy = 20e3;              % KeV

%% Generate 'plastic balls' phantom:

% Generate the shape (scatter of spheres):
flat = 0;
radius = 5;
number = 150;
seed = 10;
depth = 0.1e-3;

phantom_proj = phantoms.spheres_phantom(pci_image, flat, radius, number, seed);

% Generate refractive indexes of water and polyethelen at 20KeV for 1mm width:
n_water = phantoms.material_refraction('WATER', energy) .* depth;
n_polyethylen = phantoms.material_refraction('POLYETHYLENE', energy) .* depth;

% Generate the total projected refraction index:
refraction_image = (phantom_proj .* n_polyethylen + (1-phantom_proj) .* n_water);

% Generate the resulting field image:
field_image = exp(-refraction_image);  

% Generate an intensity image:
intensity_image = abs(field_image).^2;

% Generate images for linear phase and attenuation:
phi_image = imag(refraction_image);
mu_image = real(refraction_image);

% Show the intensity image:
show(intensity_image, [], 1); colorbar
title('Intensity image');
%% Simulation of the observed image:

% Experimental conditions:
r1 = inf;           % meters
r2 = 1;             % meters

% Propagate image (monochromatic) and calculate image in the detector plane:
pci_image = PCI_rec.forward_propagation(field_image, pixelsize, energy, r1, r2);

% Add noise (no CTF):
pci_image = PCI_rec.add_ctf_noise(pci_image, [], 10000, 'poisson');

% Show oberved phase-contrast image:
show(pci_image, [], 2);
colorbar;
title('Phase-contrast image');

%% Save the result:

% current dir:
dir = fileparts(mfilename('fullpath'));

% Save files:
save_binary(pci_image, [dir, '\plastic_phantom_1m_pci.dat'])
save_binary(phi_image, [dir, '\plastic_phantom_phi.dat']);
save_binary(mu_image, [dir, '\plastic_phantom_mu.dat']);
save_binary(intensity_image, [dir, '\plastic_phantom_intensity.dat']);