% Propagation-based X-ray PCI forward modelling.
% This script contains an example of polychromatic forward model of the
% phase-contrast imaging system. It shows how the following properties of
% the imaging system can be implemented:
%
%   1. Polychromatic spectrum of the x-ray source and the quantum efficiency 
% of the detector.
%   2. Point Spread Function (PSF) of the x-ray source and the detector
%   3. Optical properties of the specimen's depending on it's composition.
%
% See also: phantoms, detection, PCI_rec, monochromatic_simulation.m

%% Initialize:

% Coordinate system spacing:
pixelsize = 1e-6;   % 1 micron
r1 = 5;             % 5 meter
r2 = 1;             % 1 meter
photon_number = 1000;

image = zeros(512);

%% Generate simple edge phantom:

% Generate the shape:
phantom_proj = phantoms.block(image, [256, 256], 50, 200, 0);

% Use energy dependent refractive index of steel and air to generate field
% image later on (for 0.01mm thick object): 

energy = (1:1:100)*1e3;   % eV
n_steel = phantoms.material_refraction('Steel', energy) .* 0.01e-3;
n_air = phantoms.material_refraction('Air', energy) .* 0.01e-3;

%% Effective spectrum of the system:

% X-ray tube spectrum:
voltage = 60e3;     % Volt
bremss = detection.bremsstrahlung_spectrum(voltage, energy);

% Quantum Detection Efficiency
material = 'BaFBr'; % scintillator material
thikness = 0.5e-3;  % meter
qde = detection.detector_spectrum(material, thikness, energy);

% Transmission in air at 1m length
air_refraction = phantoms.material_refraction('Air', energy);
air = exp(-real(air_refraction) .* 1);

% Normalized effective spectrum:
spectrum = bremss .* qde .* air;
spectrum = spectrum / sum(spectrum);

% Plot the effective spectrum:
figure(1); clf; hold on;
plot(energy / 1e3, bremss, 'g');
plot(energy / 1e3, qde, 'r');
plot(energy / 1e3, air, 'k:');
plot(energy / 1e3, spectrum, 'b');

title('Spectrum'); xlabel('KeV');
legend('Source', 'Detector', 'Air transmission', 'Total');

%% Source and detector CTF (Contrast Transfer Functions)

% X-ray source is as a sphere (with radius = 10 micron):
radius = detection.source_size_scaling(10e-6, r1, r2);
src_ctf = detection.sperical_ctf(image, pixelsize, radius);

% Detector CTF (combination of gaussian and block PSF):
sigma = detection.detector_pixel_scaling(1e-6, r1, r2);
pitch = detection.detector_pixel_scaling(2e-6, r1, r2);

det_ctf = detection.gaussian_ctf(image, pixelsize, sigma) .* ...
    detection.sinc_ctf(image, pixelsize, pitch);

% Combined Point Spread Function:
show(fftshift(real(ifft2(src_ctf .* det_ctf))), [], 2);
title('PSF of the imaging system');
%% Simulation of the observed image:

image = image .* 0;

% Loop over different energy components:
for ii = 1:length(spectrum)
    
    if spectrum(ii) > 0
        % Calculate the field image in the object plane (depends on energy):
        field_image = exp(-(phantom_proj .* n_steel(ii) + (1-phantom_proj) .* n_air(ii)));    
    
        % Propagate image (monochromatic) and calculate image in detector plane:
        mono = PCI_rec.forward_propagation(field_image, pixelsize, energy(ii), r1, r2);
    
        % Adding monochromatic components:
        image = image + mono .* spectrum(ii);
    end
end

% Add effects of CTF(PSF) and noise:
image = PCI_rec.add_ctf_noise(image, src_ctf .* det_ctf, photon_number, 'poisson');

% Show oberved edge:
show(image, [], 3); title('Phase-contrast image of the edge');

% Show observed edge profile:
show(mean(image, 2), 'b', 4); title('Edge profile');