function [AttCoef, CompEdge] = PhotonAttenuationQ(Element, Energy, Options)
%PhotonAttenuationQ NIST attenuation cooeficiant tables
% for photon interaction with elements.
%
% [AttCoef, CompEdge] = PhotonAttenuationQ(Material, Energy, Options)
% Function providing the attenuation and energy absorption of x-ray and
% gamma-ray photons in various materials, based on NIST report 5632, by
% J. Hubbell and S.M. Seltzer. This is a quick version of the function with
% narrow range of inputs and outputs.
%
% Input :
%   Element - atomic number Z in [1, 100] range, or array of Z numbers 
%   Energy - single energy or vector of energies. For best results should
%         be in [0.001, 20] MeV range. 
%   Options - specifies what to return. String or number:
%     1 - 'mac' - function returns Mass Attenuation Coefficients in cm^2/g
%         (default)
%     2 - 'meac' - function returns Mass Energy-Absorption Coefficients 
%           in cm^2/g. Data is available only for elements with Z<93. See 
%           link below for more info:
%           http://physics.nist.gov/PhysRefData/XrayMassCoef/chap3.html
% Output :
%   AttCoef - Either Mass Attenuation Coefficients Mass Energy-Absorption 
%             Coefficients in cm^2/g depending on 'Option'. Columns 
%             correspond to material and rows correspond to energy.
%   CompEdge - a list of Compton edges in all of 'Element's. One edge per
%     row; columns are:
%     1) Edge number for this elements
%     2) Atomic number Z of elements
%     3) Energy of the edge in MeV
%     4) Attenuation Coefficients of edge's low  value
%     5) Attenuation Coefficients of edge's high value
%
% History:
%  Written by Jarek Tuszynski (SAIC), 2006
%  Inspired by John Schweppe Mathematica code available at
%    http://library.wolfram.com/infocenter/MathSource/4267/
%
% See Also:
%   PhotonAttenuation, ParseChemicalFormula, PhysProps
% 
% References:
%   Tables are based on "X-Ray Attenuation and Absorption for Materials of 
%   Dosimetric Interest" (XAAMDI) database (NIST 5632 report):
%   J. Hubbell and S.M. Seltzer, "Tables of X-Ray Mass Attenuation
%   Coefficients and Mass Energy-Absorption Coefficients 1 keV to 20 MeV for
%   Elements Z = 1 to 92 and 48 Additional Substances of Dosimetric Interest, "
%   National Institute of Standards and Technology report NISTIR 5632 (1995).
%   http://physics.nist.gov/PhysRefData/XrayMassCoef/cover.html
%
%   MAC values for elements 93 to 100 (Neptunium to Fermium) came from 
%   XCOM: Photon Cross Sections Database (NBSIR 87-3597): 
%   Those tables give photon's "total attenuation coefficients" for 
%   elements with atomic number (Z) smaller than 100.
%   Photon energy range is from 0.001 MeV to 100 GeV.
%   http://physics.nist.gov/PhysRefData/Xcom/Text/XCOM.html
% 
% Details:
%   This function is intended as a low-level, 'bare-bones' version of 
%   PhotonAttenuation function and contain mostly the NIST tables and a 
%   little code to look them up and interpolate results. Very few input
%   options are allowed. See PhotonAttenuation function for more options.
%
%   The Attenuation tables are stored in two formats: standard grid values
%   defined for all the elements and values at Compton edges are stored
%   separately. That way they take less space and Compton edge data is
%   easier to extract. The values in the table are for room temperature 
%   and pressure. 
%   
%   Interpolation is done using MATLAB's interp1 function to log-log data of 
%   coefficients as function of energy. Linear interpolation is used near
%   Compton edges and cubic otherwise.
%
% Examples:
% %% Plot Photon Mass Attenuation Coefficients of elements as function of 
% % energy and atomic number
% figure
% Z = 1:100; % elements with Z in 1-100 range 
% E = exp(-6.9:0.01:2.99);  % define energy grid
% [mac, CEdge] = PhotonAttenuationQ(Z, E);
% imagesc(log10(mac)); colorbar;
% title('Photon Mass Attenuation Coefficients and Compton edges');
% xlabel('Atomic Number of Elements');
% ylabel('Energy in MeV');
% set(gca,'YTick',1:123:1000);
% set(gca,'YTickLabel',exp(-6.9:1.23:2.99))
% hold on
% ed = accumarray([CEdge(:,1),CEdge(:,2)],CEdge(:,3));
% plot((log(ed')+6.9)/0.01 ,'LineWidth',4); % plot compton edges
% legend({'K','L1','L2','L3','M1','M2','M3','M4','M5','N1','N2','N3','N4','N5'}, 'Location', 'west');
% hold off;
%
% %% Plot Mass Energy-Absorption Coefficients of elements
% % energy and atomic number 
% figure
% Z = 1:92; % elements with Z in 1-92 range (Elements higher than 92 are not defined)
% E = exp(-6.9:0.01:2.99);  % define energy grid
% meac = PhotonAttenuationQ(Z, E, 'meac');
% imagesc(log10(meac)); colorbar;
% title('Photon Mass Energy-Absorption Coefficients');
% xlabel('Atomic Number of Elements');
% ylabel('Energy in MeV');
% set(gca,'YTick',1:123:1000);
% set(gca,'YTickLabel',exp(-6.9:1.23:2.99))
% 
% %% Plot Photon Attenuation Coefficients for Uranium
% figure
% E = exp(-6.9:0.01:2.99); % define energy grid
% mac  = PhotonAttenuationQ(92, E, 'mac');
% meac = PhotonAttenuationQ(92, E, 'meac');
% loglog(E, mac); hold on;
% loglog(E, meac, 'b-.');
% legend({'mac', 'meac'});
% ylabel('Attenuation in cm^2/g');
% xlabel('Photon Energy in MeV');
% title({'Photon Attenuation Coefficients for Uranium',...
% 'see http://physics.nist.gov/PhysRefData/XrayMassCoef/ElemTab/z92.html'});
% %compare with http://physics.nist.gov/PhysRefData/XrayMassCoef/ElemTab/z92.html
 
if (nargin<3), Options = 1; end
if (ischar(Options))
  X = {'mac','meac'};
  Options = find(strcmpi( Options, X),1);
end 
if (isempty(Options)), Options = 1; end

%% Read in Hard-wired data tables
persistent mac meac edges
%--------------------------------------------------------------------------
% Hard-wire the table of x-ray mass attenuation coefficients (MAC) and mass
% energy absorption coefficients (MEAC). Units:    
% energy is in MeV, and both MAC and MEAC are in cm^2/g. 
%-------------------------------------------------------------------------- 
energy = [...
1.000E-3, 1.500E-3, 2.000E-3, 3.000E-3, 4.000E-3, 5.000E-3, 6.000E-3, 8.000E-3, 1.000E-2, 1.500E-2, 2.000E-2, 3.000E-2, 4.000E-2, 5.000E-2, 6.000E-2, 8.000E-2, 1.000E-1, 1.500E-1, 2.000E-1, 3.000E-1, 4.000E-1, 5.000E-1, 6.000E-1, 8.000E-1, 1.000   , 1.250   , 1.500   , 2.000   , 3.000   , 4.000   , 5.000   , 6.000   , 8.000   , 1.000E+1, 1.500E+1, 2.000E+1];

%--------------------------------------------------------------------------
%% Hard-wire the table of x-ray mass attenuation coefficients incm^2/g    
%-------------------------------------------------------------------------- 
if (Options==1 && isempty(mac))
mac = [...
7.217   , 2.148   , 1.059   , 5.612E-1, 4.546E-1, 4.193E-1, 4.042E-1, 3.914E-1, 3.854E-1, 3.764E-1, 3.695E-1, 3.570E-1, 3.458E-1, 3.355E-1, 3.260E-1, 3.091E-1, 2.944E-1, 2.651E-1, 2.429E-1, 2.112E-1, 1.893E-1, 1.729E-1, 1.599E-1, 1.405E-1, 1.263E-1, 1.129E-1, 1.027E-1, 8.769E-2, 6.921E-2, 5.806E-2, 5.049E-2, 4.498E-2, 3.746E-2, 3.254E-2, 2.539E-2, 2.153E-2;...
6.084E+1, 1.676E+1, 6.863   , 2.007   , 9.329E-1, 5.766E-1, 4.195E-1, 2.933E-1, 2.476E-1, 2.092E-1, 1.960E-1, 1.838E-1, 1.763E-1, 1.703E-1, 1.651E-1, 1.562E-1, 1.486E-1, 1.336E-1, 1.224E-1, 1.064E-1, 9.535E-2, 8.707E-2, 8.054E-2, 7.076E-2, 6.362E-2, 5.688E-2, 5.173E-2, 4.422E-2, 3.503E-2, 2.949E-2, 2.577E-2, 2.307E-2, 1.940E-2, 1.703E-2, 1.363E-2, 1.183E-2;...
2.339E+2, 6.668E+1, 2.707E+1, 7.549   , 3.114   , 1.619   , 9.875E-1, 5.054E-1, 3.395E-1, 2.176E-1, 1.856E-1, 1.644E-1, 1.551E-1, 1.488E-1, 1.438E-1, 1.356E-1, 1.289E-1, 1.158E-1, 1.060E-1, 9.210E-2, 8.249E-2, 7.532E-2, 6.968E-2, 6.121E-2, 5.503E-2, 4.921E-2, 4.476E-2, 3.830E-2, 3.043E-2, 2.572E-2, 2.257E-2, 2.030E-2, 1.725E-2, 1.529E-2, 1.252E-2, 1.109E-2;...
6.041E+2, 1.797E+2, 7.469E+1, 2.127E+1, 8.685   , 4.369   , 2.527   , 1.124   , 6.466E-1, 3.070E-1, 2.251E-1, 1.792E-1, 1.640E-1, 1.554E-1, 1.493E-1, 1.401E-1, 1.328E-1, 1.190E-1, 1.089E-1, 9.463E-2, 8.471E-2, 7.739E-2, 7.155E-2, 6.286E-2, 5.652E-2, 5.054E-2, 4.597E-2, 3.938E-2, 3.138E-2, 2.664E-2, 2.347E-2, 2.121E-2, 1.819E-2, 1.627E-2, 1.361E-2, 1.227E-2;...
1.229E+3, 3.766E+2, 1.597E+2, 4.667E+1, 1.927E+1, 9.683   , 5.538   , 2.346   , 1.255   , 4.827E-1, 3.014E-1, 2.063E-1, 1.793E-1, 1.665E-1, 1.583E-1, 1.472E-1, 1.391E-1, 1.243E-1, 1.136E-1, 9.862E-2, 8.834E-2, 8.065E-2, 7.460E-2, 6.549E-2, 5.890E-2, 5.266E-2, 4.791E-2, 4.108E-2, 3.284E-2, 2.798E-2, 2.476E-2, 2.248E-2, 1.945E-2, 1.755E-2, 1.495E-2, 1.368E-2;...
2.211E+3, 7.002E+2, 3.026E+2, 9.033E+1, 3.778E+1, 1.912E+1, 1.095E+1, 4.576   , 2.373   , 8.071E-1, 4.420E-1, 2.562E-1, 2.076E-1, 1.871E-1, 1.753E-1, 1.610E-1, 1.514E-1, 1.347E-1, 1.229E-1, 1.066E-1, 9.546E-2, 8.715E-2, 8.058E-2, 7.076E-2, 6.361E-2, 5.690E-2, 5.179E-2, 4.442E-2, 3.562E-2, 3.047E-2, 2.708E-2, 2.469E-2, 2.154E-2, 1.959E-2, 1.698E-2, 1.575E-2;...
3.311E+3, 1.083E+3, 4.769E+2, 1.456E+2, 6.166E+1, 3.144E+1, 1.809E+1, 7.562   , 3.879   , 1.236   , 6.178E-1, 3.066E-1, 2.288E-1, 1.980E-1, 1.817E-1, 1.639E-1, 1.529E-1, 1.353E-1, 1.233E-1, 1.068E-1, 9.557E-2, 8.719E-2, 8.063E-2, 7.081E-2, 6.364E-2, 5.693E-2, 5.180E-2, 4.450E-2, 3.579E-2, 3.073E-2, 2.742E-2, 2.511E-2, 2.209E-2, 2.024E-2, 1.782E-2, 1.673E-2;...
4.590E+3, 1.549E+3, 6.949E+2, 2.171E+2, 9.315E+1, 4.790E+1, 2.770E+1, 1.163E+1, 5.952   , 1.836   , 8.651E-1, 3.779E-1, 2.585E-1, 2.132E-1, 1.907E-1, 1.678E-1, 1.551E-1, 1.361E-1, 1.237E-1, 1.070E-1, 9.566E-2, 8.729E-2, 8.070E-2, 7.087E-2, 6.372E-2, 5.697E-2, 5.185E-2, 4.459E-2, 3.597E-2, 3.100E-2, 2.777E-2, 2.552E-2, 2.263E-2, 2.089E-2, 1.866E-2, 1.770E-2;...
5.649E+3, 1.979E+3, 9.047E+2, 2.888E+2, 1.256E+2, 6.514E+1, 3.789E+1, 1.602E+1, 8.205   , 2.492   , 1.133   , 4.487E-1, 2.828E-1, 2.214E-1, 1.920E-1, 1.639E-1, 1.496E-1, 1.298E-1, 1.176E-1, 1.015E-1, 9.073E-2, 8.274E-2, 7.649E-2, 6.717E-2, 6.037E-2, 5.399E-2, 4.915E-2, 4.228E-2, 3.422E-2, 2.960E-2, 2.663E-2, 2.457E-2, 2.195E-2, 2.039E-2, 1.846E-2, 1.769E-2;...
7.409E+3, 2.666E+3, 1.243E+3, 4.051E+2, 1.785E+2, 9.339E+1, 5.467E+1, 2.328E+1, 1.197E+1, 3.613   , 1.606   , 5.923E-1, 3.473E-1, 2.579E-1, 2.161E-1, 1.781E-1, 1.600E-1, 1.370E-1, 1.236E-1, 1.064E-1, 9.502E-2, 8.664E-2, 8.006E-2, 7.029E-2, 6.316E-2, 5.646E-2, 5.145E-2, 4.430E-2, 3.594E-2, 3.122E-2, 2.818E-2, 2.610E-2, 2.348E-2, 2.197E-2, 2.013E-2, 1.946E-2;...
6.542E+2, 3.194E+3, 1.521E+3, 5.070E+2, 2.261E+2, 1.194E+2, 7.030E+1, 3.018E+1, 1.557E+1, 4.694   , 2.057   , 7.197E-1, 3.969E-1, 2.804E-1, 2.268E-1, 1.796E-1, 1.585E-1, 1.335E-1, 1.199E-1, 1.029E-1, 9.185E-2, 8.372E-2, 7.736E-2, 6.788E-2, 6.100E-2, 5.454E-2, 4.968E-2, 4.282E-2, 3.487E-2, 3.037E-2, 2.753E-2, 2.559E-2, 2.319E-2, 2.181E-2, 2.023E-2, 1.970E-2;...
9.225E+2, 4.004E+3, 1.932E+3, 6.585E+2, 2.974E+2, 1.583E+2, 9.381E+1, 4.061E+1, 2.105E+1, 6.358   , 2.763   , 9.306E-1, 4.881E-1, 3.292E-1, 2.570E-1, 1.951E-1, 1.686E-1, 1.394E-1, 1.245E-1, 1.065E-1, 9.492E-2, 8.647E-2, 7.988E-2, 7.008E-2, 6.296E-2, 5.629E-2, 5.129E-2, 4.426E-2, 3.613E-2, 3.159E-2, 2.873E-2, 2.681E-2, 2.445E-2, 2.313E-2, 2.168E-2, 2.127E-2;...
1.185E+3, 4.022E+2, 2.263E+3, 7.880E+2, 3.605E+2, 1.934E+2, 1.153E+2, 5.033E+1, 2.623E+1, 7.955   , 3.441   , 1.128   , 5.685E-1, 3.681E-1, 2.778E-1, 2.018E-1, 1.704E-1, 1.378E-1, 1.223E-1, 1.042E-1, 9.276E-2, 8.445E-2, 7.802E-2, 6.841E-2, 6.146E-2, 5.496E-2, 5.006E-2, 4.324E-2, 3.541E-2, 3.106E-2, 2.836E-2, 2.655E-2, 2.437E-2, 2.318E-2, 2.195E-2, 2.168E-2;...
1.570E+3, 5.355E+2, 2.777E+3, 9.784E+2, 4.529E+2, 2.450E+2, 1.470E+2, 6.468E+1, 3.389E+1, 1.034E+1, 4.464   , 1.436   , 7.012E-1, 4.385E-1, 3.207E-1, 2.228E-1, 1.835E-1, 1.448E-1, 1.275E-1, 1.082E-1, 9.614E-2, 8.748E-2, 8.077E-2, 7.082E-2, 6.361E-2, 5.688E-2, 5.183E-2, 4.480E-2, 3.678E-2, 3.240E-2, 2.967E-2, 2.788E-2, 2.574E-2, 2.462E-2, 2.352E-2, 2.338E-2;...
1.913E+3, 6.547E+2, 3.018E+2, 1.118E+3, 5.242E+2, 2.860E+2, 1.726E+2, 7.660E+1, 4.035E+1, 1.239E+1, 5.352   , 1.700   , 8.096E-1, 4.916E-1, 3.494E-1, 2.324E-1, 1.865E-1, 1.432E-1, 1.250E-1, 1.055E-1, 9.359E-2, 8.511E-2, 7.854E-2, 6.884E-2, 6.182E-2, 5.526E-2, 5.039E-2, 4.358E-2, 3.590E-2, 3.172E-2, 2.915E-2, 2.747E-2, 2.552E-2, 2.452E-2, 2.364E-2, 2.363E-2;...
2.429E+3, 8.342E+2, 3.853E+2, 1.339E+3, 6.338E+2, 3.487E+2, 2.116E+2, 9.465E+1, 5.012E+1, 1.550E+1, 6.708   , 2.113   , 9.872E-1, 5.849E-1, 4.053E-1, 2.585E-1, 2.020E-1, 1.506E-1, 1.302E-1, 1.091E-1, 9.665E-2, 8.781E-2, 8.102E-2, 7.098E-2, 6.373E-2, 5.697E-2, 5.193E-2, 4.498E-2, 3.715E-2, 3.293E-2, 3.036E-2, 2.872E-2, 2.682E-2, 2.589E-2, 2.517E-2, 2.529E-2;...
2.832E+3, 9.771E+2, 4.520E+2, 1.473E+3, 7.037E+2, 3.901E+2, 2.384E+2, 1.075E+2, 5.725E+1, 1.784E+1, 7.739   , 2.425   , 1.117   , 6.483E-1, 4.395E-1, 2.696E-1, 2.050E-1, 1.480E-1, 1.266E-1, 1.054E-1, 9.311E-2, 8.453E-2, 7.795E-2, 6.826E-2, 6.128E-2, 5.478E-2, 4.994E-2, 4.328E-2, 3.585E-2, 3.188E-2, 2.950E-2, 2.798E-2, 2.628E-2, 2.549E-2, 2.496E-2, 2.520E-2;...
3.184E+3, 1.105E+3, 5.120E+2, 1.703E+2, 7.572E+2, 4.225E+2, 2.593E+2, 1.180E+2, 6.316E+1, 1.983E+1, 8.629   , 2.697   , 1.228   , 7.012E-1, 4.664E-1, 2.760E-1, 2.043E-1, 1.427E-1, 1.205E-1, 9.953E-2, 8.776E-2, 7.958E-2, 7.335E-2, 6.419E-2, 5.762E-2, 5.150E-2, 4.695E-2, 4.074E-2, 3.384E-2, 3.019E-2, 2.802E-2, 2.667E-2, 2.517E-2, 2.451E-2, 2.418E-2, 2.453E-2;...
4.058E+3, 1.418E+3, 6.592E+2, 2.198E+2, 9.256E+2, 5.189E+2, 3.205E+2, 1.469E+2, 7.907E+1, 2.503E+1, 1.093E+1, 3.413   , 1.541   , 8.679E-1, 5.678E-1, 3.251E-1, 2.345E-1, 1.582E-1, 1.319E-1, 1.080E-1, 9.495E-2, 8.600E-2, 7.922E-2, 6.929E-2, 6.216E-2, 5.556E-2, 5.068E-2, 4.399E-2, 3.666E-2, 3.282E-2, 3.054E-2, 2.915E-2, 2.766E-2, 2.704E-2, 2.687E-2, 2.737E-2;...
4.867E+3, 1.714E+3, 7.999E+2, 2.676E+2, 1.218E+2, 6.026E+2, 3.731E+2, 1.726E+2, 9.341E+1, 2.979E+1, 1.306E+1, 4.080   , 1.830   , 1.019   , 6.578E-1, 3.656E-1, 2.571E-1, 1.674E-1, 1.376E-1, 1.116E-1, 9.783E-2, 8.851E-2, 8.148E-2, 7.122E-2, 6.388E-2, 5.709E-2, 5.207E-2, 4.524E-2, 3.780E-2, 3.395E-2, 3.170E-2, 3.035E-2, 2.892E-2, 2.839E-2, 2.838E-2, 2.903E-2;...
5.238E+3, 1.858E+3, 8.706E+2, 2.922E+2, 1.332E+2, 6.305E+2, 3.933E+2, 1.828E+2, 9.952E+1, 3.202E+1, 1.409E+1, 4.409   , 1.969   , 1.087   , 6.932E-1, 3.753E-1, 2.577E-1, 1.619E-1, 1.310E-1, 1.052E-1, 9.193E-2, 8.305E-2, 7.639E-2, 6.675E-2, 5.985E-2, 5.347E-2, 4.878E-2, 4.243E-2, 3.554E-2, 3.202E-2, 2.999E-2, 2.878E-2, 2.756E-2, 2.715E-2, 2.732E-2, 2.804E-2;...
5.869E+3, 2.096E+3, 9.860E+2, 3.323E+2, 1.517E+2, 6.838E+2, 4.323E+2, 2.023E+2, 1.107E+2, 3.587E+1, 1.585E+1, 4.972   , 2.214   , 1.213   , 7.661E-1, 4.052E-1, 2.721E-1, 1.649E-1, 1.314E-1, 1.043E-1, 9.081E-2, 8.191E-2, 7.529E-2, 6.572E-2, 5.891E-2, 5.263E-2, 4.801E-2, 4.180E-2, 3.512E-2, 3.173E-2, 2.982E-2, 2.868E-2, 2.759E-2, 2.727E-2, 2.762E-2, 2.844E-2;...
6.495E+3, 2.342E+3, 1.106E+3, 3.743E+2, 1.712E+2, 9.291E+1, 4.687E+2, 2.217E+2, 1.218E+2, 3.983E+1, 1.768E+1, 5.564   , 2.472   , 1.347   , 8.438E-1, 4.371E-1, 2.877E-1, 1.682E-1, 1.318E-1, 1.034E-1, 8.965E-2, 8.074E-2, 7.414E-2, 6.466E-2, 5.794E-2, 5.175E-2, 4.722E-2, 4.115E-2, 3.466E-2, 3.141E-2, 2.960E-2, 2.855E-2, 2.759E-2, 2.738E-2, 2.786E-2, 2.877E-2;...
7.405E+3, 2.694E+3, 1.277E+3, 4.339E+2, 1.988E+2, 1.080E+2, 5.160E+2, 2.513E+2, 1.386E+2, 4.571E+1, 2.038E+1, 6.434   , 2.856   , 1.550   , 9.639E-1, 4.905E-1, 3.166E-1, 1.788E-1, 1.378E-1, 1.067E-1, 9.213E-2, 8.281E-2, 7.598E-2, 6.620E-2, 5.930E-2, 5.295E-2, 4.832E-2, 4.213E-2, 3.559E-2, 3.235E-2, 3.057E-2, 2.956E-2, 2.869E-2, 2.855E-2, 2.920E-2, 3.026E-2;...
8.093E+3, 2.984E+3, 1.421E+3, 4.851E+2, 2.229E+2, 1.212E+2, 7.350E+1, 2.734E+2, 1.514E+2, 5.027E+1, 2.253E+1, 7.141   , 3.169   , 1.714   , 1.060   , 5.306E-1, 3.367E-1, 1.838E-1, 1.391E-1, 1.062E-1, 9.133E-2, 8.192E-2, 7.509E-2, 6.537E-2, 5.852E-2, 5.224E-2, 4.769E-2, 4.162E-2, 3.524E-2, 3.213E-2, 3.045E-2, 2.952E-2, 2.875E-2, 2.871E-2, 2.951E-2, 3.068E-2;...
9.085E+3, 3.399E+3, 1.626E+3, 5.576E+2, 2.567E+2, 1.398E+2, 8.484E+1, 3.056E+2, 1.706E+2, 5.708E+1, 2.568E+1, 8.176   , 3.629   , 1.958   , 1.205   , 5.952E-1, 3.717E-1, 1.964E-1, 1.460E-1, 1.099E-1, 9.400E-2, 8.414E-2, 7.704E-2, 6.699E-2, 5.995E-2, 5.350E-2, 4.883E-2, 4.265E-2, 3.621E-2, 3.312E-2, 3.146E-2, 3.057E-2, 2.991E-2, 2.994E-2, 3.092E-2, 3.224E-2;...
9.796E+3, 3.697E+3, 1.779E+3, 6.129E+2, 2.830E+2, 1.543E+2, 9.370E+1, 3.248E+2, 1.841E+2, 6.201E+1, 2.803E+1, 8.962   , 3.981   , 2.144   , 1.314   , 6.414E-1, 3.949E-1, 2.023E-1, 1.476E-1, 1.094E-1, 9.311E-2, 8.315E-2, 7.604E-2, 6.604E-2, 5.906E-2, 5.270E-2, 4.810E-2, 4.204E-2, 3.580E-2, 3.283E-2, 3.127E-2, 3.045E-2, 2.991E-2, 3.002E-2, 3.115E-2, 3.256E-2;...
9.855E+3, 4.234E+3, 2.049E+3, 7.094E+2, 3.282E+2, 1.793E+2, 1.090E+2, 4.952E+1, 2.090E+2, 7.081E+1, 3.220E+1, 1.034E+1, 4.600   , 2.474   , 1.512   , 7.306E-1, 4.440E-1, 2.208E-1, 1.582E-1, 1.154E-1, 9.765E-2, 8.698E-2, 7.944E-2, 6.891E-2, 6.160E-2, 5.494E-2, 5.015E-2, 4.387E-2, 3.745E-2, 3.444E-2, 3.289E-2, 3.210E-2, 3.164E-2, 3.185E-2, 3.320E-2, 3.476E-2;...
1.057E+4, 4.418E+3, 2.154E+3, 7.488E+2, 3.473E+2, 1.899E+2, 1.156E+2, 5.255E+1, 2.159E+2, 7.405E+1, 3.379E+1, 1.092E+1, 4.862   , 2.613   , 1.593   , 7.630E-1, 4.584E-1, 2.217E-1, 1.559E-1, 1.119E-1, 9.413E-2, 8.362E-2, 7.625E-2, 6.605E-2, 5.901E-2, 5.261E-2, 4.803E-2, 4.205E-2, 3.599E-2, 3.318E-2, 3.177E-2, 3.108E-2, 3.074E-2, 3.103E-2, 3.247E-2, 3.408E-2;...
1.553E+3, 4.825E+3, 2.375E+3, 8.311E+2, 3.865E+2, 2.118E+2, 1.290E+2, 5.875E+1, 2.331E+2, 8.117E+1, 3.719E+1, 1.207E+1, 5.384   , 2.892   , 1.760   , 8.364E-1, 4.973E-1, 2.341E-1, 1.617E-1, 1.141E-1, 9.539E-2, 8.450E-2, 7.695E-2, 6.656E-2, 5.941E-2, 5.296E-2, 4.834E-2, 4.235E-2, 3.634E-2, 3.360E-2, 3.225E-2, 3.160E-2, 3.138E-2, 3.175E-2, 3.335E-2, 3.509E-2;...
1.697E+3, 5.087E+3, 2.515E+3, 8.857E+2, 4.130E+2, 2.266E+2, 1.382E+2, 6.302E+1, 3.421E+1, 8.537E+1, 3.928E+1, 1.281E+1, 5.726   , 3.076   , 1.868   , 8.823E-1, 5.197E-1, 2.387E-1, 1.619E-1, 1.123E-1, 9.325E-2, 8.236E-2, 7.487E-2, 6.466E-2, 5.767E-2, 5.139E-2, 4.692E-2, 4.113E-2, 3.538E-2, 3.280E-2, 3.156E-2, 3.099E-2, 3.086E-2, 3.130E-2, 3.300E-2, 3.479E-2;...
1.893E+3, 5.475E+3, 2.711E+3, 9.613E+2, 4.497E+2, 2.472E+2, 1.509E+2, 6.890E+1, 3.742E+1, 9.152E+1, 4.222E+1, 1.385E+1, 6.207   , 3.335   , 2.023   , 9.501E-1, 5.550E-1, 2.491E-1, 1.661E-1, 1.131E-1, 9.327E-2, 8.212E-2, 7.452E-2, 6.426E-2, 5.727E-2, 5.101E-2, 4.657E-2, 4.086E-2, 3.524E-2, 3.275E-2, 3.158E-2, 3.107E-2, 3.103E-2, 3.156E-2, 3.340E-2, 3.528E-2;...
2.121E+3, 5.227E+3, 2.931E+3, 1.049E+3, 4.920E+2, 2.709E+2, 1.656E+2, 7.573E+1, 4.115E+1, 9.856E+1, 4.564E+1, 1.506E+1, 6.760   , 3.635   , 2.203   , 1.030   , 5.971E-1, 2.622E-1, 1.719E-1, 1.150E-1, 9.414E-2, 8.259E-2, 7.483E-2, 6.440E-2, 5.735E-2, 5.106E-2, 4.661E-2, 4.093E-2, 3.539E-2, 3.296E-2, 3.187E-2, 3.141E-2, 3.146E-2, 3.207E-2, 3.405E-2, 3.603E-2;...
2.317E+3, 5.336E+3, 3.098E+3, 1.116E+3, 5.252E+2, 2.896E+2, 1.773E+2, 8.116E+1, 4.414E+1, 1.033E+2, 4.818E+1, 1.596E+1, 7.184   , 3.864   , 2.341   , 1.090   , 6.278E-1, 2.703E-1, 1.742E-1, 1.144E-1, 9.299E-2, 8.129E-2, 7.350E-2, 6.314E-2, 5.619E-2, 4.999E-2, 4.564E-2, 4.010E-2, 3.476E-2, 3.247E-2, 3.145E-2, 3.105E-2, 3.119E-2, 3.186E-2, 3.395E-2, 3.599E-2;...
2.624E+3, 1.002E+3, 3.407E+3, 1.231E+3, 5.815E+2, 3.213E+2, 1.968E+2, 9.026E+1, 4.912E+1, 1.119E+2, 5.266E+1, 1.753E+1, 7.900   , 4.264   , 2.582   , 1.198   , 6.861E-1, 2.899E-1, 1.838E-1, 1.186E-1, 9.563E-2, 8.328E-2, 7.515E-2, 6.443E-2, 5.728E-2, 5.094E-2, 4.650E-2, 4.089E-2, 3.552E-2, 3.327E-2, 3.229E-2, 3.194E-2, 3.218E-2, 3.293E-2, 3.521E-2, 3.738E-2;...
2.854E+3, 1.093E+3, 3.599E+3, 1.305E+3, 6.186E+2, 3.425E+2, 2.101E+2, 9.651E+1, 5.257E+1, 1.168E+2, 5.548E+1, 1.854E+1, 8.389   , 4.523   , 2.739   , 1.267   , 7.221E-1, 2.998E-1, 1.872E-1, 1.186E-1, 9.480E-2, 8.226E-2, 7.410E-2, 6.340E-2, 5.631E-2, 5.005E-2, 4.569E-2, 4.020E-2, 3.501E-2, 3.286E-2, 3.196E-2, 3.168E-2, 3.199E-2, 3.280E-2, 3.518E-2, 3.741E-2;...
3.174E+3, 1.219E+3, 3.410E+3, 1.418E+3, 6.748E+2, 3.744E+2, 2.300E+2, 1.058E+2, 5.766E+1, 1.909E+1, 5.980E+1, 2.009E+1, 9.112   , 4.918   , 2.979   , 1.375   , 7.799E-1, 3.187E-1, 1.960E-1, 1.219E-1, 9.670E-2, 8.360E-2, 7.510E-2, 6.412E-2, 5.689E-2, 5.053E-2, 4.613E-2, 4.061E-2, 3.545E-2, 3.335E-2, 3.250E-2, 3.227E-2, 3.267E-2, 3.357E-2, 3.610E-2, 3.845E-2;...
3.494E+3, 1.347E+3, 2.589E+3, 1.525E+3, 7.297E+2, 4.058E+2, 2.496E+2, 1.150E+2, 6.274E+1, 2.079E+1, 6.386E+1, 2.157E+1, 9.818   , 5.306   , 3.214   , 1.481   , 8.365E-1, 3.369E-1, 2.042E-1, 1.247E-1, 9.811E-2, 8.443E-2, 7.570E-2, 6.447E-2, 5.714E-2, 5.072E-2, 4.630E-2, 4.079E-2, 3.569E-2, 3.365E-2, 3.286E-2, 3.268E-2, 3.317E-2, 3.414E-2, 3.683E-2, 3.927E-2;...
3.864E+3, 1.493E+3, 7.422E+2, 1.654E+3, 7.936E+2, 4.424E+2, 2.725E+2, 1.258E+2, 6.871E+1, 2.279E+1, 6.855E+1, 2.330E+1, 1.065E+1, 5.764   , 3.493   , 1.607   , 9.047E-1, 3.595E-1, 2.149E-1, 1.289E-1, 1.006E-1, 8.613E-2, 7.703E-2, 6.546E-2, 5.795E-2, 5.141E-2, 4.692E-2, 4.137E-2, 3.628E-2, 3.428E-2, 3.355E-2, 3.341E-2, 3.399E-2, 3.504E-2, 3.790E-2, 4.048E-2;...
4.210E+3, 1.631E+3, 8.115E+2, 1.772E+3, 8.507E+2, 4.755E+2, 2.935E+2, 1.356E+2, 7.417E+1, 2.463E+1, 7.237E+1, 2.485E+1, 1.139E+1, 6.173   , 3.744   , 1.721   , 9.658E-1, 3.790E-1, 2.237E-1, 1.318E-1, 1.018E-1, 8.693E-2, 7.756E-2, 6.571E-2, 5.810E-2, 5.150E-2, 4.700E-2, 4.146E-2, 3.644E-2, 3.451E-2, 3.384E-2, 3.374E-2, 3.441E-2, 3.554E-2, 3.855E-2, 4.122E-2;...
4.600E+3, 1.786E+3, 8.893E+2, 1.906E+3, 9.164E+2, 5.134E+2, 3.172E+2, 1.469E+2, 8.038E+1, 2.672E+1, 7.712E+1, 2.666E+1, 1.223E+1, 6.644   , 4.032   , 1.852   , 1.037   , 4.023E-1, 2.344E-1, 1.357E-1, 1.040E-1, 8.831E-2, 7.858E-2, 6.642E-2, 5.866E-2, 5.196E-2, 4.741E-2, 4.185E-2, 3.686E-2, 3.498E-2, 3.436E-2, 3.432E-2, 3.508E-2, 3.628E-2, 3.944E-2, 4.224E-2;...
4.942E+3, 1.925E+3, 9.593E+2, 2.011E+3, 9.703E+2, 5.450E+2, 3.373E+2, 1.565E+2, 8.576E+1, 2.854E+1, 8.054E+1, 2.810E+1, 1.294E+1, 7.037   , 4.274   , 1.962   , 1.096   , 4.208E-1, 2.423E-1, 1.379E-1, 1.047E-1, 8.848E-2, 7.851E-2, 6.619E-2, 5.837E-2, 5.167E-2, 4.713E-2, 4.163E-2, 3.675E-2, 3.496E-2, 3.439E-2, 3.440E-2, 3.523E-2, 3.650E-2, 3.978E-2, 4.264E-2;...
5.356E+3, 2.092E+3, 1.044E+3, 1.862E+3, 1.036E+3, 5.836E+2, 3.619E+2, 1.683E+2, 9.231E+1, 3.076E+1, 1.410E+1, 2.993E+1, 1.381E+1, 7.521   , 4.571   , 2.099   , 1.169   , 4.449E-1, 2.534E-1, 1.418E-1, 1.066E-1, 8.968E-2, 7.935E-2, 6.674E-2, 5.876E-2, 5.197E-2, 4.740E-2, 4.189E-2, 3.705E-2, 3.532E-2, 3.482E-2, 3.487E-2, 3.578E-2, 3.712E-2, 4.055E-2, 4.353E-2;...
5.718E+3, 2.240E+3, 1.120E+3, 1.963E+3, 1.095E+3, 6.165E+2, 3.832E+2, 1.785E+2, 9.800E+1, 3.270E+1, 1.499E+1, 3.139E+1, 1.452E+1, 7.926   , 4.822   , 2.215   , 1.232   , 4.647E-1, 2.618E-1, 1.440E-1, 1.074E-1, 8.992E-2, 7.933E-2, 6.647E-2, 5.846E-2, 5.166E-2, 4.710E-2, 4.164E-2, 3.692E-2, 3.527E-2, 3.482E-2, 3.491E-2, 3.591E-2, 3.730E-2, 4.084E-2, 4.387E-2;...
6.169E+3, 2.426E+3, 1.214E+3, 4.441E+2, 1.170E+3, 6.589E+2, 4.101E+2, 1.915E+2, 1.053E+2, 3.518E+1, 1.613E+1, 3.330E+1, 1.544E+1, 8.448   , 5.147   , 2.365   , 1.314   , 4.916E-1, 2.742E-1, 1.485E-1, 1.097E-1, 9.134E-2, 8.035E-2, 6.711E-2, 5.894E-2, 5.204E-2, 4.744E-2, 4.197E-2, 3.728E-2, 3.568E-2, 3.529E-2, 3.542E-2, 3.650E-2, 3.797E-2, 4.166E-2, 4.481E-2;...
6.538E+3, 2.579E+3, 1.292E+3, 4.730E+2, 1.227E+3, 6.912E+2, 4.308E+2, 2.017E+2, 1.110E+2, 3.715E+1, 1.704E+1, 3.465E+1, 1.614E+1, 8.850   , 5.399   , 2.481   , 1.377   , 5.115E-1, 2.827E-1, 1.506E-1, 1.103E-1, 9.134E-2, 8.010E-2, 6.669E-2, 5.849E-2, 5.159E-2, 4.702E-2, 4.162E-2, 3.704E-2, 3.552E-2, 3.518E-2, 3.537E-2, 3.651E-2, 3.802E-2, 4.181E-2, 4.501E-2;...
7.039E+3, 2.790E+3, 1.401E+3, 5.136E+2, 1.305E+3, 7.385E+2, 4.610E+2, 2.164E+2, 1.193E+2, 3.998E+1, 1.836E+1, 3.668E+1, 1.720E+1, 9.444   , 5.766   , 2.651   , 1.470   , 5.426E-1, 2.972E-1, 1.560E-1, 1.131E-1, 9.321E-2, 8.153E-2, 6.766E-2, 5.921E-2, 5.217E-2, 4.754E-2, 4.209E-2, 3.754E-2, 3.606E-2, 3.577E-2, 3.601E-2, 3.723E-2, 3.882E-2, 4.276E-2, 4.609E-2;...
7.350E+3, 2.931E+3, 1.473E+3, 5.414E+2, 1.170E+3, 7.685E+2, 4.793E+2, 2.254E+2, 1.244E+2, 4.178E+1, 1.920E+1, 3.765E+1, 1.778E+1, 9.779   , 5.975   , 2.751   , 1.524   , 5.593E-1, 3.038E-1, 1.571E-1, 1.129E-1, 9.250E-2, 8.064E-2, 6.670E-2, 5.826E-2, 5.129E-2, 4.673E-2, 4.139E-2, 3.698E-2, 3.559E-2, 3.536E-2, 3.563E-2, 3.691E-2, 3.853E-2, 4.253E-2, 4.587E-2;...
7.809E+3, 3.131E+3, 1.578E+3, 5.808E+2, 1.231E+3, 8.134E+2, 5.072E+2, 2.391E+2, 1.321E+2, 4.445E+1, 2.044E+1, 3.949E+1, 1.873E+1, 1.030E+1, 6.306   , 2.907   , 1.609   , 5.876E-1, 3.167E-1, 1.614E-1, 1.149E-1, 9.371E-2, 8.138E-2, 6.707E-2, 5.849E-2, 5.144E-2, 4.684E-2, 4.151E-2, 3.715E-2, 3.582E-2, 3.564E-2, 3.596E-2, 3.730E-2, 3.898E-2, 4.311E-2, 4.654E-2;...
8.157E+3, 3.296E+3, 1.665E+3, 6.143E+2, 9.393E+2, 8.471E+2, 5.294E+2, 2.500E+2, 1.384E+2, 4.664E+1, 2.146E+1, 4.121E+1, 1.942E+1, 1.070E+1, 6.564   , 3.029   , 1.676   , 6.091E-1, 3.260E-1, 1.639E-1, 1.156E-1, 9.374E-2, 8.113E-2, 6.662E-2, 5.800E-2, 5.095E-2, 4.638E-2, 4.112E-2, 3.686E-2, 3.561E-2, 3.548E-2, 3.583E-2, 3.724E-2, 3.895E-2, 4.315E-2, 4.662E-2;...
8.582E+3, 3.491E+3, 1.767E+3, 6.536E+2, 3.169E+2, 8.846E+2, 5.569E+2, 2.631E+2, 1.459E+2, 4.923E+1, 2.268E+1, 7.631   , 2.027E+1, 1.120E+1, 6.879   , 3.176   , 1.758   , 6.361E-1, 3.381E-1, 1.677E-1, 1.172E-1, 9.453E-2, 8.153E-2, 6.670E-2, 5.797E-2, 5.086E-2, 4.628E-2, 4.105E-2, 3.686E-2, 3.567E-2, 3.559E-2, 3.598E-2, 3.745E-2, 3.921E-2, 4.351E-2, 4.704E-2;...
8.434E+3, 3.608E+3, 1.832E+3, 6.792E+2, 3.297E+2, 9.014E+2, 5.721E+2, 2.702E+2, 1.501E+2, 5.078E+1, 2.341E+1, 7.878   , 2.064E+1, 1.145E+1, 7.041   , 3.255   , 1.801   , 6.492E-1, 3.429E-1, 1.679E-1, 1.163E-1, 9.328E-2, 8.022E-2, 6.538E-2, 5.669E-2, 4.967E-2, 4.518E-2, 4.009E-2, 3.606E-2, 3.494E-2, 3.492E-2, 3.534E-2, 3.683E-2, 3.860E-2, 4.290E-2, 4.642E-2;...
9.096E+3, 3.919E+3, 1.997E+3, 7.420E+2, 3.607E+2, 8.430E+2, 6.173E+2, 2.922E+2, 1.626E+2, 5.512E+1, 2.543E+1, 8.561   , 2.210E+1, 1.232E+1, 7.579   , 3.510   , 1.942   , 6.978E-1, 3.663E-1, 1.771E-1, 1.217E-1, 9.701E-2, 8.313E-2, 6.749E-2, 5.841E-2, 5.111E-2, 4.647E-2, 4.124E-2, 3.716E-2, 3.607E-2, 3.608E-2, 3.655E-2, 3.815E-2, 4.002E-2, 4.455E-2, 4.823E-2;...
9.413E+3, 4.085E+3, 2.088E+3, 7.780E+2, 3.787E+2, 6.392E+2, 6.376E+2, 3.032E+2, 1.690E+2, 5.743E+1, 2.652E+1, 8.930   , 2.270E+1, 1.272E+1, 7.825   , 3.633   , 2.011   , 7.202E-1, 3.760E-1, 1.797E-1, 1.223E-1, 9.699E-2, 8.281E-2, 6.696E-2, 5.785E-2, 5.054E-2, 4.594E-2, 4.078E-2, 3.681E-2, 3.577E-2, 3.583E-2, 3.634E-2, 3.797E-2, 3.987E-2, 4.445E-2, 4.815E-2;...
9.365E+3, 4.335E+3, 2.226E+3, 8.319E+2, 4.055E+2, 2.303E+2, 6.711E+2, 3.214E+2, 1.793E+2, 6.104E+1, 2.822E+1, 9.507   , 2.381E+1, 1.340E+1, 8.248   , 3.836   , 2.124   , 7.589E-1, 3.941E-1, 1.863E-1, 1.257E-1, 9.912E-2, 8.431E-2, 6.789E-2, 5.854E-2, 5.108E-2, 4.641E-2, 4.121E-2, 3.725E-2, 3.625E-2, 3.635E-2, 3.689E-2, 3.860E-2, 4.057E-2, 4.529E-2, 4.910E-2;...
8.543E+3, 4.499E+3, 2.319E+3, 8.696E+2, 4.246E+2, 2.414E+2, 6.898E+2, 3.334E+2, 1.860E+2, 6.347E+1, 2.938E+1, 9.904   , 2.457E+1, 1.379E+1, 8.511   , 3.963   , 2.196   , 7.828E-1, 4.046E-1, 1.891E-1, 1.265E-1, 9.923E-2, 8.410E-2, 6.744E-2, 5.803E-2, 5.058E-2, 4.592E-2, 4.078E-2, 3.692E-2, 3.598E-2, 3.612E-2, 3.669E-2, 3.844E-2, 4.042E-2, 4.518E-2, 4.902E-2;...
9.087E+3, 4.772E+3, 2.464E+3, 9.267E+2, 4.531E+2, 2.578E+2, 6.319E+2, 3.529E+2, 1.967E+2, 6.731E+1, 3.119E+1, 1.052E+1, 2.579E+1, 1.447E+1, 8.962   , 4.177   , 2.315   , 8.239E-1, 4.239E-1, 1.961E-1, 1.301E-1, 1.015E-1, 8.570E-2, 6.843E-2, 5.876E-2, 5.110E-2, 4.640E-2, 4.122E-2, 3.737E-2, 3.646E-2, 3.664E-2, 3.726E-2, 3.907E-2, 4.113E-2, 4.603E-2, 4.996E-2;...
9.711E+3, 5.033E+3, 2.607E+3, 9.857E+2, 4.811E+2, 2.740E+2, 4.908E+2, 3.732E+2, 2.082E+2, 7.143E+1, 3.312E+1, 1.119E+1, 5.215   , 1.520E+1, 9.447   , 4.409   , 2.445   , 8.687E-1, 4.452E-1, 2.039E-1, 1.342E-1, 1.041E-1, 8.757E-2, 6.962E-2, 5.961E-2, 5.181E-2, 4.701E-2, 4.177E-2, 3.792E-2, 3.705E-2, 3.727E-2, 3.792E-2, 3.981E-2, 4.194E-2, 4.699E-2, 5.103E-2;...
1.058E+4, 5.090E+3, 2.768E+3, 1.047E+3, 5.131E+2, 2.924E+2, 5.145E+2, 3.950E+2, 2.209E+2, 7.597E+1, 3.526E+1, 1.192E+1, 5.557   , 1.599E+1, 9.977   , 4.664   , 2.588   , 9.180E-1, 4.687E-1, 2.126E-1, 1.389E-1, 1.071E-1, 8.976E-2, 7.105E-2, 6.070E-2, 5.268E-2, 4.779E-2, 4.244E-2, 3.858E-2, 3.774E-2, 3.800E-2, 3.870E-2, 4.067E-2, 4.288E-2, 4.809E-2, 5.226E-2;...
6.627E+3, 5.273E+3, 2.878E+3, 1.093E+3, 5.366E+2, 3.061E+2, 1.927E+2, 4.094E+2, 2.300E+2, 7.925E+1, 3.684E+1, 1.247E+1, 5.809   , 1.650E+1, 1.033E+1, 4.839   , 2.687   , 9.522E-1, 4.844E-1, 2.178E-1, 1.412E-1, 1.083E-1, 9.040E-2, 7.127E-2, 6.072E-2, 5.262E-2, 4.769E-2, 4.236E-2, 3.856E-2, 3.776E-2, 3.806E-2, 3.879E-2, 4.081E-2, 4.304E-2, 4.832E-2, 5.257E-2;...
2.056E+3, 5.553E+3, 3.048E+3, 1.162E+3, 5.709E+2, 3.260E+2, 2.053E+2, 4.314E+2, 2.440E+2, 8.411E+1, 3.916E+1, 1.327E+1, 6.181   , 1.735E+1, 1.087E+1, 5.107   , 2.840   , 1.005   , 5.098E-1, 2.273E-1, 1.462E-1, 1.115E-1, 9.276E-2, 7.280E-2, 6.188E-2, 5.350E-2, 4.849E-2, 4.308E-2, 3.925E-2, 3.849E-2, 3.882E-2, 3.959E-2, 4.169E-2, 4.401E-2, 4.946E-2, 5.380E-2;...
2.107E+3, 5.358E+3, 3.120E+3, 1.193E+3, 5.873E+2, 3.356E+2, 2.115E+2, 4.401E+2, 2.499E+2, 8.633E+1, 4.025E+1, 1.365E+1, 6.362   , 1.774E+1, 1.107E+1, 5.212   , 2.901   , 1.027   , 5.192E-1, 2.296E-1, 1.466E-1, 1.112E-1, 9.218E-2, 7.201E-2, 6.106E-2, 5.271E-2, 4.773E-2, 4.240E-2, 3.868E-2, 3.797E-2, 3.833E-2, 3.912E-2, 4.124E-2, 4.355E-2, 4.899E-2, 5.335E-2;...
2.216E+3, 5.624E+3, 3.278E+3, 1.256E+3, 6.193E+2, 3.542E+2, 2.234E+2, 3.989E+2, 2.629E+2, 9.087E+1, 4.242E+1, 1.441E+1, 6.716   , 1.850E+1, 1.155E+1, 5.455   , 3.040   , 1.076   , 5.425E-1, 2.380E-1, 1.509E-1, 1.139E-1, 9.404E-2, 7.312E-2, 6.186E-2, 5.331E-2, 4.822E-2, 4.285E-2, 3.913E-2, 3.845E-2, 3.885E-2, 3.967E-2, 4.185E-2, 4.422E-2, 4.980E-2, 5.423E-2;...
2.291E+3, 5.041E+3, 3.360E+3, 1.292E+3, 6.380E+2, 3.653E+2, 2.305E+2, 4.068E+2, 2.693E+2, 9.335E+1, 4.363E+1, 1.484E+1, 6.920   , 3.859   , 1.175E+1, 5.573   , 3.109   , 1.100   , 5.534E-1, 2.410E-1, 1.517E-1, 1.139E-1, 9.371E-2, 7.252E-2, 6.120E-2, 5.262E-2, 4.759E-2, 4.228E-2, 3.865E-2, 3.802E-2, 3.844E-2, 3.928E-2, 4.147E-2, 4.384E-2, 4.943E-2, 5.385E-2;...
2.396E+3, 5.314E+3, 3.507E+3, 1.354E+3, 6.697E+2, 3.838E+2, 2.423E+2, 3.133E+2, 2.815E+2, 9.802E+1, 4.588E+1, 1.563E+1, 7.288   , 4.064   , 1.223E+1, 5.826   , 3.253   , 1.151   , 5.775E-1, 2.498E-1, 1.562E-1, 1.167E-1, 9.562E-2, 7.365E-2, 6.196E-2, 5.321E-2, 4.808E-2, 4.272E-2, 3.908E-2, 3.848E-2, 3.894E-2, 3.981E-2, 4.206E-2, 4.449E-2, 5.017E-2, 5.470E-2;...
2.494E+3, 5.550E+3, 3.467E+3, 1.405E+3, 6.953E+2, 3.988E+2, 2.520E+2, 3.269E+2, 2.902E+2, 1.016E+2, 4.765E+1, 1.625E+1, 7.582   , 4.227   , 1.259E+1, 6.012   , 3.360   , 1.189   , 5.953E-1, 2.558E-1, 1.590E-1, 1.181E-1, 9.644E-2, 7.393E-2, 6.204E-2, 5.318E-2, 4.801E-2, 4.265E-2, 3.905E-2, 3.848E-2, 3.896E-2, 3.986E-2, 4.214E-2, 4.460E-2, 5.033E-2, 5.492E-2;...
2.616E+3, 5.847E+3, 3.590E+3, 1.465E+3, 7.264E+2, 4.170E+2, 2.636E+2, 1.271E+2, 3.012E+2, 1.060E+2, 4.980E+1, 1.701E+1, 7.940   , 4.425   , 1.309E+1, 6.244   , 3.492   , 1.236   , 6.178E-1, 2.639E-1, 1.629E-1, 1.205E-1, 9.800E-2, 7.477E-2, 6.257E-2, 5.351E-2, 4.828E-2, 4.289E-2, 3.930E-2, 3.877E-2, 3.927E-2, 4.020E-2, 4.253E-2, 4.504E-2, 5.088E-2, 5.551E-2;...
2.748E+3, 6.069E+3, 3.523E+3, 1.526E+3, 7.587E+2, 4.359E+2, 2.757E+2, 1.330E+2, 3.129E+2, 1.106E+2, 5.204E+1, 1.780E+1, 8.315   , 4.634   , 1.362E+1, 6.478   , 3.628   , 1.285   , 6.415E-1, 2.724E-1, 1.672E-1, 1.230E-1, 9.968E-2, 7.569E-2, 6.317E-2, 5.392E-2, 4.860E-2, 4.317E-2, 3.958E-2, 3.907E-2, 3.961E-2, 4.056E-2, 4.294E-2, 4.549E-2, 5.141E-2, 5.613E-2;...
2.899E+3, 3.937E+3, 3.686E+3, 1.594E+3, 7.946E+2, 4.569E+2, 2.892E+2, 1.397E+2, 2.830E+2, 1.157E+2, 5.453E+1, 1.868E+1, 8.735   , 4.867   , 1.409E+1, 6.741   , 3.780   , 1.340   , 6.682E-1, 2.822E-1, 1.722E-1, 1.261E-1, 1.018E-1, 7.693E-2, 6.404E-2, 5.455E-2, 4.912E-2, 4.362E-2, 4.002E-2, 3.954E-2, 4.010E-2, 4.108E-2, 4.352E-2, 4.612E-2, 5.216E-2, 5.698E-2;...
3.017E+3, 1.350E+3, 3.797E+3, 1.640E+3, 8.193E+2, 4.717E+2, 2.988E+2, 1.444E+2, 2.893E+2, 1.193E+2, 5.628E+1, 1.932E+1, 9.040   , 5.038   , 3.147   , 6.909   , 3.881   , 1.378   , 6.860E-1, 2.882E-1, 1.749E-1, 1.274E-1, 1.025E-1, 7.710E-2, 6.397E-2, 5.439E-2, 4.895E-2, 4.347E-2, 3.989E-2, 3.944E-2, 4.002E-2, 4.101E-2, 4.347E-2, 4.609E-2, 5.218E-2, 5.700E-2;...
3.187E+3, 1.424E+3, 3.452E+3, 1.710E+3, 8.560E+2, 4.934E+2, 3.129E+2, 1.513E+2, 2.211E+2, 1.247E+2, 5.881E+1, 2.023E+1, 9.472   , 5.279   , 3.297   , 7.161   , 4.033   , 1.433   , 7.130E-1, 2.981E-1, 1.799E-1, 1.305E-1, 1.046E-1, 7.829E-2, 6.478E-2, 5.496E-2, 4.941E-2, 4.385E-2, 4.028E-2, 3.985E-2, 4.045E-2, 4.147E-2, 4.398E-2, 4.664E-2, 5.282E-2, 5.773E-2;...
3.335E+3, 1.489E+3, 3.598E+3, 1.768E+3, 8.859E+2, 5.113E+2, 3.244E+2, 1.571E+2, 2.301E+2, 1.290E+2, 6.087E+1, 2.098E+1, 9.828   , 5.478   , 3.420   , 7.352   , 4.154   , 1.477   , 7.339E-1, 3.054E-1, 1.834E-1, 1.324E-1, 1.058E-1, 7.880E-2, 6.502E-2, 5.505E-2, 4.944E-2, 4.387E-2, 4.030E-2, 3.989E-2, 4.052E-2, 4.155E-2, 4.409E-2, 4.677E-2, 5.301E-2, 5.796E-2;...
3.510E+3, 1.566E+3, 3.771E+3, 1.838E+3, 9.222E+2, 5.328E+2, 3.382E+2, 1.639E+2, 2.379E+2, 1.340E+2, 6.334E+1, 2.187E+1, 1.025E+1, 5.717   , 3.569   , 7.587   , 4.302   , 1.531   , 7.598E-1, 3.149E-1, 1.881E-1, 1.352E-1, 1.076E-1, 7.981E-2, 6.567E-2, 5.545E-2, 4.977E-2, 4.413E-2, 4.057E-2, 4.018E-2, 4.082E-2, 4.188E-2, 4.446E-2, 4.717E-2, 5.350E-2, 5.852E-2;...
3.683E+3, 1.643E+3, 3.922E+3, 1.902E+3, 9.564E+2, 5.534E+2, 3.514E+2, 1.705E+2, 9.691E+1, 1.389E+2, 6.573E+1, 2.273E+1, 1.067E+1, 5.949   , 3.713   , 7.810   , 4.438   , 1.581   , 7.844E-1, 3.238E-1, 1.925E-1, 1.378E-1, 1.093E-1, 8.066E-2, 6.618E-2, 5.577E-2, 5.000E-2, 4.433E-2, 4.075E-2, 4.038E-2, 4.103E-2, 4.210E-2, 4.472E-2, 4.747E-2, 5.384E-2, 5.893E-2;...
3.872E+3, 1.729E+3, 3.773E+3, 1.972E+3, 9.943E+2, 5.759E+2, 3.660E+2, 1.778E+2, 1.011E+2, 1.440E+2, 6.835E+1, 2.367E+1, 1.112E+1, 6.206   , 3.872   , 8.069   , 4.587   , 1.637   , 8.119E-1, 3.339E-1, 1.976E-1, 1.409E-1, 1.114E-1, 8.179E-2, 6.688E-2, 5.627E-2, 5.038E-2, 4.463E-2, 4.105E-2, 4.068E-2, 4.137E-2, 4.246E-2, 4.511E-2, 4.790E-2, 5.437E-2, 5.953E-2;...
4.032E+3, 1.801E+3, 2.218E+3, 1.938E+3, 1.023E+3, 5.936E+2, 3.776E+2, 1.836E+2, 1.045E+2, 1.478E+2, 7.039E+1, 2.443E+1, 1.149E+1, 6.414   , 4.002   , 8.290   , 4.696   , 1.680   , 8.327E-1, 3.414E-1, 2.011E-1, 1.428E-1, 1.125E-1, 8.224E-2, 6.705E-2, 5.625E-2, 5.034E-2, 4.458E-2, 4.100E-2, 4.065E-2, 4.134E-2, 4.244E-2, 4.511E-2, 4.791E-2, 5.442E-2, 5.956E-2;...
4.243E+3, 1.898E+3, 1.032E+3, 2.011E+3, 1.063E+3, 6.178E+2, 3.935E+2, 1.914E+2, 1.090E+2, 1.530E+2, 7.317E+1, 2.546E+1, 1.199E+1, 6.693   , 4.176   , 8.585   , 4.855   , 1.740   , 8.628E-1, 3.525E-1, 2.068E-1, 1.463E-1, 1.149E-1, 8.357E-2, 6.794E-2, 5.688E-2, 5.083E-2, 4.501E-2, 4.139E-2, 4.104E-2, 4.174E-2, 4.286E-2, 4.558E-2, 4.844E-2, 5.502E-2, 6.024E-2;...
4.433E+3, 1.986E+3, 1.081E+3, 1.965E+3, 1.100E+3, 6.402E+2, 4.083E+2, 1.987E+2, 1.132E+2, 1.578E+2, 7.574E+1, 2.641E+1, 1.245E+1, 6.954   , 4.339   , 8.731   , 4.993   , 1.795   , 8.896E-1, 3.625E-1, 2.118E-1, 1.492E-1, 1.168E-1, 8.456E-2, 6.857E-2, 5.727E-2, 5.112E-2, 4.522E-2, 4.160E-2, 4.124E-2, 4.196E-2, 4.310E-2, 4.584E-2, 4.872E-2, 5.537E-2, 6.064E-2;...
4.652E+3, 2.089E+3, 1.137E+3, 2.049E+3, 1.144E+3, 6.661E+2, 4.253E+2, 2.072E+2, 1.181E+2, 1.637E+2, 7.883E+1, 2.752E+1, 1.298E+1, 7.256   , 4.528   , 2.185   , 5.158   , 1.860   , 9.214E-1, 3.744E-1, 2.180E-1, 1.530E-1, 1.194E-1, 8.603E-2, 6.953E-2, 5.794E-2, 5.167E-2, 4.570E-2, 4.201E-2, 4.166E-2, 4.239E-2, 4.355E-2, 4.633E-2, 4.926E-2, 5.598E-2, 6.136E-2;...
4.830E+3, 2.174E+3, 1.184E+3, 2.117E+3, 1.179E+3, 6.869E+2, 4.387E+2, 2.140E+2, 1.221E+2, 1.681E+2, 8.123E+1, 2.841E+1, 1.342E+1, 7.504   , 4.683   , 2.259   , 5.279   , 1.909   , 9.456E-1, 3.834E-1, 2.224E-1, 1.555E-1, 1.210E-1, 8.679E-2, 6.993E-2, 5.814E-2, 5.179E-2, 4.575E-2, 4.207E-2, 4.172E-2, 4.246E-2, 4.362E-2, 4.643E-2, 4.937E-2, 5.613E-2, 6.154E-2;...
5.008E+3, 2.259E+3, 1.231E+3, 2.188E+3, 1.212E+3, 7.068E+2, 4.518E+2, 2.208E+2, 1.260E+2, 1.497E+2, 8.361E+1, 2.929E+1, 1.385E+1, 7.751   , 4.838   , 2.332   , 5.398   , 1.957   , 9.696E-1, 3.923E-1, 2.267E-1, 1.580E-1, 1.226E-1, 8.751E-2, 7.031E-2, 5.832E-2, 5.187E-2, 4.581E-2, 4.210E-2, 4.175E-2, 4.248E-2, 4.366E-2, 4.647E-2, 4.943E-2, 5.622E-2, 6.162E-2;...
5.210E+3, 2.356E+3, 1.285E+3, 1.965E+3, 1.251E+3, 7.304E+2, 4.672E+2, 2.287E+2, 1.306E+2, 1.116E+2, 8.636E+1, 3.032E+1, 1.436E+1, 8.041   , 5.021   , 2.419   , 5.549   , 2.014   , 9.985E-1, 4.031E-1, 2.323E-1, 1.614E-1, 1.248E-1, 8.870E-2, 7.102E-2, 5.876E-2, 5.222E-2, 4.606E-2, 4.234E-2, 4.197E-2, 4.272E-2, 4.391E-2, 4.675E-2, 4.972E-2, 5.658E-2, 6.206E-2;...
5.441E+3, 2.468E+3, 1.348E+3, 2.053E+3, 1.296E+3, 7.580E+2, 4.855E+2, 2.378E+2, 1.360E+2, 1.160E+2, 8.952E+1, 3.152E+1, 1.495E+1, 8.379   , 5.233   , 2.522   , 5.739   , 2.082   , 1.033   , 4.163E-1, 2.391E-1, 1.656E-1, 1.277E-1, 9.036E-2, 7.214E-2, 5.955E-2, 5.285E-2, 4.659E-2, 4.279E-2, 4.242E-2, 4.317E-2, 4.437E-2, 4.725E-2, 5.025E-2, 5.721E-2, 6.276E-2;...
5.724E+3, 2.604E+3, 1.423E+3, 2.155E+3, 1.299E+3, 7.931E+2, 5.085E+2, 2.494E+2, 1.427E+2, 1.219E+2, 9.352E+1, 3.303E+1, 1.569E+1, 8.802   , 5.499   , 2.649   , 5.991   , 2.170   , 1.078   , 4.335E-1, 2.483E-1, 1.714E-1, 1.318E-1, 9.286E-2, 7.391E-2, 6.087E-2, 5.394E-2, 4.749E-2, 4.362E-2, 4.323E-2, 4.399E-2, 4.522E-2, 4.816E-2, 5.124E-2, 5.835E-2, 6.398E-2;...
5.868E+3, 2.731E+3, 1.495E+3, 2.250E+3, 1.275E+3, 8.251E+2, 5.299E+2, 2.601E+2, 1.490E+2, 1.279E+2, 9.704E+1, 3.442E+1, 1.638E+1, 9.196   , 5.748   , 2.769   , 6.174   , 2.249   , 1.118   , 4.491E-1, 2.565E-1, 1.765E-1, 1.354E-1, 9.495E-2, 7.538E-2, 6.189E-2, 5.479E-2, 4.821E-2, 4.424E-2, 4.383E-2, 4.461E-2, 4.585E-2, 4.883E-2, 5.195E-2, 5.918E-2, 6.493E-2;...
5.826E+3, 2.719E+3, 1.490E+3, 1.554E+3, 1.266E+3, 8.163E+2, 5.240E+2, 2.577E+2, 1.477E+2, 1.259E+2, 9.563E+1, 3.408E+1, 1.624E+1, 9.125   , 5.706   , 2.749   , 6.086   , 2.215   , 1.101   , 4.420E-1, 2.518E-1, 1.728E-1, 1.322E-1, 9.230E-2, 7.303E-2, 5.984E-2, 5.290E-2, 4.649E-2, 4.264E-2, 4.224E-2, 4.298E-2, 4.418E-2, 4.705E-2, 5.007E-2, 5.704E-2, 6.260E-2;...
6.238E+3, 2.846E+3, 1.562E+3, 1.994E+3, 1.322E+3, 8.489E+2, 5.451E+2, 2.684E+2, 1.539E+2, 5.573E+1, 9.933E+1, 3.550E+1, 1.694E+1, 9.525   , 5.959   , 2.871   , 1.655   , 2.295   , 1.141   , 4.577E-1, 2.601E-1, 1.780E-1, 1.358E-1, 9.444E-2, 7.450E-2, 6.087E-2, 5.376E-2, 4.721E-2, 4.324E-2, 4.283E-2, 4.358E-2, 4.479E-2, 4.769E-2, 5.077E-2, 5.786E-2, 6.350E-2;...
6.201E+3, 2.950E+3, 1.620E+3, 6.664E+2, 1.367E+3, 8.741E+2, 5.613E+2, 2.769E+2, 1.589E+2, 5.760E+1, 1.023E+2, 3.664E+1, 1.750E+1, 9.850   , 6.166   , 2.971   , 1.712   , 2.355   , 1.172   , 4.696E-1, 2.662E-1, 1.817E-1, 1.383E-1, 9.579E-2, 7.535E-2, 6.141E-2, 5.415E-2, 4.751E-2, 4.348E-2, 4.304E-2, 4.379E-2, 4.500E-2, 4.793E-2, 5.102E-2, 5.815E-2, 6.383E-2;...
6.469E+3, 3.082E+3, 1.696E+3, 6.981E+2, 1.426E+3, 8.687E+2, 5.829E+2, 2.878E+2, 1.653E+2, 6.002E+1, 1.062E+2, 3.811E+1, 1.823E+1, 1.027E+1, 6.433   , 3.100   , 1.786   , 2.434   , 1.213   , 4.859E-1, 2.749E-1, 1.872E-1, 1.421E-1, 9.804E-2, 7.686E-2, 6.251E-2, 5.504E-2, 4.822E-2, 4.409E-2, 4.363E-2, 4.438E-2, 4.560E-2, 4.856E-2, 5.171E-2, 5.895E-2, 6.473E-2;...
6.614E+3, 3.161E+3, 1.742E+3, 7.180E+2, 1.253E+3, 8.878E+2, 5.945E+2, 2.939E+2, 1.689E+2, 6.141E+1, 9.368E+1, 3.892E+1, 1.865E+1, 1.052E+1, 6.592   , 3.178   , 1.830   , 2.472   , 1.234   , 4.939E-1, 2.789E-1, 1.895E-1, 1.435E-1, 9.861E-2, 7.709E-2, 6.251E-2, 5.498E-2, 4.812E-2, 4.396E-2, 4.347E-2, 4.421E-2, 4.542E-2, 4.836E-2, 5.149E-2, 5.871E-2, 6.447E-2;...
6.530E+3, 3.327E+3, 1.834E+3, 7.558E+2, 1.315E+3, 8.759E+2, 6.217E+2, 3.074E+2, 1.769E+2, 6.442E+1, 7.025E+1, 4.077E+1, 1.957E+1, 1.105E+1, 6.929   , 3.342   , 1.924   , 2.575   , 1.288   , 5.152E-1, 2.904E-1, 1.969E-1, 1.487E-1, 1.018E-1, 7.937E-2, 6.420E-2, 5.637E-2, 4.927E-2, 4.496E-2, 4.445E-2, 4.519E-2, 4.641E-2, 4.943E-2, 5.261E-2, 6.000E-2, 6.592E-2;...
6.626E+3, 3.382E+3, 1.865E+3, 7.692E+2, 1.329E+3, 8.891E+2, 6.284E+2, 3.108E+2, 1.791E+2, 6.528E+1, 7.106E+1, 4.128E+1, 1.983E+1, 1.121E+1, 7.035   , 3.395   , 1.954   , 2.591   , 1.298   , 5.192E-1, 2.922E-1, 1.976E-1, 1.490E-1, 1.016E-1, 7.896E-2, 6.370E-2, 5.587E-2, 4.878E-2, 4.447E-2, 4.392E-2, 4.463E-2, 4.583E-2, 4.879E-2, 5.195E-2, 5.927E-2, 6.512E-2;...
6.950E+3, 3.490E+3, 1.960E+3, 8.090E+2, 1.390E+3, 9.320E+2, 6.570E+2, 3.250E+2, 1.870E+2, 6.840E+1, 7.450E+1, 4.310E+1, 2.080E+1, 1.180E+1, 7.390   , 3.570   , 2.050   , 2.700   , 1.350   , 5.410E-1, 3.040E-1, 2.050E-1, 1.540E-1, 1.050E-1, 8.130E-2, 6.540E-2, 5.730E-2, 4.990E-2, 4.550E-2, 4.490E-2, 4.560E-2, 4.680E-2, 4.980E-2, 5.300E-2, 6.050E-2, 6.650E-2;...
7.190E+3, 3.620E+3, 2.040E+3, 8.390E+2, 1.430E+3, 9.650E+2, 6.760E+2, 3.350E+2, 1.940E+2, 7.060E+1, 7.710E+1, 4.450E+1, 2.150E+1, 1.220E+1, 7.650   , 3.700   , 2.130   , 2.770   , 1.390   , 5.570E-1, 3.120E-1, 2.100E-1, 1.580E-1, 1.070E-1, 8.260E-2, 6.630E-2, 5.800E-2, 5.050E-2, 4.590E-2, 4.530E-2, 4.600E-2, 4.720E-2, 5.020E-2, 5.350E-2, 6.100E-2, 6.710E-2;...
7.370E+3, 3.730E+3, 2.100E+3, 8.640E+2, 1.050E+3, 9.900E+2, 6.640E+2, 3.430E+2, 1.980E+2, 7.240E+1, 7.940E+1, 4.550E+1, 2.200E+1, 1.250E+1, 7.860E+0, 3.800E+0, 2.190E+0, 2.820E+0, 1.420E+0, 5.680E-1, 3.180E-1, 2.140E-1, 1.600E-1, 1.080E-1, 8.330E-2, 6.670E-2, 5.820E-2, 5.060E-2, 4.600E-2, 4.530E-2, 4.600E-2, 4.710E-2, 5.020E-2, 5.340E-2, 6.100E-2, 6.700E-2;...
7.540E+3, 3.830E+3, 2.150E+3, 8.890E+2, 1.040E+3, 1.020E+3, 6.790E+2, 3.510E+2, 2.030E+2, 7.410E+1, 8.140E+1, 4.650E+1, 2.260E+1, 1.280E+1, 8.070E+0, 3.910E+0, 2.250E+0, 2.870E+0, 1.440E+0, 5.790E-1, 3.240E-1, 2.180E-1, 1.630E-1, 1.090E-1, 8.410E-2, 6.710E-2, 5.850E-2, 5.080E-2, 4.610E-2, 4.530E-2, 4.600E-2, 4.710E-2, 5.010E-2, 5.340E-2, 6.100E-2, 6.700E-2;...
7.840E+3, 3.950E+3, 2.250E+3, 9.290E+2, 4.840E+2, 1.060E+3, 6.680E+2, 3.640E+2, 2.100E+2, 7.710E+1, 8.390E+1, 4.830E+1, 2.350E+1, 1.340E+1, 8.420E+0, 4.080E+0, 2.350E+0, 2.970E+0, 1.500E+0, 6.000E-1, 3.360E-1, 2.250E-1, 1.680E-1, 1.130E-1, 8.620E-2, 6.870E-2, 5.970E-2, 5.180E-2, 4.690E-2, 4.610E-2, 4.680E-2, 4.790E-2, 5.090E-2, 5.420E-2, 6.190E-2, 6.810E-2;...
7.890E+3, 4.060E+3, 2.310E+3, 9.540E+2, 4.970E+2, 9.270E+2, 6.850E+2, 3.720E+2, 2.150E+2, 7.890E+1, 8.580E+1, 4.920E+1, 2.400E+1, 1.370E+1, 8.640E+0, 4.190E+0, 2.410E+0, 3.030E+0, 1.520E+0, 6.110E-1, 3.420E-1, 2.290E-1, 1.710E-1, 1.140E-1, 8.700E-2, 6.910E-2, 6.000E-2, 5.200E-2, 4.700E-2, 4.620E-2, 4.680E-2, 4.790E-2, 5.090E-2, 5.420E-2, 6.190E-2, 6.810E-2;...
7.790E+3, 4.220E+3, 2.400E+3, 9.920E+2, 5.170E+2, 9.590E+2, 7.110E+2, 3.830E+2, 2.220E+2, 8.160E+1, 4.010E+1, 5.080E+1, 2.490E+1, 1.420E+1, 8.970E+0, 4.360E+0, 2.510E+0, 3.120E+0, 1.570E+0, 6.310E-1, 3.520E-1, 2.360E-1, 1.750E-1, 1.170E-1, 8.890E-2, 7.040E-2, 6.110E-2, 5.280E-2, 4.760E-2, 4.670E-2, 4.740E-2, 4.840E-2, 5.150E-2, 5.480E-2, 6.260E-2, 6.890E-2;...
7.130E+3, 4.320E+3, 2.460E+3, 1.010E+3, 5.290E+2, 9.770E+2, 7.250E+2, 3.890E+2, 2.260E+2, 8.310E+1, 4.090E+1, 5.170E+1, 2.540E+1, 1.450E+1, 9.160E+0, 4.450E+0, 2.570E+0, 3.140E+0, 1.590E+0, 6.400E-1, 3.570E-1, 2.390E-1, 1.770E-1, 1.180E-1, 8.940E-2, 7.070E-2, 6.120E-2, 5.280E-2, 4.750E-2, 4.660E-2, 4.720E-2, 4.820E-2, 5.120E-2, 5.450E-2, 6.230E-2, 6.860E-2;...
];
end

%--------------------------------------------------------------------------
% Hard-wire a table of x-ray mass energy absorption coefficients in cm^2/g. 
%-------------------------------------------------------------------------- 
if (Options==2 && isempty(meac))
meac = [ ...
6.820   , 1.752   , 6.643E-1, 1.693E-1, 6.549E-2, 3.278E-2, 1.996E-2, 1.160E-2, 9.849E-3, 1.102E-2, 1.355E-2, 1.863E-2, 2.315E-2, 2.709E-2, 3.053E-2, 3.620E-2, 4.063E-2, 4.813E-2, 5.254E-2, 5.695E-2, 5.860E-2, 5.900E-2, 5.875E-2, 5.739E-2, 5.556E-2, 5.311E-2, 5.075E-2, 4.650E-2, 3.992E-2, 3.523E-2, 3.174E-2, 2.905E-2, 2.515E-2, 2.247E-2, 1.837E-2, 1.606E-2, ;...
6.045E+1, 1.638E+1, 6.503   , 1.681   , 6.379E-1, 3.061E-1, 1.671E-1, 6.446E-2, 3.260E-2, 1.246E-2, 9.410E-3, 1.003E-2, 1.190E-2, 1.375E-2, 1.544E-2, 1.826E-2, 2.047E-2, 2.424E-2, 2.647E-2, 2.868E-2, 2.951E-2, 2.971E-2, 2.959E-2, 2.890E-2, 2.797E-2, 2.674E-2, 2.555E-2, 2.343E-2, 2.019E-2, 1.790E-2, 1.622E-2, 1.493E-2, 1.308E-2, 1.183E-2, 9.948E-3, 8.914E-3;...
2.335E+2, 6.629E+1, 2.672E+1, 7.249   , 2.840   , 1.364   , 7.477E-1, 2.888E-1, 1.387E-1, 3.910E-2, 1.885E-2, 1.138E-2, 1.131E-2, 1.237E-2, 1.361E-2, 1.588E-2, 1.776E-2, 2.098E-2, 2.290E-2, 2.481E-2, 2.552E-2, 2.569E-2, 2.559E-2, 2.499E-2, 2.419E-2, 2.312E-2, 2.210E-2, 2.028E-2, 1.753E-2, 1.561E-2, 1.422E-2, 1.316E-2, 1.167E-2, 1.066E-2, 9.182E-3, 8.385E-3;...
6.035E+2, 1.791E+2, 7.422E+1, 2.090E+1, 8.367   , 4.081   , 2.260   , 8.839E-1, 4.255E-1, 1.143E-1, 4.780E-2, 1.898E-2, 1.438E-2, 1.401E-2, 1.468E-2, 1.658E-2, 1.836E-2, 2.157E-2, 2.353E-2, 2.548E-2, 2.620E-2, 2.639E-2, 2.627E-2, 2.565E-2, 2.483E-2, 2.373E-2, 2.268E-2, 2.083E-2, 1.806E-2, 1.617E-2, 1.479E-2, 1.377E-2, 1.233E-2, 1.138E-2, 1.001E-2, 9.294E-3;...
1.228E+3, 3.759E+2, 1.591E+2, 4.617E+1, 1.886E+1, 9.332   , 5.223   , 2.072   , 1.006   , 2.698E-1, 1.084E-1, 3.506E-2, 2.084E-2, 1.737E-2, 1.680E-2, 1.785E-2, 1.940E-2, 2.255E-2, 2.453E-2, 2.654E-2, 2.731E-2, 2.749E-2, 2.737E-2, 2.671E-2, 2.586E-2, 2.472E-2, 2.362E-2, 2.171E-2, 1.889E-2, 1.698E-2, 1.562E-2, 1.461E-2, 1.322E-2, 1.232E-2, 1.104E-2, 1.039E-2;...
2.209E+3, 6.990E+2, 3.016E+2, 8.963E+1, 3.723E+1, 1.866E+1, 1.054E+1, 4.242   , 2.078   , 5.627E-1, 2.238E-1, 6.614E-2, 3.343E-2, 2.397E-2, 2.098E-2, 2.037E-2, 2.147E-2, 2.449E-2, 2.655E-2, 2.870E-2, 2.950E-2, 2.969E-2, 2.956E-2, 2.885E-2, 2.792E-2, 2.669E-2, 2.551E-2, 2.345E-2, 2.048E-2, 1.849E-2, 1.710E-2, 1.607E-2, 1.468E-2, 1.380E-2, 1.258E-2, 1.198E-2;...
3.306E+3, 1.080E+3, 4.755E+2, 1.447E+2, 6.094E+1, 3.086E+1, 1.759E+1, 7.170   , 3.545   , 9.715E-1, 3.867E-1, 1.099E-1, 5.051E-2, 3.217E-2, 2.548E-2, 2.211E-2, 2.231E-2, 2.472E-2, 2.665E-2, 2.873E-2, 2.952E-2, 2.969E-2, 2.956E-2, 2.886E-2, 2.792E-2, 2.669E-2, 2.550E-2, 2.347E-2, 2.057E-2, 1.867E-2, 1.734E-2, 1.639E-2, 1.512E-2, 1.434E-2, 1.332E-2, 1.285E-2;...
4.576E+3, 1.545E+3, 6.926E+2, 2.158E+2, 9.221E+1, 4.715E+1, 2.708E+1, 1.116E+1, 5.565   , 1.545   , 6.179E-1, 1.729E-1, 7.530E-2, 4.414E-2, 3.207E-2, 2.468E-2, 2.355E-2, 2.506E-2, 2.679E-2, 2.877E-2, 2.953E-2, 2.971E-2, 2.957E-2, 2.887E-2, 2.794E-2, 2.669E-2, 2.551E-2, 2.350E-2, 2.066E-2, 1.882E-2, 1.757E-2, 1.668E-2, 1.553E-2, 1.483E-2, 1.396E-2, 1.360E-2;...
5.615E+3, 1.969E+3, 9.005E+2, 2.870E+2, 1.244E+2, 6.424E+1, 3.716E+1, 1.548E+1, 7.776   , 2.186   , 8.796E-1, 2.451E-1, 1.036E-1, 5.747E-2, 3.903E-2, 2.676E-2, 2.394E-2, 2.417E-2, 2.554E-2, 2.729E-2, 2.800E-2, 2.815E-2, 2.801E-2, 2.734E-2, 2.645E-2, 2.527E-2, 2.416E-2, 2.226E-2, 1.964E-2, 1.797E-2, 1.685E-2, 1.607E-2, 1.508E-2, 1.451E-2, 1.382E-2, 1.357E-2;...
7.326E+3, 2.645E+3, 1.234E+3, 4.021E+2, 1.767E+2, 9.214E+1, 5.369E+1, 2.260E+1, 1.143E+1, 3.253   , 1.317   , 3.676E-1, 1.528E-1, 8.182E-2, 5.287E-2, 3.273E-2, 2.733E-2, 2.590E-2, 2.697E-2, 2.862E-2, 2.931E-2, 2.946E-2, 2.930E-2, 2.860E-2, 2.766E-2, 2.641E-2, 2.526E-2, 2.330E-2, 2.061E-2, 1.894E-2, 1.784E-2, 1.709E-2, 1.616E-2, 1.564E-2, 1.508E-2, 1.491E-2;...
6.522E+2, 3.151E+3, 1.504E+3, 5.023E+2, 2.238E+2, 1.178E+2, 6.915E+1, 2.941E+1, 1.499E+1, 4.313   , 1.759   , 4.928E-1, 2.031E-1, 1.063E-1, 6.625E-2, 3.761E-2, 2.931E-2, 2.579E-2, 2.635E-2, 2.771E-2, 2.833E-2, 2.845E-2, 2.830E-2, 2.760E-2, 2.669E-2, 2.549E-2, 2.437E-2, 2.249E-2, 1.997E-2, 1.842E-2, 1.743E-2, 1.675E-2, 1.595E-2, 1.552E-2, 1.508E-2, 1.496E-2;...
9.203E+2, 3.918E+3, 1.899E+3, 6.499E+2, 2.937E+2, 1.561E+2, 9.227E+1, 3.965E+1, 2.036E+1, 5.925   , 2.432   , 6.855E-1, 2.815E-1, 1.451E-1, 8.820E-2, 4.671E-2, 3.410E-2, 2.766E-2, 2.761E-2, 2.871E-2, 2.928E-2, 2.938E-2, 2.921E-2, 2.848E-2, 2.753E-2, 2.629E-2, 2.514E-2, 2.322E-2, 2.067E-2, 1.915E-2, 1.819E-2, 1.756E-2, 1.683E-2, 1.646E-2, 1.614E-2, 1.609E-2;...
1.183E+3, 4.001E+2, 2.204E+3, 7.732E+2, 3.545E+2, 1.902E+2, 1.133E+2, 4.918E+1, 2.543E+1, 7.487   , 3.094   , 8.778E-1, 3.601E-1, 1.840E-1, 1.099E-1, 5.511E-2, 3.794E-2, 2.827E-2, 2.745E-2, 2.816E-2, 2.862E-2, 2.868E-2, 2.851E-2, 2.778E-2, 2.686E-2, 2.565E-2, 2.451E-2, 2.266E-2, 2.024E-2, 1.882E-2, 1.795E-2, 1.739E-2, 1.678E-2, 1.650E-2, 1.631E-2, 1.633E-2;...
1.567E+3, 5.331E+2, 2.669E+3, 9.516E+2, 4.427E+2, 2.400E+2, 1.439E+2, 6.313E+1, 3.289E+1, 9.794   , 4.076   , 1.164   , 4.782E-1, 2.430E-1, 1.434E-1, 6.896E-2, 4.513E-2, 3.086E-2, 2.905E-2, 2.932E-2, 2.968E-2, 2.971E-2, 2.951E-2, 2.875E-2, 2.778E-2, 2.652E-2, 2.535E-2, 2.345E-2, 2.101E-2, 1.963E-2, 1.878E-2, 1.827E-2, 1.773E-2, 1.753E-2, 1.746E-2, 1.757E-2;...
1.910E+3, 6.522E+2, 2.996E+2, 1.074E+3, 5.079E+2, 2.782E+2, 1.682E+2, 7.457E+1, 3.912E+1, 1.179E+1, 4.939   , 1.422   , 5.850E-1, 2.965E-1, 1.735E-1, 8.083E-2, 5.068E-2, 3.188E-2, 2.899E-2, 2.870E-2, 2.892E-2, 2.891E-2, 2.869E-2, 2.793E-2, 2.698E-2, 2.575E-2, 2.462E-2, 2.279E-2, 2.049E-2, 1.921E-2, 1.846E-2, 1.801E-2, 1.760E-2, 1.747E-2, 1.754E-2, 1.771E-2;...
2.426E+3, 8.314E+2, 3.828E+2, 1.265E+3, 6.066E+2, 3.360E+2, 2.046E+2, 9.171E+1, 4.847E+1, 1.477E+1, 6.235   , 1.809   , 7.466E-1, 3.779E-1, 2.199E-1, 1.000E-1, 6.052E-2, 3.516E-2, 3.080E-2, 2.983E-2, 2.991E-2, 2.984E-2, 2.959E-2, 2.878E-2, 2.780E-2, 2.652E-2, 2.535E-2, 2.349E-2, 2.118E-2, 1.993E-2, 1.923E-2, 1.884E-2, 1.850E-2, 1.845E-2, 1.864E-2, 1.889E-2;...
2.829E+3, 9.742E+2, 4.494E+2, 1.361E+3, 6.626E+2, 3.711E+2, 2.282E+2, 1.034E+2, 5.510E+1, 1.700E+1, 7.227   , 2.114   , 8.756E-1, 4.433E-1, 2.570E-1, 1.148E-1, 6.745E-2, 3.639E-2, 3.067E-2, 2.898E-2, 2.887E-2, 2.874E-2, 2.847E-2, 2.767E-2, 2.671E-2, 2.549E-2, 2.436E-2, 2.258E-2, 2.043E-2, 1.931E-2, 1.871E-2, 1.839E-2, 1.819E-2, 1.824E-2, 1.861E-2, 1.899E-2;...
3.180E+3, 1.102E+3, 5.093E+2, 1.682E+2, 6.979E+2, 3.953E+2, 2.449E+2, 1.125E+2, 6.038E+1, 1.886E+1, 8.074   , 2.382   , 9.907E-1, 5.020E-1, 2.904E-1, 1.280E-1, 7.344E-2, 3.703E-2, 2.998E-2, 2.757E-2, 2.727E-2, 2.708E-2, 2.679E-2, 2.601E-2, 2.510E-2, 2.394E-2, 2.288E-2, 2.123E-2, 1.927E-2, 1.827E-2, 1.777E-2, 1.753E-2, 1.742E-2, 1.754E-2, 1.800E-2, 1.842E-2;...
4.053E+3, 1.415E+3, 6.563E+2, 2.174E+2, 8.304E+2, 4.752E+2, 2.974E+2, 1.383E+2, 7.490E+1, 2.370E+1, 1.023E+1, 3.045   , 1.272   , 6.454E-1, 3.730E-1, 1.628E-1, 9.161E-2, 4.346E-2, 3.378E-2, 3.015E-2, 2.959E-2, 2.929E-2, 2.895E-2, 2.807E-2, 2.707E-2, 2.581E-2, 2.467E-2, 2.290E-2, 2.084E-2, 1.985E-2, 1.935E-2, 1.914E-2, 1.910E-2, 1.928E-2, 1.983E-2, 2.029E-2;...
4.861E+3, 1.710E+3, 7.966E+2, 2.650E+2, 1.197E+2, 5.373E+2, 3.387E+2, 1.600E+2, 8.744E+1, 2.804E+1, 1.220E+1, 3.665   , 1.538   , 7.822E-1, 4.520E-1, 1.958E-1, 1.085E-1, 4.876E-2, 3.639E-2, 3.146E-2, 3.060E-2, 3.019E-2, 2.979E-2, 2.884E-2, 2.780E-2, 2.650E-2, 2.532E-2, 2.352E-2, 2.147E-2, 2.052E-2, 2.007E-2, 1.992E-2, 1.997E-2, 2.022E-2, 2.088E-2, 2.140E-2;...
5.231E+3, 1.853E+3, 8.672E+2, 2.896E+2, 1.311E+2, 5.436E+2, 3.474E+2, 1.661E+2, 9.175E+1, 2.988E+1, 1.311E+1, 3.975   , 1.677   , 8.552E-1, 4.944E-1, 2.132E-1, 1.167E-1, 4.998E-2, 3.586E-2, 2.997E-2, 2.887E-2, 2.837E-2, 2.795E-2, 2.703E-2, 2.603E-2, 2.480E-2, 2.370E-2, 2.202E-2, 2.016E-2, 1.933E-2, 1.897E-2, 1.887E-2, 1.900E-2, 1.929E-2, 2.000E-2, 2.052E-2;...
5.860E+3, 2.091E+3, 9.824E+2, 3.295E+2, 1.494E+2, 5.657E+2, 3.691E+2, 1.793E+2, 1.001E+2, 3.311E+1, 1.465E+1, 4.488   , 1.904   , 9.737E-1, 5.634E-1, 2.422E-1, 1.312E-1, 5.393E-2, 3.726E-2, 3.007E-2, 2.864E-2, 2.804E-2, 2.756E-2, 2.661E-2, 2.561E-2, 2.439E-2, 2.330E-2, 2.166E-2, 1.989E-2, 1.913E-2, 1.884E-2, 1.879E-2, 1.899E-2, 1.933E-2, 2.013E-2, 2.067E-2;...
6.483E+3, 2.336E+3, 1.102E+3, 3.713E+2, 1.688E+2, 9.089E+1, 3.841E+2, 1.908E+2, 1.077E+2, 3.627E+1, 1.620E+1, 5.015   , 2.140   , 1.098   , 6.364E-1, 2.731E-1, 1.469E-1, 5.821E-2, 3.879E-2, 3.019E-2, 2.843E-2, 2.770E-2, 2.717E-2, 2.618E-2, 2.518E-2, 2.396E-2, 2.289E-2, 2.130E-2, 1.961E-2, 1.892E-2, 1.869E-2, 1.869E-2, 1.896E-2, 1.937E-2, 2.022E-2, 2.079E-2;...
7.388E+3, 2.687E+3, 1.272E+3, 4.305E+2, 1.961E+2, 1.057E+2, 4.027E+2, 2.087E+2, 1.193E+2, 4.093E+1, 1.846E+1, 5.780   , 2.482   , 1.278   , 7.420E-1, 3.182E-1, 1.701E-1, 6.536E-2, 4.211E-2, 3.160E-2, 2.938E-2, 2.849E-2, 2.788E-2, 2.680E-2, 2.576E-2, 2.450E-2, 2.340E-2, 2.178E-2, 2.011E-2, 1.947E-2, 1.929E-2, 1.933E-2, 1.970E-2, 2.016E-2, 2.112E-2, 2.174E-2;...
8.069E+3, 2.975E+3, 1.415E+3, 4.815E+2, 2.201E+2, 1.188E+2, 7.150E+1, 2.177E+2, 1.262E+2, 4.414E+1, 2.015E+1, 6.382   , 2.757   , 1.425   , 8.294E-1, 3.558E-1, 1.894E-1, 7.085E-2, 4.421E-2, 3.196E-2, 2.932E-2, 2.827E-2, 2.760E-2, 2.647E-2, 2.541E-2, 2.416E-2, 2.307E-2, 2.149E-2, 1.989E-2, 1.933E-2, 1.920E-2, 1.930E-2, 1.973E-2, 2.025E-2, 2.127E-2, 2.193E-2;...
9.052E+3, 3.388E+3, 1.620E+3, 5.535E+2, 2.536E+2, 1.372E+2, 8.265E+1, 2.316E+2, 1.369E+2, 4.896E+1, 2.260E+1, 7.251   , 3.155   , 1.638   , 9.555E-1, 4.104E-1, 2.177E-1, 7.961E-2, 4.825E-2, 3.361E-2, 3.039E-2, 2.914E-2, 2.836E-2, 2.714E-2, 2.603E-2, 2.472E-2, 2.360E-2, 2.199E-2, 2.042E-2, 1.990E-2, 1.983E-2, 1.997E-2, 2.050E-2, 2.108E-2, 2.221E-2, 2.292E-2;...
9.750E+3, 3.682E+3, 1.771E+3, 6.084E+2, 2.796E+2, 1.515E+2, 9.138E+1, 2.322E+2, 1.413E+2, 5.178E+1, 2.421E+1, 7.873   , 3.450   , 1.799   , 1.052   , 4.528E-1, 2.397E-1, 8.597E-2, 5.071E-2, 3.406E-2, 3.034E-2, 2.890E-2, 2.805E-2, 2.677E-2, 2.564E-2, 2.434E-2, 2.323E-2, 2.165E-2, 2.016E-2, 1.971E-2, 1.969E-2, 1.988E-2, 2.048E-2, 2.110E-2, 2.230E-2, 2.303E-2;...
9.797E+3, 4.214E+3, 2.039E+3, 7.042E+2, 3.244E+2, 1.761E+2, 1.064E+2, 4.758E+1, 1.524E+2, 5.734E+1, 2.722E+1, 8.982   , 3.967   , 2.078   , 1.219   , 5.259E-1, 2.781E-1, 9.812E-2, 5.649E-2, 3.659E-2, 3.209E-2, 3.036E-2, 2.937E-2, 2.795E-2, 2.674E-2, 2.536E-2, 2.420E-2, 2.257E-2, 2.107E-2, 2.066E-2, 2.070E-2, 2.094E-2, 2.163E-2, 2.234E-2, 2.368E-2, 2.446E-2;...
1.049E+4, 4.393E+3, 2.142E+3, 7.430E+2, 3.432E+2, 1.866E+2, 1.128E+2, 5.054E+1, 1.484E+2, 5.788E+1, 2.788E+1, 9.349   , 4.163   , 2.192   , 1.290   , 5.581E-1, 2.949E-1, 1.027E-1, 5.781E-2, 3.617E-2, 3.121E-2, 2.933E-2, 2.826E-2, 2.681E-2, 2.562E-2, 2.428E-2, 2.316E-2, 2.160E-2, 2.023E-2, 1.989E-2, 1.998E-2, 2.027E-2, 2.100E-2, 2.174E-2, 2.309E-2, 2.387E-2;...
1.548E+3, 4.791E+3, 2.359E+3, 8.244E+2, 3.820E+2, 2.082E+2, 1.261E+2, 5.660E+1, 1.497E+2, 6.099E+1, 2.986E+1, 1.018E+1, 4.570   , 2.419   , 1.429   , 6.203E-1, 3.278E-1, 1.128E-1, 6.225E-2, 3.764E-2, 3.195E-2, 2.979E-2, 2.861E-2, 2.704E-2, 2.580E-2, 2.443E-2, 2.329E-2, 2.174E-2, 2.041E-2, 2.014E-2, 2.028E-2, 2.061E-2, 2.143E-2, 2.223E-2, 2.367E-2, 2.451E-2;...
1.692E+3, 5.044E+3, 2.496E+3, 8.782E+2, 4.082E+2, 2.229E+2, 1.352E+2, 6.082E+1, 3.250E+1, 6.135E+1, 3.059E+1, 1.061E+1, 4.810   , 2.560   , 1.518   , 6.613E-1, 3.497E-1, 1.193E-1, 6.463E-2, 3.782E-2, 3.156E-2, 2.920E-2, 2.793E-2, 2.631E-2, 2.506E-2, 2.371E-2, 2.260E-2, 2.110E-2, 1.986E-2, 1.966E-2, 1.985E-2, 2.021E-2, 2.108E-2, 2.191E-2, 2.339E-2, 2.425E-2;...
1.887E+3, 5.420E+3, 2.688E+3, 9.527E+2, 4.445E+2, 2.433E+2, 1.477E+2, 6.660E+1, 3.564E+1, 6.256E+1, 3.178E+1, 1.126E+1, 5.152   , 2.759   , 1.642   , 7.184E-1, 3.803E-1, 1.288E-1, 6.865E-2, 3.891E-2, 3.193E-2, 2.930E-2, 2.790E-2, 2.618E-2, 2.489E-2, 2.353E-2, 2.242E-2, 2.094E-2, 1.977E-2, 1.962E-2, 1.987E-2, 2.027E-2, 2.120E-2, 2.208E-2, 2.364E-2, 2.452E-2;...
2.116E+3, 5.164E+3, 2.902E+3, 1.039E+3, 4.862E+2, 2.667E+2, 1.622E+2, 7.330E+1, 3.928E+1, 6.371E+1, 3.310E+1, 1.198E+1, 5.537   , 2.983   , 1.783   , 7.837E-1, 4.155E-1, 1.399E-1, 7.350E-2, 4.043E-2, 3.261E-2, 2.966E-2, 2.813E-2, 2.628E-2, 2.494E-2, 2.355E-2, 2.243E-2, 2.096E-2, 1.984E-2, 1.974E-2, 2.005E-2, 2.049E-2, 2.148E-2, 2.241E-2, 2.404E-2, 2.495E-2;...
2.312E+3, 5.260E+3, 3.062E+3, 1.104E+3, 5.187E+2, 2.851E+2, 1.737E+2, 7.865E+1, 4.221E+1, 6.270E+1, 3.352E+1, 1.240E+1, 5.796   , 3.143   , 1.886   , 8.332E-1, 4.426E-1, 1.483E-1, 7.695E-2, 4.113E-2, 3.261E-2, 2.941E-2, 2.775E-2, 2.581E-2, 2.446E-2, 2.306E-2, 2.195E-2, 2.052E-2, 1.947E-2, 1.944E-2, 1.978E-2, 2.025E-2, 2.130E-2, 2.226E-2, 2.394E-2, 2.487E-2;...
2.618E+3, 9.964E+2, 3.360E+3, 1.216E+3, 5.740E+2, 3.163E+2, 1.930E+2, 8.756E+1, 4.706E+1, 6.336E+1, 3.502E+1, 1.328E+1, 6.267   , 3.431   , 2.068   , 9.185E-1, 4.890E-1, 1.634E-1, 8.378E-2, 4.360E-2, 3.398E-2, 3.036E-2, 2.851E-2, 2.640E-2, 2.496E-2, 2.350E-2, 2.237E-2, 2.092E-2, 1.991E-2, 1.995E-2, 2.036E-2, 2.090E-2, 2.207E-2, 2.314E-2, 2.507E-2, 2.618E-2;...
2.848E+3, 1.087E+3, 3.541E+3, 1.288E+3, 6.101E+2, 3.371E+2, 2.060E+2, 9.371E+1, 5.044E+1, 6.112E+1, 3.509E+1, 1.365E+1, 6.538   , 3.596   , 2.178   , 9.729E-1, 5.192E-1, 1.731E-1, 8.787E-2, 4.459E-2, 3.414E-2, 3.023E-2, 2.825E-2, 2.603E-2, 2.456E-2, 2.310E-2, 2.197E-2, 2.055E-2, 1.961E-2, 1.969E-2, 2.014E-2, 2.071E-2, 2.192E-2, 2.302E-2, 2.497E-2, 2.610E-2;...
3.168E+3, 1.213E+3, 3.347E+3, 1.397E+3, 6.648E+2, 3.682E+2, 2.255E+2, 1.028E+2, 5.541E+1, 1.775E+1, 3.581E+1, 1.436E+1, 6.963   , 3.858   , 2.349   , 1.056   , 5.649E-1, 1.881E-1, 9.465E-2, 4.690E-2, 3.531E-2, 3.098E-2, 2.878E-2, 2.638E-2, 2.484E-2, 2.332E-2, 2.217E-2, 2.074E-2, 1.983E-2, 1.996E-2, 2.045E-2, 2.105E-2, 2.231E-2, 2.343E-2, 2.538E-2, 2.645E-2;...
3.487E+3, 1.341E+3, 2.539E+3, 1.500E+3, 7.181E+2, 3.988E+2, 2.446E+2, 1.118E+2, 6.037E+1, 1.938E+1, 3.602E+1, 1.493E+1, 7.346   , 4.103   , 2.511   , 1.135   , 6.093E-1, 2.028E-1, 1.013E-1, 4.906E-2, 3.634E-2, 3.156E-2, 2.917E-2, 2.660E-2, 2.498E-2, 2.342E-2, 2.225E-2, 2.082E-2, 1.995E-2, 2.013E-2, 2.066E-2, 2.130E-2, 2.260E-2, 2.377E-2, 2.578E-2, 2.684E-2;...
3.857E+3, 1.487E+3, 7.365E+2, 1.623E+3, 7.798E+2, 4.344E+2, 2.670E+2, 1.224E+2, 6.619E+1, 2.130E+1, 3.620E+1, 1.558E+1, 7.790   , 4.388   , 2.699   , 1.228   , 6.616E-1, 2.203E-1, 1.093E-1, 5.186E-2, 3.779E-2, 3.249E-2, 2.986E-2, 2.708E-2, 2.537E-2, 2.375E-2, 2.254E-2, 2.109E-2, 2.026E-2, 2.048E-2, 2.107E-2, 2.174E-2, 2.311E-2, 2.432E-2, 2.638E-2, 2.746E-2;...
4.203E+3, 1.625E+3, 8.057E+2, 1.734E+3, 8.344E+2, 4.664E+2, 2.873E+2, 1.320E+2, 7.150E+1, 2.308E+1, 3.555E+1, 1.600E+1, 8.129   , 4.621   , 2.859   , 1.310   , 7.080E-1, 2.361E-1, 1.164E-1, 5.420E-2, 3.885E-2, 3.311E-2, 3.025E-2, 2.726E-2, 2.547E-2, 2.380E-2, 2.257E-2, 2.112E-2, 2.033E-2, 2.061E-2, 2.123E-2, 2.193E-2, 2.335E-2, 2.460E-2, 2.670E-2, 2.778E-2;...
4.592E+3, 1.780E+3, 8.832E+2, 1.859E+3, 8.970E+2, 5.027E+2, 3.103E+2, 1.429E+2, 7.754E+1, 2.508E+1, 3.493E+1, 1.648E+1, 8.511   , 4.884   , 3.040   , 1.403   , 7.608E-1, 2.542E-1, 1.247E-1, 5.705E-2, 4.026E-2, 3.396E-2, 3.085E-2, 2.764E-2, 2.575E-2, 2.402E-2, 2.276E-2, 2.130E-2, 2.054E-2, 2.086E-2, 2.153E-2, 2.227E-2, 2.375E-2, 2.503E-2, 2.718E-2, 2.828E-2;...
4.935E+3, 1.918E+3, 9.531E+2, 1.956E+3, 9.475E+2, 5.328E+2, 3.295E+2, 1.522E+2, 8.275E+1, 2.684E+1, 3.293E+1, 1.664E+1, 8.757   , 5.074   , 3.178   , 1.477   , 8.042E-1, 2.693E-1, 1.316E-1, 5.919E-2, 4.117E-2, 3.437E-2, 3.104E-2, 2.764E-2, 2.567E-2, 2.390E-2, 2.263E-2, 2.118E-2, 2.046E-2, 2.084E-2, 2.153E-2, 2.231E-2, 2.382E-2, 2.513E-2, 2.731E-2, 2.840E-2;...
5.348E+3, 2.085E+3, 1.037E+3, 1.805E+3, 1.009E+3, 5.694E+2, 3.530E+2, 1.635E+2, 8.908E+1, 2.898E+1, 1.290E+1, 1.693E+1, 9.075   , 5.312   , 3.348   , 1.568   , 8.572E-1, 2.880E-1, 1.403E-1, 6.214E-2, 4.257E-2, 3.521E-2, 3.159E-2, 2.797E-2, 2.589E-2, 2.406E-2, 2.276E-2, 2.129E-2, 2.061E-2, 2.103E-2, 2.178E-2, 2.258E-2, 2.415E-2, 2.549E-2, 2.774E-2, 2.884E-2;...
5.709E+3, 2.233E+3, 1.113E+3, 1.894E+3, 1.063E+3, 6.000E+2, 3.731E+2, 1.733E+2, 9.456E+1, 3.085E+1, 1.376E+1, 1.689E+1, 9.251   , 5.477   , 3.476   , 1.641   , 9.012E-1, 3.037E-1, 1.475E-1, 6.445E-2, 4.355E-2, 3.567E-2, 3.182E-2, 2.796E-2, 2.581E-2, 2.393E-2, 2.262E-2, 2.115E-2, 2.052E-2, 2.098E-2, 2.176E-2, 2.259E-2, 2.419E-2, 2.556E-2, 2.783E-2, 2.893E-2;...
6.160E+3, 2.419E+3, 1.207E+3, 4.382E+2, 1.132E+3, 6.396E+2, 3.985E+2, 1.857E+2, 1.015E+2, 3.322E+1, 1.484E+1, 1.699E+1, 9.520   , 5.706   , 3.649   , 1.737   , 9.581E-1, 3.241E-1, 1.571E-1, 6.776E-2, 4.517E-2, 3.663E-2, 3.247E-2, 2.834E-2, 2.608E-2, 2.413E-2, 2.278E-2, 2.131E-2, 2.071E-2, 2.122E-2, 2.204E-2, 2.290E-2, 2.456E-2, 2.597E-2, 2.831E-2, 2.943E-2;...
6.529E+3, 2.571E+3, 1.285E+3, 4.668E+2, 1.182E+3, 6.688E+2, 4.175E+2, 1.953E+2, 1.070E+2, 3.511E+1, 1.571E+1, 1.669E+1, 9.611   , 5.833   , 3.759   , 1.804   , 1.000   , 3.397E-1, 1.644E-1, 7.007E-2, 4.611E-2, 3.704E-2, 3.262E-2, 2.828E-2, 2.594E-2, 2.395E-2, 2.259E-2, 2.112E-2, 2.056E-2, 2.111E-2, 2.196E-2, 2.284E-2, 2.453E-2, 2.595E-2, 2.832E-2, 2.944E-2;...
7.029E+3, 2.782E+3, 1.393E+3, 5.073E+2, 1.252E+3, 7.122E+2, 4.457E+2, 2.092E+2, 1.149E+2, 3.780E+1, 1.695E+1, 1.660E+1, 9.869   , 6.066   , 3.938   , 1.907   , 1.062   , 3.622E-1, 1.751E-1, 7.392E-2, 4.803E-2, 3.822E-2, 3.347E-2, 2.881E-2, 2.632E-2, 2.425E-2, 2.284E-2, 2.135E-2, 2.082E-2, 2.142E-2, 2.232E-2, 2.324E-2, 2.499E-2, 2.647E-2, 2.889E-2, 3.005E-2;...
7.340E+3, 2.922E+3, 1.466E+3, 5.350E+2, 1.118E+3, 7.383E+2, 4.619E+2, 2.174E+2, 1.197E+2, 3.952E+1, 1.776E+1, 1.594E+1, 9.812   , 6.115   , 4.001   , 1.957   , 1.096   , 3.753E-1, 1.813E-1, 7.580E-2, 4.868E-2, 3.838E-2, 3.339E-2, 2.854E-2, 2.597E-2, 2.387E-2, 2.247E-2, 2.099E-2, 2.051E-2, 2.114E-2, 2.206E-2, 2.300E-2, 2.477E-2, 2.625E-2, 2.869E-2, 2.985E-2;...
7.797E+3, 3.122E+3, 1.570E+3, 5.743E+2, 1.170E+3, 7.780E+2, 4.871E+2, 2.301E+2, 1.269E+2, 4.206E+1, 1.893E+1, 1.553E+1, 9.911   , 6.262   , 4.135   , 2.043   , 1.150   , 3.960E-1, 1.913E-1, 7.923E-2, 5.031E-2, 3.932E-2, 3.398E-2, 2.883E-2, 2.615E-2, 2.398E-2, 2.254E-2, 2.105E-2, 2.060E-2, 2.127E-2, 2.223E-2, 2.321E-2, 2.502E-2, 2.653E-2, 2.903E-2, 3.020E-2;...
8.144E+3, 3.287E+3, 1.657E+3, 6.077E+2, 8.945E+2, 8.064E+2, 5.065E+2, 2.400E+2, 1.327E+2, 4.413E+1, 1.990E+1, 1.490E+1, 9.835   , 6.314   , 4.211   , 2.101   , 1.189   , 4.119E-1, 1.990E-1, 8.179E-2, 5.138E-2, 3.980E-2, 3.417E-2, 2.878E-2, 2.601E-2, 2.379E-2, 2.233E-2, 2.085E-2, 2.044E-2, 2.115E-2, 2.213E-2, 2.312E-2, 2.496E-2, 2.649E-2, 2.901E-2, 3.018E-2;...
8.568E+3, 3.481E+3, 1.759E+3, 6.469E+2, 3.113E+2, 8.377E+2, 5.305E+2, 2.518E+2, 1.396E+2, 4.657E+1, 2.105E+1, 6.755   , 9.789   , 6.400   , 4.311   , 2.173   , 1.237   , 4.312E-1, 2.084E-1, 8.504E-2, 5.288E-2, 4.061E-2, 3.465E-2, 2.896E-2, 2.608E-2, 2.378E-2, 2.230E-2, 2.081E-2, 2.043E-2, 2.118E-2, 2.219E-2, 2.321E-2, 2.509E-2, 2.664E-2, 2.920E-2, 3.038E-2;...
8.419E+3, 3.598E+3, 1.824E+3, 6.726E+2, 3.240E+2, 8.487E+2, 5.423E+2, 2.578E+2, 1.433E+2, 4.802E+1, 2.175E+1, 6.997   , 9.480   , 6.330   , 4.306   , 2.195   , 1.256   , 4.409E-1, 2.132E-1, 8.646E-2, 5.326E-2, 4.055E-2, 3.440E-2, 2.854E-2, 2.559E-2, 2.327E-2, 2.179E-2, 2.033E-2, 1.998E-2, 2.075E-2, 2.177E-2, 2.279E-2, 2.466E-2, 2.620E-2, 2.873E-2, 2.989E-2;...
9.078E+3, 3.908E+3, 1.988E+3, 7.351E+2, 3.548E+2, 7.903E+2, 5.822E+2, 2.777E+2, 1.548E+2, 5.208E+1, 2.363E+1, 7.622   , 9.616   , 6.573   , 4.518   , 2.331   , 1.342   , 4.742E-1, 2.295E-1, 9.257E-2, 5.650E-2, 4.267E-2, 3.598E-2, 2.962E-2, 2.646E-2, 2.399E-2, 2.243E-2, 2.092E-2, 2.059E-2, 2.142E-2, 2.250E-2, 2.357E-2, 2.553E-2, 2.714E-2, 2.980E-2, 3.101E-2;...
9.393E+3, 4.073E+3, 2.078E+3, 7.709E+2, 3.728E+2, 6.015E+2, 5.979E+2, 2.871E+2, 1.605E+2, 5.420E+1, 2.465E+1, 7.969   , 9.323   , 6.540   , 4.541   , 2.374   , 1.376   , 4.894E-1, 2.373E-1, 9.522E-2, 5.761E-2, 4.317E-2, 3.617E-2, 2.956E-2, 2.630E-2, 2.378E-2, 2.221E-2, 2.070E-2, 2.042E-2, 2.128E-2, 2.240E-2, 2.350E-2, 2.553E-2, 2.720E-2, 3.002E-2, 3.139E-2;...
9.343E+3, 4.322E+3, 2.216E+3, 8.246E+2, 3.994E+2, 2.251E+2, 6.253E+2, 3.029E+2, 1.697E+2, 5.753E+1, 2.623E+1, 8.502   , 9.185   , 6.618   , 4.653   , 2.464   , 1.438   , 5.154E-1, 2.502E-1, 1.000E-1, 6.003E-2, 4.463E-2, 3.717E-2, 3.014E-2, 2.671E-2, 2.408E-2, 2.246E-2, 2.091E-2, 2.064E-2, 2.153E-2, 2.268E-2, 2.380E-2, 2.583E-2, 2.750E-2, 3.026E-2, 3.152E-2;...
8.521E+3, 4.485E+3, 2.308E+3, 8.622E+2, 4.184E+2, 2.361E+2, 6.383E+2, 3.127E+2, 1.754E+2, 5.972E+1, 2.730E+1, 8.875   , 8.842   , 6.534   , 4.660   , 2.501   , 1.470   , 5.309E-1, 2.583E-1, 1.028E-1, 6.125E-2, 4.521E-2, 3.743E-2, 3.011E-2, 2.657E-2, 2.388E-2, 2.224E-2, 2.069E-2, 2.044E-2, 2.135E-2, 2.250E-2, 2.363E-2, 2.566E-2, 2.731E-2, 3.003E-2, 3.126E-2;...
9.061E+3, 4.755E+3, 2.453E+3, 9.190E+2, 4.468E+2, 2.524E+2, 5.819E+2, 3.290E+2, 1.847E+2, 6.321E+1, 2.897E+1, 9.444   , 8.608   , 6.560   , 4.756   , 2.587   , 1.532   , 5.576E-1, 2.718E-1, 1.079E-1, 6.382E-2, 4.677E-2, 3.849E-2, 3.073E-2, 2.700E-2, 2.418E-2, 2.249E-2, 2.090E-2, 2.068E-2, 2.161E-2, 2.280E-2, 2.395E-2, 2.602E-2, 2.769E-2, 3.043E-2, 3.163E-2;...
9.679E+3, 5.015E+3, 2.595E+3, 9.776E+2, 4.745E+2, 2.684E+2, 4.550E+2, 3.458E+2, 1.945E+2, 6.690E+1, 3.073E+1, 1.005E+1, 4.496   , 6.569   , 4.847   , 2.677   , 1.597   , 5.862E-1, 2.865E-1, 1.134E-1, 6.664E-2, 4.851E-2, 3.970E-2, 3.145E-2, 2.750E-2, 2.456E-2, 2.281E-2, 2.118E-2, 2.096E-2, 2.194E-2, 2.316E-2, 2.434E-2, 2.645E-2, 2.816E-2, 3.094E-2, 3.216E-2;...
1.054E+4, 5.069E+3, 2.755E+3, 1.038E+3, 5.061E+2, 2.865E+2, 4.739E+2, 3.634E+2, 2.052E+2, 7.094E+1, 3.267E+1, 1.072E+1, 4.803   , 6.567   , 4.942   , 2.773   , 1.667   , 6.172E-1, 3.025E-1, 1.195E-1, 6.980E-2, 5.047E-2, 4.107E-2, 3.229E-2, 2.811E-2, 2.503E-2, 2.320E-2, 2.152E-2, 2.131E-2, 2.233E-2, 2.359E-2, 2.481E-2, 2.697E-2, 2.872E-2, 3.155E-2, 3.277E-2;...
6.598E+3, 5.250E+3, 2.864E+3, 1.084E+3, 5.294E+2, 3.000E+2, 1.874E+2, 3.739E+2, 2.125E+2, 7.376E+1, 3.407E+1, 1.121E+1, 5.033   , 6.418   , 4.928   , 2.815   , 1.706   , 6.376E-1, 3.133E-1, 1.236E-1, 7.177E-2, 5.157E-2, 4.174E-2, 3.258E-2, 2.823E-2, 2.505E-2, 2.318E-2, 2.148E-2, 2.129E-2, 2.233E-2, 2.360E-2, 2.483E-2, 2.701E-2, 2.876E-2, 3.159E-2, 3.283E-2;...
2.046E+3, 5.526E+3, 3.031E+3, 1.152E+3, 5.633E+2, 3.196E+2, 1.998E+2, 3.908E+2, 2.240E+2, 7.799E+1, 3.614E+1, 1.193E+1, 5.366   , 6.363   , 4.989   , 2.904   , 1.776   , 6.699E-1, 3.302E-1, 1.301E-1, 7.517E-2, 5.369E-2, 4.323E-2, 3.349E-2, 2.889E-2, 2.553E-2, 2.360E-2, 2.184E-2, 2.166E-2, 2.273E-2, 2.405E-2, 2.532E-2, 2.754E-2, 2.933E-2, 3.222E-2, 3.344E-2;...
2.097E+3, 5.329E+3, 3.102E+3, 1.183E+3, 5.795E+2, 3.291E+2, 2.059E+2, 3.951E+2, 2.278E+2, 7.972E+1, 3.706E+1, 1.227E+1, 5.532   , 6.094   , 4.872   , 2.893   , 1.786   , 6.806E-1, 3.366E-1, 1.326E-1, 7.620E-2, 5.411E-2, 4.334E-2, 3.333E-2, 2.862E-2, 2.522E-2, 2.325E-2, 2.150E-2, 2.133E-2, 2.241E-2, 2.372E-2, 2.498E-2, 2.719E-2, 2.896E-2, 3.180E-2, 3.302E-2;...
2.206E+3, 5.590E+3, 3.258E+3, 1.245E+3, 6.111E+2, 3.474E+2, 2.175E+2, 3.560E+2, 2.378E+2, 8.352E+1, 3.895E+1, 1.295E+1, 5.848   , 5.927   , 4.863   , 2.953   , 1.841   , 7.088E-1, 3.518E-1, 1.385E-1, 7.927E-2, 5.598E-2, 4.462E-2, 3.406E-2, 2.912E-2, 2.557E-2, 2.353E-2, 2.173E-2, 2.158E-2, 2.269E-2, 2.404E-2, 2.532E-2, 2.758E-2, 2.938E-2, 3.228E-2, 3.351E-2;...
2.281E+3, 5.008E+3, 3.337E+3, 1.280E+3, 6.296E+2, 3.584E+2, 2.246E+2, 3.595E+2, 2.416E+2, 8.538E+1, 3.994E+1, 1.333E+1, 6.033   , 3.242   , 4.722   , 2.937   , 1.849   , 7.197E-1, 3.584E-1, 1.409E-1, 8.039E-2, 5.650E-2, 4.483E-2, 3.399E-2, 2.893E-2, 2.530E-2, 2.325E-2, 2.144E-2, 2.129E-2, 2.241E-2, 2.374E-2, 2.502E-2, 2.725E-2, 2.902E-2, 3.188E-2, 3.306E-2;...
2.386E+3, 5.274E+3, 3.481E+3, 1.342E+3, 6.607E+2, 3.765E+2, 2.362E+2, 2.810E+2, 2.502E+2, 8.914E+1, 4.184E+1, 1.402E+1, 6.357   , 3.422   , 4.673   , 2.986   , 1.899   , 7.474E-1, 3.736E-1, 1.470E-1, 8.354E-2, 5.842E-2, 4.614E-2, 3.474E-2, 2.942E-2, 2.565E-2, 2.352E-2, 2.166E-2, 2.151E-2, 2.266E-2, 2.402E-2, 2.532E-2, 2.758E-2, 2.937E-2, 3.224E-2, 3.343E-2;...
2.483E+3, 5.503E+3, 3.438E+3, 1.391E+3, 6.858E+2, 3.913E+2, 2.456E+2, 2.906E+2, 2.554E+2, 9.187E+1, 4.328E+1, 1.456E+1, 6.616   , 3.566   , 4.554   , 2.993   , 1.925   , 7.660E-1, 3.846E-1, 1.514E-1, 8.578E-2, 5.971E-2, 4.694E-2, 3.509E-2, 2.958E-2, 2.570E-2, 2.352E-2, 2.163E-2, 2.149E-2, 2.264E-2, 2.401E-2, 2.532E-2, 2.758E-2, 2.938E-2, 3.223E-2, 3.342E-2;...
2.605E+3, 5.790E+3, 3.558E+3, 1.450E+3, 7.164E+2, 4.091E+2, 2.570E+2, 1.220E+2, 2.623E+2, 9.521E+1, 4.504E+1, 1.521E+1, 6.928   , 3.740   , 4.466   , 3.015   , 1.961   , 7.901E-1, 3.984E-1, 1.570E-1, 8.869E-2, 6.145E-2, 4.811E-2, 3.571E-2, 2.997E-2, 2.593E-2, 2.369E-2, 2.175E-2, 2.161E-2, 2.279E-2, 2.417E-2, 2.549E-2, 2.777E-2, 2.958E-2, 3.245E-2, 3.363E-2;...
2.737E+3, 6.003E+3, 3.488E+3, 1.510E+3, 7.480E+2, 4.276E+2, 2.688E+2, 1.278E+2, 2.694E+2, 9.860E+1, 4.683E+1, 1.588E+1, 7.253   , 3.922   , 4.349   , 3.028   , 1.995   , 8.141E-1, 4.125E-1, 1.629E-1, 9.173E-2, 6.329E-2, 4.934E-2, 3.638E-2, 3.040E-2, 2.620E-2, 2.388E-2, 2.190E-2, 2.176E-2, 2.295E-2, 2.435E-2, 2.569E-2, 2.799E-2, 2.981E-2, 3.268E-2, 3.387E-2;...
2.888E+3, 3.892E+3, 3.646E+3, 1.576E+3, 7.831E+2, 4.482E+2, 2.820E+2, 1.342E+2, 2.420E+2, 1.023E+2, 4.882E+1, 1.663E+1, 7.615   , 4.124   , 4.203   , 3.044   , 2.033   , 8.410E-1, 4.283E-1, 1.695E-1, 9.522E-2, 6.544E-2, 5.081E-2, 3.721E-2, 3.095E-2, 2.658E-2, 2.417E-2, 2.214E-2, 2.199E-2, 2.321E-2, 2.463E-2, 2.598E-2, 2.832E-2, 3.016E-2, 3.305E-2, 3.424E-2;...
3.006E+3, 1.340E+3, 3.751E+3, 1.620E+3, 8.072E+2, 4.626E+2, 2.913E+2, 1.389E+2, 2.444E+2, 1.047E+2, 5.012E+1, 1.716E+1, 7.875   , 4.273   , 2.584   , 3.008   , 2.038   , 8.554E-1, 4.381E-1, 1.737E-1, 9.741E-2, 6.671E-2, 5.159E-2, 3.754E-2, 3.107E-2, 2.658E-2, 2.413E-2, 2.208E-2, 2.191E-2, 2.314E-2, 2.457E-2, 2.593E-2, 2.826E-2, 3.010E-2, 3.302E-2, 3.419E-2;...
3.175E+3, 1.413E+3, 3.406E+3, 1.688E+3, 8.430E+2, 4.839E+2, 3.051E+2, 1.456E+2, 1.916E+2, 1.084E+2, 5.206E+1, 1.791E+1, 8.242   , 4.480   , 2.712   , 3.000   , 2.067   , 8.805E-1, 4.534E-1, 1.803E-1, 1.009E-1, 6.886E-2, 5.306E-2, 3.836E-2, 3.161E-2, 2.695E-2, 2.441E-2, 2.228E-2, 2.212E-2, 2.337E-2, 2.481E-2, 2.618E-2, 2.853E-2, 3.038E-2, 3.329E-2, 3.446E-2;...
3.324E+3, 1.478E+3, 3.546E+3, 1.745E+3, 8.720E+2, 5.013E+2, 3.164E+2, 1.512E+2, 1.975E+2, 1.111E+2, 5.353E+1, 1.851E+1, 8.542   , 4.651   , 2.820   , 2.958   , 2.075   , 8.973E-1, 4.645E-1, 1.853E-1, 1.035E-1, 7.044E-2, 5.409E-2, 3.886E-2, 3.188E-2, 2.708E-2, 2.447E-2, 2.230E-2, 2.212E-2, 2.338E-2, 2.483E-2, 2.620E-2, 2.855E-2, 3.040E-2, 3.330E-2, 3.445E-2;...
3.498E+3, 1.555E+3, 3.711E+3, 1.812E+3, 9.073E+2, 5.223E+2, 3.299E+2, 1.579E+2, 2.021E+2, 1.143E+2, 5.532E+1, 1.923E+1, 8.899   , 4.854   , 2.947   , 2.924   , 2.092   , 9.188E-1, 4.784E-1, 1.915E-1, 1.069E-1, 7.248E-2, 5.545E-2, 3.960E-2, 3.235E-2, 2.736E-2, 2.468E-2, 2.245E-2, 2.227E-2, 2.354E-2, 2.499E-2, 2.638E-2, 2.874E-2, 3.058E-2, 3.349E-2, 3.464E-2;...
3.671E+3, 1.632E+3, 3.853E+3, 1.873E+3, 9.405E+2, 5.423E+2, 3.428E+2, 1.643E+2, 9.204E+1, 1.172E+2, 5.697E+1, 1.991E+1, 9.240   , 5.050   , 3.070   , 2.879   , 2.100   , 9.378E-1, 4.913E-1, 1.973E-1, 1.100E-1, 7.440E-2, 5.673E-2, 4.028E-2, 3.276E-2, 2.761E-2, 2.484E-2, 2.256E-2, 2.236E-2, 2.363E-2, 2.510E-2, 2.649E-2, 2.886E-2, 3.072E-2, 3.360E-2, 3.475E-2;...
3.860E+3, 1.717E+3, 3.702E+3, 1.939E+3, 9.771E+2, 5.642E+2, 3.569E+2, 1.714E+2, 9.610E+1, 1.201E+2, 5.876E+1, 2.064E+1, 9.614   , 5.265   , 3.205   , 2.832   , 2.107   , 9.580E-1, 5.054E-1, 2.038E-1, 1.135E-1, 7.658E-2, 5.822E-2, 4.109E-2, 3.327E-2, 2.794E-2, 2.508E-2, 2.273E-2, 2.251E-2, 2.380E-2, 2.527E-2, 2.667E-2, 2.906E-2, 3.092E-2, 3.382E-2, 3.496E-2;...
4.019E+3, 1.790E+3, 2.179E+3, 1.904E+3, 1.004E+3, 5.813E+2, 3.682E+2, 1.770E+2, 9.940E+1, 1.218E+2, 5.999E+1, 2.120E+1, 9.907   , 5.437   , 3.314   , 2.756   , 2.092   , 9.696E-1, 5.150E-1, 2.085E-1, 1.161E-1, 7.813E-2, 5.923E-2, 4.157E-2, 3.351E-2, 2.802E-2, 2.511E-2, 2.272E-2, 2.248E-2, 2.376E-2, 2.523E-2, 2.663E-2, 2.901E-2, 3.086E-2, 3.375E-2, 3.486E-2;...
4.230E+3, 1.886E+3, 1.021E+3, 1.973E+3, 1.043E+3, 6.047E+2, 3.837E+2, 1.846E+2, 1.038E+2, 1.244E+2, 6.179E+1, 2.198E+1, 1.030E+1, 5.667   , 3.460   , 2.694   , 2.093   , 9.903E-1, 5.290E-1, 2.155E-1, 1.199E-1, 8.054E-2, 6.088E-2, 4.250E-2, 3.412E-2, 2.843E-2, 2.541E-2, 2.295E-2, 2.269E-2, 2.398E-2, 2.546E-2, 2.687E-2, 2.927E-2, 3.115E-2, 3.405E-2, 3.516E-2;...
4.421E+3, 1.974E+3, 1.069E+3, 1.925E+3, 1.078E+3, 6.262E+2, 3.979E+2, 1.917E+2, 1.078E+2, 1.265E+2, 6.333E+1, 2.268E+1, 1.067E+1, 5.879   , 3.595   , 2.592   , 2.081   , 1.006   , 5.413E-1, 2.216E-1, 1.233E-1, 8.265E-2, 6.230E-2, 4.325E-2, 3.459E-2, 2.871E-2, 2.560E-2, 2.307E-2, 2.279E-2, 2.408E-2, 2.557E-2, 2.699E-2, 2.939E-2, 3.127E-2, 3.418E-2, 3.529E-2;...
4.639E+3, 2.076E+3, 1.125E+3, 2.005E+3, 1.120E+3, 6.512E+2, 4.143E+2, 1.999E+2, 1.126E+2, 1.294E+2, 6.522E+1, 2.349E+1, 1.109E+1, 6.124   , 3.751   , 1.720   , 2.074   , 1.026   , 5.563E-1, 2.289E-1, 1.274E-1, 8.523E-2, 6.409E-2, 4.427E-2, 3.525E-2, 2.915E-2, 2.593E-2, 2.333E-2, 2.302E-2, 2.432E-2, 2.582E-2, 2.725E-2, 2.968E-2, 3.159E-2, 3.450E-2, 3.565E-2;...
4.817E+3, 2.161E+3, 1.172E+3, 2.069E+3, 1.153E+3, 6.710E+2, 4.272E+2, 2.065E+2, 1.164E+2, 1.307E+2, 6.647E+1, 2.410E+1, 1.141E+1, 6.321   , 3.878   , 1.782   , 2.043   , 1.036   , 5.661E-1, 2.342E-1, 1.304E-1, 8.711E-2, 6.536E-2, 4.493E-2, 3.563E-2, 2.936E-2, 2.605E-2, 2.339E-2, 2.307E-2, 2.436E-2, 2.586E-2, 2.730E-2, 2.975E-2, 3.165E-2, 3.460E-2, 3.576E-2;...
4.995E+3, 2.247E+3, 1.219E+3, 2.135E+3, 1.185E+3, 6.900E+2, 4.398E+2, 2.131E+2, 1.202E+2, 1.154E+2, 6.761E+1, 2.467E+1, 1.173E+1, 6.513   , 4.003   , 1.844   , 2.007   , 1.044   , 5.753E-1, 2.393E-1, 1.334E-1, 8.897E-2, 6.661E-2, 4.557E-2, 3.600E-2, 2.955E-2, 2.616E-2, 2.345E-2, 2.309E-2, 2.437E-2, 2.587E-2, 2.731E-2, 2.975E-2, 3.167E-2, 3.463E-2, 3.578E-2;...
5.197E+3, 2.344E+3, 1.274E+3, 1.913E+3, 1.221E+3, 7.124E+2, 4.546E+2, 2.207E+2, 1.247E+2, 9.100E+1, 6.899E+1, 2.536E+1, 1.211E+1, 6.740   , 4.149   , 1.916   , 1.976   , 1.056   , 5.870E-1, 2.455E-1, 1.370E-1, 9.128E-2, 6.819E-2, 4.644E-2, 3.654E-2, 2.988E-2, 2.640E-2, 2.360E-2, 2.322E-2, 2.449E-2, 2.600E-2, 2.744E-2, 2.989E-2, 3.181E-2, 3.478E-2, 3.595E-2;...
5.427E+3, 2.455E+3, 1.336E+3, 1.996E+3, 1.263E+3, 7.386E+2, 4.721E+2, 2.295E+2, 1.298E+2, 9.358E+1, 7.060E+1, 2.617E+1, 1.255E+1, 7.004   , 4.320   , 1.999   , 1.951   , 1.071   , 6.014E-1, 2.532E-1, 1.414E-1, 9.411E-2, 7.017E-2, 4.758E-2, 3.730E-2, 3.039E-2, 2.678E-2, 2.389E-2, 2.347E-2, 2.475E-2, 2.626E-2, 2.772E-2, 3.020E-2, 3.213E-2, 3.514E-2, 3.632E-2;...
5.710E+3, 2.591E+3, 1.411E+3, 2.090E+3, 1.264E+3, 7.721E+2, 4.942E+2, 2.406E+2, 1.363E+2, 9.728E+1, 7.278E+1, 2.721E+1, 1.311E+1, 7.334   , 4.532   , 2.103   , 1.936   , 1.095   , 6.207E-1, 2.630E-1, 1.471E-1, 9.781E-2, 7.281E-2, 4.916E-2, 3.839E-2, 3.117E-2, 2.740E-2, 2.438E-2, 2.392E-2, 2.521E-2, 2.674E-2, 2.822E-2, 3.074E-2, 3.271E-2, 3.577E-2, 3.694E-2;...
5.854E+3, 2.717E+3, 1.482E+3, 2.178E+3, 1.240E+3, 8.025E+2, 5.146E+2, 2.509E+2, 1.423E+2, 1.008E+2, 7.446E+1, 2.812E+1, 1.361E+1, 7.637   , 4.729   , 2.200   , 1.897   , 1.112   , 6.369E-1, 2.718E-1, 1.522E-1, 1.012E-1, 7.517E-2, 5.055E-2, 3.934E-2, 3.181E-2, 2.791E-2, 2.478E-2, 2.427E-2, 2.556E-2, 2.710E-2, 2.859E-2, 3.113E-2, 3.311E-2, 3.619E-2, 3.739E-2;...
5.812E+3, 2.706E+3, 1.478E+3, 1.507E+3, 1.229E+3, 7.931E+2, 5.085E+2, 2.485E+2, 1.411E+2, 9.807E+1, 7.231E+1, 2.760E+1, 1.342E+1, 7.552   , 4.686   , 2.185   , 1.766   , 1.073   , 6.204E-1, 2.668E-1, 1.496E-1, 9.941E-2, 7.377E-2, 4.942E-2, 3.832E-2, 3.089E-2, 2.704E-2, 2.396E-2, 2.344E-2, 2.469E-2, 2.620E-2, 2.766E-2, 3.016E-2, 3.216E-2, 3.534E-2, 3.670E-2;...
6.224E+3, 2.833E+3, 1.549E+3, 1.928E+3, 1.281E+3, 8.238E+2, 5.285E+2, 2.588E+2, 1.471E+2, 5.174E+1, 7.395E+1, 2.849E+1, 1.391E+1, 7.851   , 4.881   , 2.283   , 1.261   , 1.087   , 6.351E-1, 2.750E-1, 1.542E-1, 1.025E-1, 7.597E-2, 5.067E-2, 3.920E-2, 3.151E-2, 2.752E-2, 2.432E-2, 2.372E-2, 2.495E-2, 2.644E-2, 2.788E-2, 3.033E-2, 3.227E-2, 3.525E-2, 3.641E-2;...
6.187E+3, 2.937E+3, 1.608E+3, 6.555E+2, 1.322E+3, 8.472E+2, 5.438E+2, 2.669E+2, 1.519E+2, 5.353E+1, 7.496E+1, 2.913E+1, 1.429E+1, 8.088   , 5.039   , 2.363   , 1.308   , 1.091   , 6.440E-1, 2.811E-1, 1.580E-1, 1.050E-1, 7.772E-2, 5.165E-2, 3.983E-2, 3.191E-2, 2.779E-2, 2.452E-2, 2.387E-2, 2.509E-2, 2.658E-2, 2.802E-2, 3.050E-2, 3.245E-2, 3.549E-2, 3.668E-2;...
6.455E+3, 3.069E+3, 1.683E+3, 6.870E+2, 1.377E+3, 8.408E+2, 5.641E+2, 2.773E+2, 1.580E+2, 5.583E+1, 7.645E+1, 3.000E+1, 1.479E+1, 8.396   , 5.242   , 2.465   , 1.367   , 1.100   , 6.581E-1, 2.896E-1, 1.632E-1, 1.085E-1, 8.018E-2, 5.311E-2, 4.080E-2, 3.258E-2, 2.831E-2, 2.490E-2, 2.419E-2, 2.539E-2, 2.688E-2, 2.832E-2, 3.079E-2, 3.275E-2, 3.575E-2, 3.692E-2;...
6.600E+3, 3.148E+3, 1.729E+3, 7.070E+2, 1.208E+3, 8.581E+2, 5.749E+2, 2.830E+2, 1.615E+2, 5.718E+1, 6.714E+1, 3.032E+1, 1.503E+1, 8.558   , 5.355   , 2.526   , 1.403   , 1.090   , 6.603E-1, 2.931E-1, 1.656E-1, 1.101E-1, 8.128E-2, 5.366E-2, 4.109E-2, 3.269E-2, 2.834E-2, 2.487E-2, 2.411E-2, 2.528E-2, 2.674E-2, 2.817E-2, 3.060E-2, 3.253E-2, 3.550E-2, 3.664E-2;...
6.515E+3, 3.313E+3, 1.821E+3, 7.444E+2, 1.265E+3, 8.454E+2, 6.004E+2, 2.958E+2, 1.690E+2, 6.000E+1, 5.486E+1, 3.146E+1, 1.567E+1, 8.950   , 5.612   , 2.655   , 1.477   , 1.106   , 6.792E-1, 3.042E-1, 1.723E-1, 1.146E-1, 8.453E-2, 5.563E-2, 4.247E-2, 3.367E-2, 2.911E-2, 2.547E-2, 2.463E-2, 2.580E-2, 2.727E-2, 2.870E-2, 3.117E-2, 3.311E-2, 3.609E-2, 3.724E-2;...
6.612E+3, 3.368E+3, 1.852E+3, 7.578E+2, 1.277E+3, 8.569E+2, 6.062E+2, 2.989E+2, 1.711E+2, 6.084E+1, 5.496E+1, 3.149E+1, 1.576E+1, 9.034   , 5.678   , 2.695   , 1.502   , 1.083   , 6.746E-1, 3.050E-1, 1.732E-1, 1.152E-1, 8.494E-2, 5.574E-2, 4.241E-2, 3.351E-2, 2.891E-2, 2.523E-2, 2.434E-2, 2.546E-2, 2.689E-2, 2.829E-2, 3.068E-2, 3.259E-2, 3.552E-2, 3.662E-2;...
NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      1        ;...
NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      1        ;...
NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      1        ;...
NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      1        ;...
NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      1        ;...
NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      1        ;...
NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      1        ;...
NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      NaN,      1        ;...
];
end

%--------------------------------------------------------------------------
%% Hard-wire the table with description of each compton edge    
%   Z,  Energy,    mac low,  mac high, meac low, meac high
%-------------------------------------------------------------------------- 
if (isempty(edges))
edges = [ ...
    11, 0.0010721, 5.429E+2, 6.435E+3, 5.410E+2, 6.320E+3;...
    12, 0.001305 , 4.530E+2, 5.444E+3, 4.509E+2, 5.310E+3;...
    13, 0.0015596, 3.621E+2, 3.957E+3, 3.600E+2, 3.829E+3;...
    14, 0.0018389, 3.092E+2, 3.192E+3, 3.070E+2, 3.059E+3;...
    15, 0.0021455, 2.494E+2, 2.473E+3, 2.473E+2, 2.343E+3;...
    16, 0.002472 , 2.168E+2, 2.070E+3, 2.145E+2, 1.935E+3;...
    17, 0.0028224, 1.774E+2, 1.637E+3, 1.752E+2, 1.506E+3;...
    18, 0.0032029, 1.424E+2, 1.275E+3, 1.403E+2, 1.153E+3;...
    19, 0.0036074, 1.327E+2, 1.201E+3, 1.306E+2, 1.065E+3;...
    20, 0.0040381, 1.187E+2, 1.023E+3, 1.166E+2, 8.887E+2;...
    21, 0.0044928, 9.687E+1, 8.148E+2, 9.490E+1, 6.911E+2;...
    22, 0.0049664, 8.380E+1, 6.878E+2, 8.188E+1, 5.684E+2;...
    23, 0.0054651, 7.277E+1, 5.870E+2, 7.090E+1, 4.717E+2;...
    24, 0.0059892, 6.574E+1, 5.160E+2, 6.383E+1, 4.027E+2;...
    25, 0.006539 , 5.803E+1, 4.520E+2, 5.619E+1, 3.415E+2;...
    26, 0.007112 , 5.319E+1, 4.076E+2, 5.133E+1, 2.978E+2;...
    27, 0.0077089, 4.710E+1, 3.555E+2, 4.530E+1, 2.507E+2;...
    28, 0.0010081, 9.654E+3, 1.099E+4, 9.598E+3, 1.093E+4;...
    28, 0.0083328, 4.428E+1, 3.294E+2, 4.242E+1, 2.240E+2;...
    29, 0.0010961, 8.242E+3, 9.347E+3, 8.186E+3, 9.282E+3;...
    29, 0.0089789, 3.829E+1, 2.784E+2, 3.652E+1, 1.824E+2;...
    30, 0.0010197, 1.484E+3, 3.804E+3, 1.478E+3, 3.777E+3;...
    30, 0.0010428, 6.518E+3, 8.274E+3, 6.464E+3, 8.202E+3;...
    30, 0.0011936, 7.371E+3, 8.396E+3, 7.312E+3, 8.328E+3;...
    30, 0.0096586, 3.505E+1, 2.536E+2, 3.332E+1, 1.599E+2;...
    31, 0.0011154, 1.312E+3, 3.990E+3, 1.307E+3, 3.955E+3;...
    31, 0.0011423, 5.664E+3, 7.405E+3, 5.611E+3, 7.332E+3;...
    31, 0.0012977, 6.358E+3, 7.206E+3, 6.299E+3, 7.139E+3;...
    31, 0.0103671, 3.099E+1, 2.214E+2, 2.936E+1, 1.344E+2;...
    32, 0.0012167, 1.190E+3, 4.389E+3, 1.185E+3, 4.343E+3;...
    32, 0.0012478, 4.974E+3, 6.698E+3, 4.922E+3, 6.622E+3;...
    32, 0.0014143, 5.554E+3, 6.287E+3, 5.495E+3, 6.221E+3;...
    32, 0.0111031, 2.811E+1, 1.981E+2, 2.653E+1, 1.157E+2;...
    33, 0.0013231, 1.092E+3, 4.513E+3, 1.087E+3, 4.459E+3;...
    33, 0.0013586, 4.452E+3, 6.093E+3, 4.398E+3, 6.015E+3;...
    33, 0.0015265, 4.997E+3, 5.653E+3, 4.938E+3, 5.585E+3;...
    33, 0.0118667, 2.577E+1, 1.792E+2, 2.425E+1, 1.008E+2;...
    34, 0.0014358, 9.814E+2, 4.347E+3, 9.760E+2, 4.287E+3;...
    34, 0.0014762, 3.907E+3, 5.186E+3, 3.855E+3, 5.112E+3;...
    34, 0.0016539, 4.342E+3, 4.915E+3, 4.284E+3, 4.849E+3;...
    34, 0.0126578, 2.318E+1, 1.589E+2, 2.173E+1, 8.599E+1;...
    35, 0.0015499, 9.255E+2, 4.289E+3, 9.200E+2, 4.223E+3;...
    35, 0.001596 , 3.587E+3, 5.097E+3, 3.534E+3, 5.014E+3;...
    35, 0.001782 , 3.969E+3, 4.495E+3, 3.910E+3, 4.427E+3;...
    35, 0.0134737, 2.176E+1, 1.471E+2, 2.033E+1, 7.668E+1;...
    36, 0.0016749, 8.361E+2, 3.909E+3, 8.305E+2, 3.842E+3;...
    36, 0.0017272, 3.166E+3, 4.566E+3, 3.114E+3, 4.484E+3;...
    36, 0.001921 , 3.482E+3, 3.948E+3, 3.425E+3, 3.882E+3;...
    36, 0.0143256, 1.971E+1, 1.313E+2, 1.836E+1, 6.604E+1;...
    37, 0.0018044, 7.782E+2, 3.096E+3, 7.726E+2, 3.040E+3;...
    37, 0.0018639, 2.861E+3, 3.957E+3, 2.810E+3, 3.880E+3;...
    37, 0.0020651, 3.153E+3, 3.606E+3, 3.096E+3, 3.540E+3;...
    37, 0.0151997, 1.842E+1, 1.208E+2, 1.709E+1, 5.864E+1;...
    38, 0.0019396, 7.207E+2, 2.864E+3, 7.152E+2, 2.807E+3;...
    38, 0.0020068, 2.571E+3, 3.577E+3, 2.521E+3, 3.501E+3;...
    38, 0.0022163, 2.842E+3, 3.241E+3, 2.786E+3, 3.176E+3;...
    38, 0.0161046, 1.714E+1, 1.108E+2, 1.585E+1, 5.200E+1;...
    39, 0.00208  , 6.738E+2, 2.627E+3, 6.682E+2, 2.569E+3;...
    39, 0.0021555, 2.342E+3, 3.264E+3, 2.292E+3, 3.187E+3;...
    39, 0.0023725, 2.597E+3, 2.962E+3, 2.540E+3, 2.897E+3;...
    39, 0.0170384, 1.612E+1, 1.029E+2, 1.486E+1, 4.672E+1;...
    40, 0.0022223, 6.258E+2, 2.392E+3, 6.201E+2, 2.335E+3;...
    40, 0.0023067, 2.120E+3, 2.953E+3, 2.071E+3, 2.878E+3;...
    40, 0.0025316, 2.359E+3, 2.691E+3, 2.303E+3, 2.627E+3;...
    40, 0.0179976, 1.501E+1, 9.470E+1, 1.380E+1, 4.164E+1;...
    41, 0.0023705, 5.844E+2, 2.181E+3, 5.786E+2, 2.125E+3;...
    41, 0.0024647, 1.935E+3, 2.694E+3, 1.886E+3, 2.619E+3;...
    41, 0.0026977, 2.161E+3, 2.470E+3, 2.104E+3, 2.405E+3;...
    41, 0.0189856, 1.409E+1, 8.784E+1, 1.291E+1, 3.746E+1;...
    42, 0.0025202, 5.415E+2, 1.979E+3, 5.358E+2, 1.924E+3;...
    42, 0.0026251, 1.750E+3, 2.433E+3, 1.703E+3, 2.360E+3;...
    42, 0.0028655, 1.961E+3, 2.243E+3, 1.906E+3, 2.179E+3;...
    42, 0.0199995, 1.308E+1, 8.055E+1, 1.193E+1, 3.293E+1;...
    43, 0.0026769, 5.072E+2, 1.812E+3, 5.014E+2, 1.758E+3;...
    43, 0.0027932, 1.602E+3, 2.223E+3, 1.555E+3, 2.151E+3;...
    43, 0.0030425, 1.800E+3, 2.059E+3, 1.745E+3, 1.995E+3;...
    43, 0.021044 , 1.229E+1, 7.481E+1, 1.117E+1, 3.014E+1;...
    44, 0.0028379, 4.704E+2, 1.644E+3, 4.646E+2, 1.591E+3;...
    44, 0.0029669, 1.452E+3, 2.317E+3, 1.407E+3, 2.233E+3;...
    44, 0.003224 , 1.638E+3, 1.874E+3, 1.584E+3, 1.811E+3;...
    44, 0.0221172, 1.143E+1, 6.876E+1, 1.035E+1, 2.700E+1;...
    45, 0.0030038, 4.427E+2, 1.513E+3, 4.368E+2, 1.461E+3;...
    45, 0.0031461, 1.337E+3, 1.847E+3, 1.292E+3, 1.777E+3;...
    45, 0.0034119, 1.512E+3, 1.731E+3, 1.458E+3, 1.669E+3;...
    45, 0.0232199, 1.079E+1, 6.414E+1, 9.736   , 2.458E+1;...
    46, 0.0031733, 4.106E+2, 1.355E+3, 4.047E+2, 1.305E+3;...
    46, 0.0033303, 1.215E+3, 1.664E+3, 1.171E+3, 1.597E+3;...
    46, 0.0036043, 1.379E+3, 1.582E+3, 1.326E+3, 1.520E+3;...
    46, 0.0243503, 1.003E+1, 5.898E+1, 9.016   , 2.208E+1;...
    47, 0.0033511, 3.887E+2, 1.274E+3, 3.827E+2, 1.223E+3;...
    47, 0.0035237, 1.126E+3, 1.547E+3, 1.082E+3, 1.479E+3;...
    47, 0.0038058, 1.282E+3, 1.468E+3, 1.229E+3, 1.406E+3;...
    47, 0.025514 , 9.527   , 5.539E+1, 8.534   , 2.031E+1;...
    48, 0.0035375, 3.575E+2, 1.152E+3, 3.517E+2, 1.103E+3;...
    48, 0.003727 , 1.013E+3, 1.389E+3, 9.715E+2, 1.324E+3;...
    48, 0.004018 , 1.157E+3, 1.324E+3, 1.106E+3, 1.264E+3;...
    48, 0.0267112, 8.809   , 5.065E+1, 7.863   , 1.821E+1;...
    49, 0.0037301, 3.356E+2, 1.046E+3, 3.298E+2, 9.989E+2;...
    49, 0.003938 , 9.313E+2, 1.261E+3, 8.905E+2, 1.199E+3;...
    49, 0.0042375, 1.066E+3, 1.223E+3, 1.016E+3, 1.164E+3;...
    49, 0.0279399, 8.316   , 4.733E+1, 7.395   , 1.670E+1;...
    50, 0.0039288, 3.114E+2, 9.285E+2, 3.058E+2, 8.845E+2;...
    50, 0.0041561, 8.469E+2, 1.145E+3, 8.074E+2, 1.084E+3;...
    50, 0.0044647, 9.712E+2, 1.117E+3, 9.221E+2, 1.059E+3;...
    50, 0.0292001, 7.760   , 4.360E+1, 6.876   , 1.514E+1;...
    51, 0.0041322, 2.918E+2, 8.691E+2, 2.863E+2, 8.252E+2;...
    51, 0.0043804, 7.776E+2, 1.050E+3, 7.392E+2, 9.906E+2;...
    51, 0.0046983, 8.939E+2, 1.029E+3, 8.457E+2, 9.725E+2;...
    51, 0.0304912, 7.307   , 4.073E+1, 6.452   , 1.391E+1;...
    52, 0.001006 , 8.326E+3, 8.684E+3, 8.311E+3, 8.668E+3;...
    52, 0.0043414, 2.678E+2, 7.882E+2, 2.625E+2, 7.460E+2;...
    52, 0.004612 , 6.995E+2, 9.445E+2, 6.629E+2, 8.880E+2;...
    52, 0.0049392, 8.062E+2, 9.292E+2, 7.600E+2, 8.744E+2;...
    52, 0.0318138, 6.738   , 3.719E+1, 5.929   , 1.251E+1;...
    53, 0.0010721, 7.863E+3, 8.198E+3, 7.847E+3, 8.181E+3;...
    53, 0.0045571, 2.592E+2, 7.550E+2, 2.537E+2, 7.121E+2;...
    53, 0.0048521, 6.636E+2, 8.943E+2, 6.270E+2, 8.375E+2;...
    53, 0.0051881, 7.665E+2, 8.837E+2, 7.198E+2, 8.283E+2;...
    53, 0.0331694, 6.553   , 3.582E+1, 5.744   , 1.188E+1;...
    54, 0.001149 , 7.035E+3, 7.338E+3, 7.019E+3, 7.321E+3;...
    54, 0.0047822, 2.408E+2, 6.941E+2, 2.355E+2, 6.524E+2;...
    54, 0.0051037, 6.044E+2, 8.181E+2, 5.693E+2, 7.630E+2;...
    54, 0.0054528, 6.991E+2, 8.064E+2, 6.539E+2, 7.527E+2;...
    54, 0.0345614, 6.129   , 3.316E+1, 5.352   , 1.086E+1;...
    55, 0.001065 , 8.214E+3, 8.685E+3, 8.194E+3, 8.663E+3;...
    55, 0.0012171, 6.584E+3, 6.888E+3, 6.567E+3, 6.870E+3;...
    55, 0.0050119, 2.290E+2, 6.674E+2, 2.237E+2, 6.248E+2;...
    55, 0.0053594, 5.645E+2, 7.692E+2, 5.299E+2, 7.142E+2;...
    55, 0.0057143, 6.556E+2, 7.547E+2, 6.108E+2, 7.015E+2;...
    55, 0.0359846, 5.863   , 3.143E+1, 5.102   , 1.017E+1;...
    56, 0.0010622, 7.465E+3, 8.547E+3, 7.445E+3, 8.524E+3;...
    56, 0.0011367, 7.404E+3, 7.837E+3, 7.384E+3, 7.815E+3;...
    56, 0.0012928, 5.985E+3, 6.255E+3, 5.968E+3, 6.237E+3;...
    56, 0.005247 , 2.135E+2, 6.098E+2, 2.084E+2, 5.689E+2;...
    56, 0.0056236, 5.169E+2, 7.016E+2, 4.835E+2, 6.486E+2;...
    56, 0.0059888, 6.013E+2, 6.930E+2, 5.577E+2, 6.412E+2;...
    56, 0.0374406, 5.498   , 2.919E+1, 4.768   , 9.346   ;...
    57, 0.0011234, 7.000E+3, 8.025E+3, 6.979E+3, 8.001E+3;...
    57, 0.0012044, 6.911E+3, 7.321E+3, 6.890E+3, 7.298E+3;...
    57, 0.0013613, 5.652E+3, 5.905E+3, 5.634E+3, 5.886E+3;...
    57, 0.0054827, 2.039E+2, 5.764E+2, 1.988E+2, 5.356E+2;...
    57, 0.0058906, 4.846E+2, 6.545E+2, 4.517E+2, 6.023E+2;...
    57, 0.0062663, 5.654E+2, 6.518E+2, 5.221E+2, 6.003E+2;...
    57, 0.0389246, 5.271   , 2.772E+1, 4.554   , 8.785   ;...
    58, 0.0011854, 6.563E+3, 7.545E+3, 6.541E+3, 7.519E+3;...
    58, 0.0012728, 6.465E+3, 6.853E+3, 6.442E+3, 6.829E+3;...
    58, 0.0014366, 5.293E+3, 5.532E+3, 5.273E+3, 5.512E+3;...
    58, 0.0057234, 1.943E+2, 5.454E+2, 1.892E+2, 5.046E+2;...
    58, 0.0061642, 4.554E+2, 6.188E+2, 4.228E+2, 5.665E+2;...
    58, 0.0065488, 5.332E+2, 6.149E+2, 4.899E+2, 5.633E+2;...
    58, 0.040443 , 5.066   , 2.635E+1, 4.359   , 8.272   ;...
    59, 0.0012422, 6.278E+3, 7.224E+3, 6.254E+3, 7.196E+3;...
    59, 0.0013374, 6.142E+3, 6.513E+3, 6.118E+3, 6.487E+3;...
    59, 0.001511 , 5.006E+3, 5.235E+3, 4.986E+3, 5.214E+3;...
    59, 0.0059643, 1.868E+2, 5.214E+2, 1.816E+2, 4.802E+2;...
    59, 0.0064404, 4.310E+2, 5.857E+2, 3.984E+2, 5.333E+2;...
    59, 0.0068348, 5.060E+2, 5.836E+2, 4.625E+2, 5.317E+2;...
    59, 0.0419906, 4.892   , 2.518E+1, 4.192   , 7.833   ;...
    60, 0.001005, 6.949E+3, 8.554E+3, 6.918E+3, 8.516E+3;...
    60, 0.0012974, 5.933E+3, 6.833E+3, 5.908E+3, 6.803E+3;...
    60, 0.0014028, 5.744E+3, 6.093E+3, 5.719E+3, 6.066E+3;...
    60, 0.0015753, 4.738E+3, 4.951E+3, 4.717E+3, 4.929E+3;...
    60, 0.0062079, 1.767E+2, 4.892E+2, 1.715E+2, 4.484E+2;...
    60, 0.0067215, 4.013E+2, 5.456E+2, 3.694E+2, 4.939E+2;...
    60, 0.007126 , 4.723E+2, 5.448E+2, 4.293E+2, 4.936E+2;...
    60, 0.0435689, 4.642   , 2.366E+1, 3.962   , 7.296   ;...
    61, 0.0010269, 1.959E+3, 2.729E+3, 1.949E+3, 2.715E+3;...
    61, 0.0010515, 5.830E+3, 7.242E+3, 5.801E+3, 7.205E+3;...
    61, 0.0013569, 5.687E+3, 6.555E+3, 5.660E+3, 6.524E+3;...
    61, 0.0014714, 5.466E+3, 5.799E+3, 5.439E+3, 5.770E+3;...
    61, 0.001653 , 4.497E+3, 4.699E+3, 4.474E+3, 4.676E+3;...
    61, 0.0064593, 1.702E+2, 4.678E+2, 1.649E+2, 4.267E+2;...
    61, 0.0070128, 3.805E+2, 5.174E+2, 3.487E+2, 4.657E+2;...
    61, 0.0074279, 4.494E+2, 5.184E+2, 4.062E+2, 4.669E+2;...
    61, 0.045184 , 4.491   , 2.265E+1, 3.816   , 6.930   ;...
    62, 0.0010802, 1.827E+3, 2.635E+3, 1.817E+3, 2.620E+3;...
    62, 0.001106 , 5.613E+3, 6.852E+3, 5.582E+3, 6.814E+3;...
    62, 0.0014198, 5.262E+3, 6.072E+3, 5.234E+3, 6.039E+3;...
    62, 0.0015407, 5.046E+3, 5.355E+3, 5.018E+3, 5.325E+3;...
    62, 0.0017228, 4.189E+3, 4.376E+3, 4.165E+3, 4.352E+3;...
    62, 0.0067162, 1.587E+2, 4.334E+2, 1.536E+2, 3.933E+2;...
    62, 0.0073118, 3.496E+2, 4.756E+2, 3.189E+2, 4.255E+2;...
    62, 0.0077368, 4.141E+2, 4.777E+2, 3.721E+2, 4.276E+2;...
    62, 0.0468342, 4.208   , 2.100E+1, 3.562   , 6.378   ;...
    63, 0.0011309, 1.763E+3, 2.443E+3, 1.753E+3, 2.428E+3;...
    63, 0.0011606, 5.416E+3, 6.540E+3, 5.382E+3, 6.498E+3;...
    63, 0.0014806, 5.031E+3, 5.810E+3, 5.001E+3, 5.774E+3;...
    63, 0.0016139, 4.771E+3, 5.064E+3, 4.742E+3, 5.034E+3;...
    63, 0.0018   , 3.980E+3, 4.158E+3, 3.955E+3, 4.132E+3;...
    63, 0.0069769, 1.522E+2, 4.127E+2, 1.471E+2, 3.725E+2;...
    63, 0.0076171, 3.302E+2, 4.492E+2, 2.998E+2, 3.993E+2;...
    63, 0.008052 , 3.921E+2, 4.524E+2, 3.502E+2, 4.024E+2;...
    63, 0.048519 , 4.051   , 2.001E+1, 3.414   , 6.036   ;...
    64, 0.0011852, 1.668E+3, 1.931E+3, 1.658E+3, 1.919E+3;...
    64, 0.0012172, 3.961E+3, 4.764E+3, 3.934E+3, 4.731E+3;...
    64, 0.001544 , 4.701E+3, 5.432E+3, 4.670E+3, 5.395E+3;...
    64, 0.0016883, 4.421E+3, 4.694E+3, 4.391E+3, 4.662E+3;...
    64, 0.0018808, 3.691E+3, 3.854E+3, 3.666E+3, 3.829E+3;...
    64, 0.0072428, 1.429E+2, 3.844E+2, 1.379E+2, 3.452E+2;...
    64, 0.0079303, 3.049E+2, 4.149E+2, 2.755E+2, 3.665E+2;...
    64, 0.0083756, 3.631E+2, 4.190E+2, 3.223E+2, 3.702E+2;...
    64, 0.0502391, 3.812   , 1.864E+1, 3.199   , 5.585   ;...
    65, 0.0012412, 1.591E+3, 2.337E+3, 1.581E+3, 2.321E+3;...
    65, 0.001275 , 5.023E+3, 5.928E+3, 4.984E+3, 5.881E+3;...
    65, 0.0016113, 4.464E+3, 5.164E+3, 4.431E+3, 5.125E+3;...
    65, 0.0017677, 4.166E+3, 4.424E+3, 4.135E+3, 4.391E+3;...
    65, 0.0019675, 3.482E+3, 3.638E+3, 3.456E+3, 3.611E+3;...
    65, 0.007514 , 1.368E+2, 3.658E+2, 1.318E+2, 3.266E+2;...
    65, 0.0082516, 2.881E+2, 3.925E+2, 2.590E+2, 3.443E+2;...
    65, 0.008708 , 3.441E+2, 3.970E+2, 3.033E+2, 3.483E+2;...
    65, 0.0519957, 3.672   , 1.777E+1, 3.068   , 5.291   ;...
    66, 0.0012949, 1.521E+3, 2.173E+3, 1.511E+3, 2.156E+3;...
    66, 0.0013325, 4.745E+3, 5.551E+3, 4.704E+3, 5.502E+3;...
    66, 0.0016756, 4.229E+3, 4.896E+3, 4.194E+3, 4.855E+3;...
    66, 0.0018418, 3.924E+3, 4.168E+3, 3.892E+3, 4.133E+3;...
    66, 0.0020468, 3.286E+3, 3.433E+3, 3.259E+3, 3.404E+3;...
    66, 0.0077901, 1.299E+2, 3.437E+2, 1.249E+2, 3.051E+2;...
    66, 0.0085806, 2.695E+2, 3.672E+2, 2.410E+2, 3.198E+2;...
    66, 0.0090458, 3.228E+2, 3.725E+2, 2.826E+2, 3.245E+2;...
    66, 0.0537885, 3.500   , 1.676E+1, 2.911   , 4.961   ;...
    67, 0.0013514, 1.462E+3, 2.108E+3, 1.452E+3, 2.091E+3;...
    67, 0.0013915, 4.528E+3, 5.239E+3, 4.485E+3, 5.187E+3;...
    67, 0.0017412, 4.038E+3, 4.677E+3, 4.001E+3, 4.633E+3;...
    67, 0.0019228, 3.703E+3, 3.934E+3, 3.669E+3, 3.898E+3;...
    67, 0.0021283, 3.127E+3, 3.266E+3, 3.099E+3, 3.236E+3;...
    67, 0.0080711, 1.242E+2, 3.283E+2, 1.192E+2, 2.897E+2;...
    67, 0.0089178, 2.540E+2, 3.463E+2, 2.260E+2, 2.994E+2;...
    67, 0.0093942, 3.051E+2, 3.520E+2, 2.652E+2, 3.044E+2;...
    67, 0.0556177, 3.361   , 1.592E+1, 2.783   , 4.697   ;...
    68, 0.0014093, 1.408E+3, 2.069E+3, 1.397E+3, 2.051E+3;...
    68, 0.0014533, 4.339E+3, 4.952E+3, 4.293E+3, 4.898E+3;...
    68, 0.0018118, 3.841E+3, 4.452E+3, 3.802E+3, 4.406E+3;...
    68, 0.0020058, 3.499E+3, 3.717E+3, 3.464E+3, 3.680E+3;...
    68, 0.0022065, 2.995E+3, 3.127E+3, 2.965E+3, 3.096E+3;...
    68, 0.0083579, 1.190E+2, 3.126E+2, 1.140E+2, 2.741E+2;...
    68, 0.0092643, 2.397E+2, 3.271E+2, 2.121E+2, 2.806E+2;...
    68, 0.0097513, 2.886E+2, 3.330E+2, 2.491E+2, 2.858E+2;...
    68, 0.0574855, 3.232   , 1.514E+1, 2.664   , 4.442   ;...
    69, 0.0014677, 1.362E+3, 1.919E+3, 1.352E+3, 1.901E+3;...
    69, 0.0015146, 4.145E+3, 4.830E+3, 4.098E+3, 4.773E+3;...
    69, 0.0018845, 3.668E+3, 4.254E+3, 3.627E+3, 4.206E+3;...
    69, 0.0020898, 3.326E+3, 3.534E+3, 3.290E+3, 3.495E+3;...
    69, 0.0023068, 2.821E+3, 2.945E+3, 2.791E+3, 2.914E+3;...
    69, 0.008648 , 1.146E+2, 2.991E+2, 1.096E+2, 2.606E+2;...
    69, 0.0096169, 2.274E+2, 3.101E+2, 2.001E+2, 2.640E+2;...
    69, 0.0101157, 2.745E+2, 3.167E+2, 2.351E+2, 2.697E+2;...
    69, 0.0593896, 3.122   , 1.460E+1, 2.562   , 4.721   ;...
    70, 0.0015278, 1.300E+3, 1.965E+3, 1.289E+3, 1.944E+3;...
    70, 0.0015763, 3.908E+3, 4.540E+3, 3.859E+3, 4.482E+3;...
    70, 0.0019498, 3.495E+3, 4.054E+3, 3.453E+3, 4.004E+3;...
    70, 0.002173 , 3.128E+3, 3.323E+3, 3.091E+3, 3.284E+3;...
    70, 0.0023981, 2.652E+3, 2.769E+3, 2.621E+3, 2.737E+3;...
    70, 0.0089436, 1.088E+2, 2.824E+2, 1.039E+2, 2.444E+2;...
    70, 0.0099782, 2.128E+2, 2.906E+2, 1.862E+2, 2.455E+2;...
    70, 0.0104864, 2.577E+2, 2.973E+2, 2.189E+2, 2.512E+2;...
    70, 0.0613323, 2.975   , 1.365E+1, 2.432   , 3.964   ;...
    71, 0.0015885, 1.264E+3, 1.597E+3, 1.253E+3, 1.581E+3;...
    71, 0.0016394, 3.442E+3, 3.935E+3, 3.396E+3, 3.881E+3;...
    71, 0.0020236, 3.352E+3, 3.890E+3, 3.307E+3, 3.838E+3;...
    71, 0.0022635, 2.969E+3, 3.155E+3, 2.931E+3, 3.115E+3;...
    71, 0.0024912, 2.531E+3, 2.640E+3, 2.499E+3, 2.607E+3;...
    71, 0.0092441, 1.049E+2, 2.703E+2, 9.994E+1, 2.324E+2;...
    71, 0.0103486, 2.019E+2, 2.757E+2, 1.756E+2, 2.310E+2;...
    71, 0.0108704, 2.450E+2, 2.826E+2, 2.065E+2, 2.369E+2;...
    71, 0.0633138, 2.874   , 1.305E+1, 2.339   , 3.770   ;...
    72, 0.0016617, 1.203E+3, 1.505E+3, 1.192E+3, 1.489E+3;...
    72, 0.0017164, 3.273E+3, 3.682E+3, 3.225E+3, 3.628E+3;...
    72, 0.0021076, 3.152E+3, 3.659E+3, 3.108E+3, 3.607E+3;...
    72, 0.0023654, 2.769E+3, 2.943E+3, 2.731E+3, 2.903E+3;...
    72, 0.0026009, 2.363E+3, 2.465E+3, 2.331E+3, 2.432E+3;...
    72, 0.0095607, 9.997E+1, 2.557E+2, 9.510E+1, 2.184E+2;...
    72, 0.0107394, 1.894E+2, 2.589E+2, 1.639E+2, 2.151E+2;...
    72, 0.0112707, 2.306E+2, 2.659E+2, 1.928E+2, 2.212E+2;...
    72, 0.0653508, 2.751   , 1.237E+1, 2.228   , 3.557   ;...
    73, 0.0017351, 1.154E+3, 1.417E+3, 1.144E+3, 1.401E+3;...
    73, 0.0017932, 3.082E+3, 3.421E+3, 3.034E+3, 3.367E+3;...
    73, 0.002194 , 2.985E+3, 3.464E+3, 2.939E+3, 3.411E+3;...
    73, 0.0024687, 2.604E+3, 2.768E+3, 2.566E+3, 2.727E+3;...
    73, 0.002708 , 2.233E+3, 2.329E+3, 2.201E+3, 2.296E+3;...
    73, 0.0098811, 9.599E+1, 2.443E+2, 9.117E+1, 2.073E+2;...
    73, 0.0111361, 1.791E+2, 2.449E+2, 1.540E+2, 2.018E+2;...
    73, 0.0116815, 2.182E+2, 2.518E+2, 1.811E+2, 2.077E+2;...
    73, 0.0674164, 2.652   , 1.180E+1, 2.139   , 3.379   ;...
    74, 0.0018092, 1.108E+3, 1.327E+3, 1.097E+3, 1.311E+3;...
    74, 0.0018716, 2.901E+3, 3.170E+3, 2.853E+3, 3.116E+3;...
    74, 0.002281 , 2.828E+3, 3.279E+3, 2.781E+3, 3.226E+3;...
    74, 0.0025749, 2.445E+3, 2.599E+3, 2.407E+3, 2.558E+3;...
    74, 0.0028196, 2.104E+3, 2.194E+3, 2.071E+3, 2.160E+3;...
    74, 0.0102068, 9.201E+1, 2.334E+2, 8.724E+1, 1.966E+2;...
    74, 0.011544 , 1.689E+2, 2.312E+2, 1.444E+2, 1.889E+2;...
    74, 0.0120998, 2.065E+2, 2.382E+2, 1.699E+2, 1.948E+2;...
    74, 0.069525 , 2.552   , 1.123E+1, 2.049   , 3.212   ;...
    75, 0.0018224, 1.147E+3, 1.152E+3, 1.135E+3, 1.141E+3;...
    75, 0.0019489, 2.689E+3, 3.802E+3, 2.642E+3, 3.730E+3;...
    75, 0.0023673, 2.696E+3, 3.124E+3, 2.648E+3, 3.069E+3;...
    75, 0.0026816, 2.309E+3, 2.454E+3, 2.270E+3, 2.412E+3;...
    75, 0.0029317, 1.992E+3, 2.078E+3, 1.959E+3, 2.043E+3;...
    75, 0.0105353, 8.858E+1, 2.234E+2, 8.386E+1, 1.869E+2;...
    75, 0.0119587, 1.600E+2, 2.191E+2, 1.360E+2, 1.776E+2;...
    75, 0.0125267, 1.961E+2, 2.262E+2, 1.600E+2, 1.834E+2;...
    75, 0.0716764, 2.464   , 1.073E+1, 1.970   , 3.058   ;...
    76, 0.0019601, 1.023E+3, 2.003E+3, 1.012E+3, 1.969E+3;...
    76, 0.0020308, 2.622E+3, 2.864E+3, 2.573E+3, 2.810E+3;...
    76, 0.0024572, 2.546E+3, 2.948E+3, 2.498E+3, 2.893E+3;...
    76, 0.0027922, 2.161E+3, 2.296E+3, 2.123E+3, 2.255E+3;...
    76, 0.0030485, 1.869E+3, 1.949E+3, 1.836E+3, 1.915E+3;...
    76, 0.0108709, 8.458E+1, 2.121E+2, 7.994E+1, 1.762E+2;...
    76, 0.012385 , 1.503E+2, 2.060E+2, 1.271E+2, 1.654E+2;...
    76, 0.012968 , 1.846E+2, 2.129E+2, 1.494E+2, 1.711E+2;...
    76, 0.0738708, 2.360   , 1.016E+1, 1.879   , 2.888   ;...
    77, 0.0020404, 9.882E+2, 1.066E+3, 9.771E+2, 1.053E+3;...
    77, 0.0021161, 2.478E+3, 2.608E+3, 2.429E+3, 2.556E+3;...
    77, 0.0025507, 2.424E+3, 2.800E+3, 2.376E+3, 2.744E+3;...
    77, 0.0029087, 2.041E+3, 2.166E+3, 2.002E+3, 2.125E+3;...
    77, 0.0031737, 1.767E+3, 1.843E+3, 1.734E+3, 1.809E+3;...
    77, 0.0112154, 8.155E+1, 2.038E+2, 7.695E+1, 1.681E+2;...
    77, 0.0128241, 1.426E+2, 1.959E+2, 1.199E+2, 1.559E+2;...
    77, 0.0134185, 1.756E+2, 2.025E+2, 1.409E+2, 1.613E+2;...
    77, 0.076111 , 2.284   , 9.728   , 1.810   , 2.758   ;...
    78, 0.0021216, 9.506E+2, 1.034E+3, 9.393E+2, 1.021E+3;...
    78, 0.0022019, 2.341E+3, 2.487E+3, 2.292E+3, 2.434E+3;...
    78, 0.0026454, 2.307E+3, 2.663E+3, 2.258E+3, 2.606E+3;...
    78, 0.0030265, 1.923E+3, 2.041E+3, 1.884E+3, 2.000E+3;...
    78, 0.003296 , 1.671E+3, 1.742E+3, 1.637E+3, 1.707E+3;...
    78, 0.0115637, 7.844E+1, 1.946E+2, 7.388E+1, 1.594E+2;...
    78, 0.0132726, 1.349E+2, 1.853E+2, 1.127E+2, 1.463E+2;...
    78, 0.0138799, 1.666E+2, 1.921E+2, 1.325E+2, 1.517E+2;...
    78, 0.0783948, 2.203   , 9.301   , 1.738   , 2.627   ;...
    79, 0.0022057, 9.187E+2, 9.971E+2, 9.074E+2, 9.836E+2;...
    79, 0.0022911, 2.258E+3, 2.389E+3, 2.208E+3, 2.336E+3;...
    79, 0.002743 , 2.203E+3, 2.541E+3, 2.154E+3, 2.484E+3;...
    79, 0.0031478, 1.822E+3, 1.933E+3, 1.783E+3, 1.892E+3;...
    79, 0.0034249, 1.585E+3, 1.652E+3, 1.552E+3, 1.618E+3;...
    79, 0.0119187, 7.582E+1, 1.870E+2, 7.129E+1, 1.521E+2;...
    79, 0.0137336, 1.283E+2, 1.764E+2, 1.066E+2, 1.379E+2;...
    79, 0.0143528, 1.588E+2, 1.830E+2, 1.252E+2, 1.432E+2;...
    79, 0.0807249, 2.137   , 8.904   , 1.678   , 2.512   ;...
    80, 0.0022949, 8.773E+2, 9.925E+2, 8.660E+2, 9.778E+2;...
    80, 0.0023849, 2.151E+3, 2.316E+3, 2.101E+3, 2.261E+3;...
    80, 0.0028471, 2.080E+3, 2.400E+3, 2.030E+3, 2.343E+3;...
    80, 0.0032785, 1.704E+3, 1.808E+3, 1.666E+3, 1.767E+3;...
    80, 0.0035616, 1.486E+3, 1.549E+3, 1.454E+3, 1.515E+3;...
    80, 0.0122839, 7.266E+1, 1.780E+2, 6.821E+1, 1.437E+2;...
    80, 0.0142087, 1.209E+2, 1.663E+2, 9.988E+1, 1.289E+2;...
    80, 0.0148393, 1.501E+2, 1.728E+2, 1.173E+2, 1.340E+2;...
    80, 0.0831023, 2.055   , 8.464   , 1.607   , 2.384   ;...
    81, 0.0023893, 8.352E+2, 1.136E+3, 8.240E+2, 1.115E+3;...
    81, 0.0024851, 2.049E+3, 2.364E+3, 1.999E+3, 2.303E+3;...
    81, 0.0029566, 1.958E+3, 2.267E+3, 1.910E+3, 2.210E+3;...
    81, 0.0034157, 1.591E+3, 1.687E+3, 1.553E+3, 1.647E+3;...
    81, 0.0037041, 1.392E+3, 1.452E+3, 1.360E+3, 1.419E+3;...
    81, 0.0126575, 6.957E+1, 1.693E+2, 6.521E+1, 1.357E+2;...
    81, 0.0146979, 1.139E+2, 1.572E+2, 9.356E+1, 1.207E+2;...
    81, 0.0153467, 1.416E+2, 1.632E+2, 1.097E+2, 1.254E+2;...
    81, 0.0855304, 1.976   , 8.046   , 1.539   , 2.264   ;...
    82, 0.002484 , 8.006E+2, 1.397E+3, 7.895E+2, 1.366E+3;...
    82, 0.0025856, 1.944E+3, 2.458E+3, 1.895E+3, 2.390E+3;...
    82, 0.0030664, 1.857E+3, 2.146E+3, 1.808E+3, 2.090E+3;...
    82, 0.0035542, 1.496E+3, 1.585E+3, 1.459E+3, 1.546E+3;...
    82, 0.0038507, 1.311E+3, 1.368E+3, 1.279E+3, 1.335E+3;...
    82, 0.0130352, 6.701E+1, 1.621E+2, 6.270E+1, 1.291E+2;...
    82, 0.0152   , 1.078E+2, 1.485E+2, 8.807E+1, 1.131E+2;...
    82, 0.0158608, 1.344E+2, 1.548E+2, 1.032E+2, 1.180E+2;...
    82, 0.0880045, 1.910   , 7.683   , 1.482   , 2.160   ;...
    83, 0.0025796, 7.724E+2, 1.777E+3, 7.612E+2, 1.731E+3;...
    83, 0.0026876, 1.852E+3, 2.576E+3, 1.802E+3, 2.500E+3;...
    83, 0.0031769, 1.774E+3, 2.048E+3, 1.725E+3, 1.992E+3;...
    83, 0.0036963, 1.415E+3, 1.498E+3, 1.378E+3, 1.460E+3;...
    83, 0.0039991, 1.243E+3, 1.297E+3, 1.211E+3, 1.264E+3;...
    83, 0.0134186, 6.491E+1, 1.560E+2, 6.064E+1, 1.234E+2;...
    83, 0.0157111, 1.027E+2, 1.416E+2, 8.346E+1, 1.069E+2;...
    83, 0.0163875, 1.282E+2, 1.478E+2, 9.769E+1, 1.116E+2;...
    83, 0.0905259, 1.856   , 7.380   , 1.434   , 2.073   ;...
    84, 0.002683 , 7.482E+2, 2.206E+3, 7.369E+2, 2.142E+3;...
    84, 0.002798 , 1.775E+3, 2.711E+3, 1.726E+3, 2.625E+3;...
    84, 0.0033019, 1.693E+3, 1.956E+3, 1.645E+3, 1.900E+3;...
    84, 0.0038541, 1.339E+3, 1.419E+3, 1.303E+3, 1.381E+3;...
    84, 0.0041494, 1.190E+3, 1.242E+3, 1.158E+3, 1.209E+3;...
    84, 0.0138138, 6.337E+1, 1.513E+2, 5.910E+1, 1.189E+2;...
    84, 0.0162443, 9.849E+1, 1.361E+2, 7.965E+1, 1.019E+2;...
    84, 0.0169393, 1.232E+2, 1.420E+2, 9.308E+1, 1.063E+2;...
    84, 0.093105 , 1.820   , 7.140   , 1.400   , 2.005   ;...
    85, 0.001042 , 5.435E+3, 5.551E+3, 5.420E+3, 5.537E+3;...
    85, 0.0027867, 7.225E+2, 2.431E+3, 7.112E+2, 2.355E+3;...
    85, 0.0029087, 1.686E+3, 2.763E+3, 1.638E+3, 2.671E+3;...
    85, 0.003426 , 1.614E+3, 1.866E+3, 1.566E+3, 1.811E+3;...
    85, 0.004008 , 1.269E+3, 1.345E+3, 1.233E+3, 1.308E+3;...
    85, 0.004317 , 1.126E+3, 1.175E+3, 1.095E+3, 1.142E+3;...
    85, 0.0142135, 6.162E+1, 1.461E+2, 5.738E+1, 1.140E+2;...
    85, 0.0167847, 9.417E+1, 1.302E+2, 7.576E+1, 9.670E+1;...
    85, 0.017493 , 1.181E+2, 1.360E+2, 8.847E+1, 1.010E+2;...
    85, 0.0957299, 1.777   , 6.895   , 1.361   , 1.935   ;...
    86, 0.001097 , 4.909E+3, 5.013E+3, 4.895E+3, 5.000E+3;...
    86, 0.0028924, 6.634E+2, 2.319E+3, 6.526E+2, 2.243E+3;...
    86, 0.0030215, 1.524E+3, 2.470E+3, 1.479E+3, 2.384E+3;...
    86, 0.003538 , 1.479E+3, 1.710E+3, 1.433E+3, 1.657E+3;...
    86, 0.004159 , 1.148E+3, 1.217E+3, 1.115E+3, 1.182E+3;...
    86, 0.004482 , 1.017E+3, 1.061E+3, 9.877E+2, 1.031E+3;...
    86, 0.0146194, 5.697E+1, 1.344E+2, 5.296E+1, 1.042E+2;...
    86, 0.0173371, 8.557E+1, 1.185E+2, 6.851E+1, 8.724E+1;...
    86, 0.018049 , 1.077E+2, 1.241E+2, 8.002E+1, 9.130E+1;...
    86, 0.098404 , 1.649   , 6.371   , 1.258   , 1.779   ;...
    87, 0.001153 , 4.673E+3, 4.772E+3, 4.659E+3, 4.758E+3;...
    87, 0.0029999, 6.414E+2, 1.995E+3, 6.306E+2, 1.929E+3;...
    87, 0.0031362, 1.449E+3, 2.255E+3, 1.405E+3, 2.175E+3;...
    87, 0.003663 , 1.415E+3, 1.641E+3, 1.370E+3, 1.588E+3;...
    87, 0.004327 , 1.086E+3, 1.153E+3, 1.053E+3, 1.118E+3;...
    87, 0.004652 , 9.666E+2, 1.008E+3, 9.377E+2, 9.780E+2;...
    87, 0.0150312, 5.544E+1, 1.304E+2, 5.146E+1, 1.003E+2;...
    87, 0.0179065, 8.184E+1, 1.137E+2, 6.521E+1, 8.299E+1;...
    87, 0.018639 , 1.031E+2, 1.187E+2, 7.602E+1, 8.666E+1;...
    87, 0.101137 , 1.610   , 6.089   , 1.224   , 1.710   ;...
    88, 0.0010576, 5.602E+3, 5.683E+3, 5.588E+3, 5.669E+3;...
    88, 0.0012084, 4.424E+3, 4.516E+3, 4.411E+3, 4.502E+3;...
    88, 0.0031049, 6.169E+2, 1.715E+3, 6.061E+2, 1.657E+3;...
    88, 0.0032484, 1.374E+3, 2.039E+3, 1.331E+3, 1.965E+3;...
    88, 0.0037918, 1.342E+3, 1.558E+3, 1.297E+3, 1.506E+3;...
    88, 0.0044895, 1.024E+3, 1.086E+3, 9.917E+2, 1.052E+3;...
    88, 0.004822 , 9.129E+2, 9.522E+2, 8.846E+2, 9.228E+2;...
    88, 0.0154444, 5.355E+1, 1.253E+2, 4.962E+1, 9.581E+1;...
    88, 0.0184843, 7.766E+1, 1.081E+2, 6.160E+1, 7.835E+1;...
    88, 0.0192367, 9.799E+1, 1.128E+2, 7.175E+1, 8.172E+1;...
    88, 0.103922 , 1.560   , 5.826   , 1.181   , 1.638   ;...
    89, 0.00108  , 5.622E+3, 5.702E+3, 5.608E+3, 5.688E+3;...
    89, 0.001269 , 4.203E+3, 4.289E+3, 4.190E+3, 4.275E+3;...
    89, 0.003219 , 5.959E+2, 1.537E+3, 5.851E+2, 1.485E+3;...
    89, 0.0033702, 1.307E+3, 1.892E+3, 1.264E+3, 1.822E+3;...
    89, 0.003909 , 1.296E+3, 1.504E+3, 1.252E+3, 1.453E+3;...
    89, 0.004656 , 9.745E+2, 1.034E+3, 9.428E+2, 1.000E+3;...
    89, 0.005002 , 8.678E+2, 9.051E+2, 8.399E+2, 8.761E+2;...
    89, 0.015871 , 5.213E+1, 1.214E+2, 4.823E+1, 9.224E+1;...
    89, 0.0190832, 7.426E+1, 1.036E+2, 5.865E+1, 7.448E+1;...
    89, 0.01984  , 9.403E+1, 1.082E+2, 6.831E+1, 7.774E+1;...
    89, 0.106756 , 1.525   , 5.622   , 1.150   , 1.582   ;...
    90, 0.0011683, 4.976E+3, 5.048E+3, 4.962E+3, 5.034E+3;...
    90, 0.0013295, 3.937E+3, 4.017E+3, 3.924E+3, 4.003E+3;...
    90, 0.003332 , 5.671E+2, 1.394E+3, 5.565E+2, 1.346E+3;...
    90, 0.0034908, 1.226E+3, 1.749E+3, 1.185E+3, 1.683E+3;...
    90, 0.0040461, 1.217E+3, 1.415E+3, 1.174E+3, 1.364E+3;...
    90, 0.0048304, 9.103E+2, 9.655E+2, 8.797E+2, 9.330E+2;...
    90, 0.0051823, 8.129E+2, 8.477E+2, 7.859E+2, 8.196E+2;...
    90, 0.0163003, 4.991E+1, 1.156E+2, 4.611E+1, 8.732E+1;...
    90, 0.0196932, 6.984E+1, 9.527E+1, 5.492E+1, 6.836E+1;...
    90, 0.0204721, 8.851E+1, 1.018E+2, 6.383E+1, 7.259E+1;...
    90, 0.109651 , 1.465   , 5.336   , 1.100   , 1.502   ;...
    91, 0.0010067, 6.451E+3, 6.868E+3, 6.437E+3, 6.853E+3;...
    91, 0.0012243, 4.794E+3, 4.863E+3, 4.780E+3, 4.848E+3;...
    91, 0.0013871, 3.815E+3, 3.891E+3, 3.801E+3, 3.877E+3;...
    91, 0.0034418, 5.548E+2, 1.363E+3, 5.440E+2, 1.314E+3;...
    91, 0.0036112, 1.185E+3, 1.695E+3, 1.144E+3, 1.628E+3;...
    91, 0.0041738, 1.182E+3, 1.374E+3, 1.139E+3, 1.324E+3;...
    91, 0.0050009, 8.755E+2, 9.288E+2, 8.450E+2, 8.964E+2;...
    91, 0.0053669, 7.810E+2, 8.144E+2, 7.541E+2, 7.864E+2;...
    91, 0.0167331, 4.903E+1, 1.130E+2, 4.522E+1, 8.483E+1;...
    91, 0.0203137, 6.747E+1, 9.452E+1, 5.284E+1, 6.704E+1;...
    91, 0.0211046, 8.576E+1, 9.861E+1, 6.153E+1, 6.990E+1;...
    91, 0.112601 , 1.445   , 5.198   , 1.081   , 1.466   ;...
    92, 0.0010449, 6.127E+3, 6.519E+3, 6.112E+3, 6.504E+3;...
    92, 0.0012726, 4.526E+3, 4.589E+3, 4.511E+3, 4.575E+3;...
    92, 0.0014408, 3.598E+3, 3.668E+3, 3.584E+3, 3.654E+3;...
    92, 0.0035517, 5.257E+2, 1.266E+3, 5.151E+2, 1.220E+3;...
    92, 0.0037276, 1.112E+3, 1.582E+3, 1.072E+3, 1.517E+3;...
    92, 0.0043034, 1.110E+3, 1.292E+3, 1.068E+3, 1.242E+3;...
    92, 0.0051822, 8.118E+2, 8.611E+2, 7.825E+2, 8.301E+2;...
    92, 0.005548 , 7.282E+2, 7.592E+2, 7.022E+2, 7.322E+2;...
    92, 0.0171663, 4.663E+1, 1.070E+2, 4.293E+1, 7.980E+1;...
    92, 0.0209476, 6.300E+1, 8.838E+1, 4.914E+1, 6.222E+1;...
    92, 0.0217574, 8.023E+1, 9.222E+1, 5.715E+1, 6.487E+1;...
    92, 0.115606 , 1.378   , 4.893   , 1.027   , 1.382   ;...
    93, 1.087E-3 , 6.00E+3, 6.38E+3, NaN,  NaN;...
    93, 1.328E-3 , 4.38E+3, 4.44E+3, NaN,  NaN;...
    93, 1.501E-3 , 3.49E+3, 3.55E+3, NaN,  NaN;...
    93, 3.666E-3 , 5.15E+2, 1.22E+3, NaN,  NaN;...
    93, 3.850E-3 , 1.08E+3, 1.52E+3, NaN,  NaN;...
    93, 4.435E-3 , 1.08E+3, 1.26E+3, NaN,  NaN;...
    93, 5.366E-3 , 7.80E+2, 8.27E+2, NaN,  NaN;...
    93, 5.723E-3 , 7.06E+2, 7.36E+2, NaN,  NaN;...
    93, 1.761E-2 , 4.59E+1, 1.05E+2, NaN,  NaN;...
    93, 2.160E-2 , 6.08E+1, 8.55E+1, NaN,  NaN;...
    93, 2.243E-2 , 7.76E+1, 8.92E+1, NaN,  NaN;...
    93, 1.187E-1 , 1.36   , 4.77   , NaN,  NaN;...
    94, 1.115E-3 , 5.94E+3, 6.32E+3, NaN,  NaN;...
    94, 1.372E-3 , 4.27E+3, 4.32E+3, NaN,  NaN;...
    94, 1.559E-3 , 3.35E+3, 3.42E+3, NaN,  NaN;...
    94, 3.778E-3 , 4.99E+2, 1.18E+3, NaN,  NaN;...
    94, 3.973E-3 , 1.03E+3, 1.40E+3, NaN,  NaN;...
    94, 4.557E-3 , 1.04E+3, 1.21E+3, NaN,  NaN;...
    94, 5.541E-3 , 7.43E+2, 7.88E+2, NaN,  NaN;...
    94, 5.933E-3 , 6.67E+2, 6.95E+2, NaN,  NaN;...
    94, 1.806E-2 , 4.45E+1, 1.01E+2, NaN,  NaN;...
    94, 2.227E-2 , 5.80E+1, 8.17E+1, NaN,  NaN;...
    94, 2.310E-2 , 7.44E+1, 8.54E+1, NaN,  NaN;...
    94, 1.218E-1 , 1.32   , 4.59   , NaN,  NaN;...
    95, 1.14E-03, 5.90E+03, 6.27E+03,  NaN,  NaN;... % 5N3
    95, 1.41E-03, 4.15E+03, 4.20E+03,  NaN,  NaN;... %95N2
    95, 1.62E-03, 3.20E+03, 3.26E+03,  NaN,  NaN;... %95N1
    95, 3.89E-03, 4.81E+02, 1.11E+03,  NaN,  NaN;... %95M5
    95, 4.09E-03, 9.85E+02, 1.39E+03,  NaN,  NaN;... %95M4
    95, 4.67E-03, 1.01E+03, 1.17E+03,  NaN,  NaN;... %95M3
    95, 5.71E-03, 7.07E+02, 7.49E+02,  NaN,  NaN;... %95M2
    95, 6.12E-03, 6.32E+02, 6.58E+02,  NaN,  NaN;... %95M1
    95, 1.85E-02, 4.30E+01, 9.70E+01,  NaN,  NaN;... %95L3
    95, 2.29E-02, 5.50E+01, 7.75E+01,  NaN,  NaN;... %95L2
    95, 2.38E-02, 7.07E+01, 8.12E+01,  NaN,  NaN;... %95L1
    95, 1.25E-01, 1.28E+00, 4.38E+00,  NaN,  NaN;... %95K
    96, 1.15E-03, 5.88E+03, 6.24E+03,  NaN,  NaN;... %96N3
    96, 1.44E-03, 4.10E+03, 4.19E+03,  NaN,  NaN;... %96N2
    96, 1.64E-03, 3.18E+03, 3.24E+03,  NaN,  NaN;... %96N1
    96, 3.97E-03, 4.71E+02, 1.05E+03,  NaN,  NaN;... %96M5
    96, 4.23E-03, 9.32E+02, 1.30E+03,  NaN,  NaN;... %96M4
    96, 4.80E-03, 9.65E+02, 1.12E+03,  NaN,  NaN;... %96M3
    96, 5.90E-03, 6.69E+02, 7.09E+02,  NaN,  NaN;... %96M2
    96, 6.29E-03, 6.05E+02, 6.30E+02,  NaN,  NaN;... %96M1
    96, 1.89E-02, 4.16E+01, 9.35E+01,  NaN,  NaN;... %96L3
    96, 2.38E-02, 5.12E+01, 7.22E+01,  NaN,  NaN;... %96L2
    96, 2.45E-02, 6.73E+01, 7.72E+01,  NaN,  NaN;... %96L1
    96, 1.28E-01, 1.24E+00, 4.18E+00,  NaN,  NaN;... %96K
    97, 1.24E-03, 5.41E+03, 5.75E+03,  NaN,  NaN;... %97N3
    97, 1.55E-03, 3.68E+03, 3.73E+03,  NaN,  NaN;... %97N2
    97, 1.76E-03, 2.90E+03, 2.96E+03,  NaN,  NaN;... %97N1
    97, 4.13E-03, 4.50E+02, 9.93E+02,  NaN,  NaN;... %97M5
    97, 4.37E-03, 8.97E+02, 1.24E+03,  NaN,  NaN;... %97M4
    97, 4.98E-03, 9.16E+02, 1.07E+03,  NaN,  NaN;... %97M3
    97, 6.15E-03, 6.28E+02, 6.65E+02,  NaN,  NaN;... %97M2
    97, 6.56E-03, 5.68E+02, 5.91E+02,  NaN,  NaN;... %97M1
    97, 1.95E-02, 4.05E+01, 9.09E+01,  NaN,  NaN;... %97L3
    97, 2.44E-02, 5.00E+01, 7.07E+01,  NaN,  NaN;... %97L2
    97, 2.53E-02, 6.43E+01, 7.38E+01,  NaN,  NaN;... %97L1
    97, 1.32E-01, 1.22E+00, 4.06E+00,  NaN,  NaN;... %97K
    98, 1.28E-03, 5.20E+03, 5.54E+03,  NaN,  NaN;... %98N3
    98, 1.62E-03, 3.50E+03, 3.54E+03,  NaN,  NaN;... %98N2
    98, 1.80E-03, 2.84E+03, 2.89E+03,  NaN,  NaN;... %98N1
    98, 4.25E-03, 4.32E+02, 9.55E+02,  NaN,  NaN;... %98M5
    98, 4.50E-03, 8.55E+02, 1.18E+03,  NaN,  NaN;... %98M4
    98, 5.11E-03, 8.79E+02, 1.02E+03,  NaN,  NaN;... %98M3
    98, 6.36E-03, 5.90E+02, 6.25E+02,  NaN,  NaN;... %98M2
    98, 6.75E-03, 5.39E+02, 5.61E+02,  NaN,  NaN;... %98M1
    98, 1.99E-02, 3.91E+01, 8.61E+01,  NaN,  NaN;... %98L3
    98, 2.53E-02, 4.66E+01, 6.60E+01,  NaN,  NaN;... %98L2
    98, 2.61E-02, 6.06E+01, 6.95E+01,  NaN,  NaN;... %98L1
    98, 1.36E-01, 1.16E+00, 3.81E+00,  NaN,  NaN;... %98K
    99, 1.03E-03, 7.41E+03, 7.64E+03,  NaN,  NaN;... %99N4
    99, 1.32E-03, 5.08E+03, 5.41E+03,  NaN,  NaN;... %99N3
    99, 1.68E-03, 3.36E+03, 3.40E+03,  NaN,  NaN;... %99N2
    99, 1.87E-03, 2.73E+03, 2.77E+03,  NaN,  NaN;... %99N1
    99, 4.37E-03, 4.21E+02, 9.16E+02,  NaN,  NaN;... %99M5
    99, 4.63E-03, 8.24E+02, 1.14E+03,  NaN,  NaN;... %99M4
    99, 5.25E-03, 8.49E+02, 9.90E+02,  NaN,  NaN;... %99M3
    99, 6.57E-03, 5.63E+02, 5.96E+02,  NaN,  NaN;... %99M2
    99, 6.98E-03, 5.15E+02, 5.36E+02,  NaN,  NaN;... %99M1
    99, 2.04E-02, 3.81E+01, 8.46E+01,  NaN,  NaN;... %99L3
    99, 2.60E-02, 4.46E+01, 6.33E+01,  NaN,  NaN;... %99L2
    99, 2.69E-02, 5.81E+01, 6.67E+01,  NaN,  NaN;... %99L1
    99, 1.40E-01, 1.13E+00, 3.68E+00,  NaN,  NaN;... %99K
    100, 1.00E-03, 7.11E+03, 7.42E+03,  NaN,  NaN;... %100N5
    100, 1.07E-03, 7.11E+03, 7.32E+03,  NaN,  NaN;... %100N4
    100, 1.37E-03, 4.87E+03, 5.18E+03,  NaN,  NaN;... %100N3
    100, 1.75E-03, 3.17E+03, 3.21E+03,  NaN,  NaN;... %100N2
    100, 1.94E-03, 2.58E+03, 2.63E+03,  NaN,  NaN;... %100N1
    100, 4.50E-03, 4.03E+02, 8.66E+02,  NaN,  NaN;... %100M5
    100, 4.77E-03, 7.83E+02, 1.07E+03,  NaN,  NaN;... %100M4
    100, 5.40E-03, 8.09E+02, 9.43E+02,  NaN,  NaN;... %100M3
    100, 6.79E-03, 5.28E+02, 5.59E+02,  NaN,  NaN;... %100M2
    100, 7.21E-03, 4.83E+02, 5.03E+02,  NaN,  NaN;... %100M1
    100, 2.09E-02, 3.66E+01, 8.10E+01,  NaN,  NaN;... %100L3
    100, 2.68E-02, 4.21E+01, 5.98E+01,  NaN,  NaN;... %100L2
    100, 2.77E-02, 5.50E+01, 6.30E+01,  NaN,  NaN;... %100L1
    100, 1.43E-01, 1.09E+00, 3.50E+00,  NaN,  NaN;... %100K
];

end

%% Initialize variables
Element  = Element(:);
nData    = max(size(mac, 1),size(meac, 1));
nElement = length(Element);
nEnergy  = length(Energy);
AttCoef  = zeros(nEnergy,  nElement);
CompEdge = [];
if (nEnergy==0 || any(Energy<0.0009) || any(Energy>21))
  warning('PhotonAttenuation function: energy is outside of the recomended range from 0.001 MeV to 20 MeV');
end
Energy   = log(Energy(:));

%% Main Loop
for i = 1:nElement
  element = round(Element(i));
  
  if (ischar(element) || any(element<1) || any(element>nData))
   error(['Error in PhotonAttenuationQ function: Z number outside [1, 100] range: ', num2str(element)]);
  end

  x = energy';
  if(Options==1), y = mac (element, :)';
  else            y = meac(element, :)';  end
  if (element>10)      % if compton edges are present for this element
    ed = edges(edges(:, 1)==element, :); % check if 'element' has edges
    nEdge = size(ed, 1);
    e  = ed(:, 2)';    % energy of compton edges   
    d  = 4*eps(e);     % create offset to create 2 different energies for upper and lower end of the edge
    xx = [e-d; e+d];   % energies
    if(Options==1), yy = ed(:, 3:4)';     % mac values
    else            yy = ed(:, 5:6)'; end % meac values
    w  = [zeros(size(x)); ones(2*nEdge, 1)]; % this is array that will help keep track of edges
    x  = log([x; xx(:)]);
    y  = log([y; yy(:)]);
    [x ix] = sort(x);  % merge compton edge data with other data
    y  = y(ix, :);
    w  = w(ix, :); % array with 0 for 'grid' points and 1's for compton edge points
    w  = (convn(w, [1; 1; 1],  'same')>0); % meighbors of edges will be marked with 1's too
    b  = interp1(x, [y, w], Energy,  'linear',  'extrap'); % merge inputs for speed
    a  = interp1(x,  y,     Energy,  'cubic');
    v  = b(:, 2);
    b  = b(:, 1);
    a  = a.*(1-v) + b.*v; % smoothly merge curves using linear near edges and cubic otherwise
    %figure; plot(x,  y(:, 1), 'mx'); hold on; plot(Energy,  a(:, 1));
    CompEdge = [CompEdge; [(1:nEdge)',  flipud(ed)]];
  else % if there are no compton edges than life is easy
    a = interp1(log(x),  log(y),  Energy,  'cubic');
  end
  AttCoef(:,i) = exp(a);
end