%Solution for phase-attenuation duality PCI model.
%
% This script demonstrates the results of phase retrieval based on
% TV-minimization versus the standard Wiener deconvolution. 
% The model used to approximate phase-contrast effect is what we call 
% Dual CTF model or CTF for homogeneous objects 
% [L. D. Turner, OPTICS EXPRESS(2004), 12, pp. 2960].
%
% Input phase-contrast image is simulated using Fresnel propagation in
% <a href="matlab:help monochromatic_simulation.m">monochromatic_simulation.m</a> script.
%
% Similar example, that demonstrates phase-retrieval from multiple 
% phase-contrast images, can be found in <a href="matlab:help dual_ctf_multy_example.m">dual_ctf_multy_example.m</a>

close all
clear all
%% Load the input data:

% current dir:
dir = fileparts(mfilename('fullpath'));

% Load images:
pci_image = load_binary([dir, '\monochromatic_simulation\plastic_phantom_2m_pci.dat'], [256 256]);
ground_truth = load_binary([dir, '\monochromatic_simulation\plastic_phantom_mu.dat'], [256 256]);

% Show pci image:
show(pci_image, [], 1, 'Phase-contrast image');

%% Initialize linear propagator based on Dual CTF model:

% Experimental conditions:
energy = 20e3;          % KeV
pixelsize = 1e-6;       % microns
r1 = inf;               % parallel beam
r2 = 2;                 % meters

% Ratio between phase and attenuation coeficients for water-polyethylen
% sample at 20KeV: 
n_water = phantoms.material_refraction('WATER', energy);
n_polyethylen = phantoms.material_refraction('POLYETHYLENE', energy);

% Ratio between phase and attenuation effects needed to construct propagator 
delta = imag(n_water - n_polyethylen) ./ real(n_water - n_polyethylen);

% Propagator (Dual CTF):
p = PCI_rec.dual_ctf_propagator(pci_image, pixelsize, energy, r1, r2, delta);

%% Solve the inverse problem using Wiener deconvolution:

% Generate a model of the objet power spectrum (inverse):
w = PCI_rec.frequency_space(pci_image, pixelsize, energy, r1, r2);
ops = ifftshift(rrr(pci_image)) .* (w > pi / 2) .* numel(pci_image);

% Determine STD of the noise from the background (use the boundary of the image):
s = 2*std(pci_image(1,:)) ./ (max(pci_image(:)) - min(pci_image(:)));

% Wiener deconvolution:
wiener_recon = fft2(pci_image) .* p ./ (p.^2 + ops .* s^2);

% Set zero frequency to 1:
wiener_recon(1, 1) = 0;
wiener_recon = real(ifft2(wiener_recon));

% Show the result:
show(wiener_recon, [], 2, 'Wiener deconvolution');

%% Solve the inverse problem using TV-minimization approach:

% Initialize the TV_min:
f = TV_min();
 
% Define the linear model A(x):
f.A = @(x) p .* fft2(x);
f.A_trans = @(x) real(ifft2( p .* x ));

% Define x and y in A(x) = y:
f.X = wiener_recon .* 0;
f.Y = fft2(pci_image);
 
% Calculate the Lipschitz constant L = 2*max(|A|.^2):
f.L = 2*max(max(p.^2));
 
% Initialization parameters of the algorithm:
f.n_iter = 300;
f.lambda = f.L / numel(pci_image);
f.bounds = [-inf, inf];
f.X_true = ground_truth;
 
% Calculate!
[tv_recon, history] = f.run();
 
% Set mean to zero:
tv_recon = tv_recon - mean(tv_recon(:));

% Display the result:
show(tv_recon, [], 3 , 'TV-minimization');

% Display the history of convergence:
figure(4); 
semilogy(history.L2 + history.TV); title('Cost function');

figure(5); 
semilogy(history.err); title('Error');

%% Compare the results in real and in frequency space:

% Real space error of the Wiener reconstruction:
wiener_err = wiener_recon - ground_truth;
wiener_err = wiener_err - mean(wiener_err(:));

% Real space error of the Wiener reconstruction:
tv_err = tv_recon - ground_truth;
tv_err = tv_err - mean(tv_err(:));

% Show error in real space:
show(wiener_err, [], 5, 'Real space error: Wiener deconvolution'); 
colorbar;
show(tv_err, [], 6, 'Real space error: TV-mimimization deconvolution'); 
colorbar;

% Calculate average Fourier spectrum of the images and the associated
% errors:
ft_image = ifftshift(fft2(ground_truth));
image_spectrum = radial_average(abs(ft_image));

ft_image = ifftshift(fft2(wiener_err));
wiener_error_spectrum = radial_average(abs(ft_image));

ft_image = ifftshift(fft2(tv_err));
tv_error_spectrum = radial_average(abs(ft_image));

% 1D version of the propagator (to overlay it with the image spectra):
p_1d = radial_average(abs(ifftshift(p)));
p_1d = p_1d ./ max(p_1d);

% Show error in frequency space:
show(wiener_error_spectrum ./ image_spectrum, 'b', 7); hold on;
show(tv_error_spectrum ./ image_spectrum, 'r', 7); 
show(p_1d, 'k:', 7); hold off;

legend('Wiener error', 'TV error', 'Propagator envelope');
title('Error in frequency space');

%% Calculate the descripancy between Wiener and TV:
change_sp = radial_average(abs(ifftshift(fft2(wiener_recon - tv_recon))));
wiener_sp = radial_average(abs(ifftshift(fft2(wiener_recon))));

%Show:
show(change_sp ./ wiener_sp, 'b', 8);hold on;
show(p_1d ./ max(p_1d), 'k:', 8); hold off;
xlim([2,128]);
title('Difference with Wiener deconvolution');