%LSQ solution for multiple distances PCI based on CTF model.
%
% This script demonstrates the results of phase retrieval based on
% TV-minimization versus the standard Wiener deconvolution. 
% The model used to approximate phase-contrast effect is so called 
% CTF model. [X. Wu, H. Liu, J. X-Ray Sci. and Tech. (2003) 3342]
%
% Input phase-contrast images is simulated using Fresnel propagation in
% <a href="matlab:help monochromatic_simulation.m">monochromatic_simulation.m</a> script.
%
% In this script phase-retrieval is based on multiple phase-contrast images
% recorded at different distances. Similar example, that demonstrates
% phase-retrieval using additional assumption of homogeneity can be found
% in <a href="matlab:help dual_ctf_multy_example.m">dual_ctf_multy_example.m</a>.

close all
clear all
%% Load the input data:

% current dir:
dir = fileparts(mfilename('fullpath'));

% Load images:
pci_images = ...
    {load_binary([dir, '\monochromatic_simulation\plastic_phantom_0.5m_pci.dat'], [256 256]),...
    load_binary([dir, '\monochromatic_simulation\plastic_phantom_0.1m_pci.dat'], [256 256])}

ground_truth_phase = load_binary([dir, '\monochromatic_simulation\plastic_phantom_phi.dat'], [256 256]);

% Show pci image and ground truth:
show(pci_images{1}, [], 1, 'Phase-contrast image');
show(ground_truth_phase, [], 2, 'Phase retrieved image (ground truth)');

%% Initialize linear propagator based on Dual CTF model:

% Experimental conditions:
energy = 20e3;          % KeV
pixelsize = 1e-6;       % microns
r1 = inf;               % parallel beam
r2 = { 0.5, 0.1};       % meters

% Ratio between phase and attenuation coeficients for water-polyethylen
% sample at 20KeV: 
n_water = phantoms.material_refraction('WATER', energy);
n_polyethylen = phantoms.material_refraction('POLYETHYLENE', energy);

% Ratio between phase and attenuation effects needed to construct propagator 
delta = imag(n_water - n_polyethylen) ./ real(n_water - n_polyethylen);

%% Construct the Least-squares solution for multiple distances:
% [Langer M., Medical Physics, Vol. 35, No. 10, October 2008]
sz = size(pci_images{1});

% Intermediate images needed to compute the solution:
A = zeros(sz);
B = zeros(sz);
C = zeros(sz);
CC = zeros(sz);
AA = zeros(sz);

for ii = 1:length(pci_images)
    
    % Frequency space:
    w = PCI_rec.frequency_space(sz, pixelsize, energy, r1, r2{ii});  
    
    s = sin(w);
    c = cos(w); 

    A = A + s .* c;
    B = B + s.^2;
    C = C + c.^2;
    
    im_ft = fft2(pci_images{ii});
    AA = AA + im_ft .* c;
    CC = CC + im_ft .* s;
end

% Solution (phase image and attenuation image):

epsilon = 1e-3;
phi = 1 ./ (2 * (B.*C - A.^2) + epsilon) .* (C .* CC - A .* AA);
i0 =  1 ./ (2 * (B.*C - A.^2) + 1e-2) .* (A .* CC - B .* AA);

wiener_recon = real(ifft2(phi));

% show:
show(wiener_recon, [], 3, 'Wiener deconvolution: phase');

%% Solve the inverse problem using TV-minimization approach:

% Initialize the TV_min:
f = TV_min();
 
% Define the linear model A(x):
p = (2 * (B.*C - A.^2));
f.A = @(x) p .* fft2(x);
f.A_trans = @(x) real(ifft2( p .* x ));

% Define x and y in A(x) = y:
f.X = wiener_recon *0;
f.Y = (C .* CC - A .* AA);
 
% Calculate the Lipschitz constant L = 2*max(|A|.^2):
f.L = 2*max(max((p).^2));
 
% Initialization parameters of the algorithm:
f.n_iter = 6000;
f.lambda = f.L / numel(wiener_recon) * 250;
f.bounds = [-inf, inf];
f.X_true = ground_truth_phase;
 
% Calculate!
[tv_recon, history] = f.run();
 
% Set mean to zero:
tv_recon = tv_recon - mean(tv_recon(:));

% Display the result:
show(tv_recon, [], 4 , 'TV-minimization');

% Display the history of convergence:
figure(5); 
semilogy(history.L2 + history.TV); title('Cost function');

figure(6); 
semilogy(history.err); title('Error');

%% Compare the results in real and in frequency space:

% Real space error of the Wiener reconstruction:
wiener_err = wiener_recon - ground_truth_phase;
wiener_err = wiener_err - mean(wiener_err(:));

% Real space error of the Wiener reconstruction:
tv_err = tv_recon - ground_truth_phase;
tv_err = tv_err - mean(tv_err(:));

% Show error in real space:
show(wiener_err, [], 7, 'Real space error: Wiener deconvolution'); 
colorbar;
show(tv_err, [], 8, 'Real space error: TV-mimimization deconvolution'); 
colorbar;

% Calculate average Fourier spectrum of the images and the associated
% errors:
ft_image = ifftshift(fft2(ground_truth_phase));
image_spectrum = radial_average(abs(ft_image));

ft_image = ifftshift(fft2(wiener_err));
wiener_error_spectrum = radial_average(abs(ft_image));

ft_image = ifftshift(fft2(tv_err));
tv_error_spectrum = radial_average(abs(ft_image));

% 1D version of the propagator (to overlay it with the image spectra):
p_1d = sqrt(radial_average(abs(ifftshift(p))));
p_1d = p_1d ./ max(p_1d);

% Show error in frequency space:
show(wiener_error_spectrum ./ image_spectrum, 'b', 9); hold on;
show(tv_error_spectrum ./ image_spectrum, 'r',9); 
show(p_1d, 'k:', 9); hold off;
ylim([0, 2]);

legend('Wiener error', 'TV error', 'Propagator envelope');
title('Error in frequency space');

%% Calculate the descripancy between Wiener and TV:
change_sp = radial_average(abs(ifftshift(fft2(wiener_recon - tv_recon))));
wiener_sp = radial_average(abs(ifftshift(fft2(wiener_recon))));

%Show:
show(change_sp ./ wiener_sp, 'b', 10);hold on;
show(p_1d ./ max(p_1d), 'k:', 10); hold off;
xlim([2,128]);
title('Difference with Wiener deconvolution');