%Solution for multiple distances PCI based on CTF model.
%
% This script demonstrates the results of phase retrieval based on
% TV-minimization versus the standard Wiener deconvolution. 
%
% Check the <a href="matlab:help full_ctf_example.m">full_ctf_example.m</a> for more info. This version of the script
% uses simultaneous optimization of CTF equations for all distances instead
% of optimization of a single equation that corresponds to the LSQ solution.

close all
clear all
%% Load the input data:

% current dir:
dir = fileparts(mfilename('fullpath'));

% Load images:
pci_images = ...
    {load_binary([dir, '\monochromatic_simulation\plastic_phantom_0.1m_pci.dat'], [256 256]),...
    load_binary([dir, '\monochromatic_simulation\plastic_phantom_0.5m_pci.dat'], [256 256])}

ground_truth.phi = load_binary([dir, '\monochromatic_simulation\plastic_phantom_phi.dat'], [256 256]);
ground_truth.mu = load_binary([dir, '\monochromatic_simulation\plastic_phantom_mu.dat'], [256 256]);

% Show pci image and ground truth:
show(pci_images{1}, [], 1, 'Phase-contrast image');
show(ground_truth.phi, [], 2, 'Phase retrieved image (ground truth)');

%% Initialize linear propagator based on Dual CTF model:

% Experimental conditions:
energy = 20e3;          % KeV
pixelsize = 1e-6;       % microns
r1 = inf;               % parallel beam
r2 = {0.1, 0.5};       % meters

%% Solve the inverse problem using LSQ-based Wiener deconvolution:
sz = size(pci_images{1});

% Intermediate images needed to compute the solution:
A = zeros(sz);
B = zeros(sz);
C = zeros(sz);
CC = zeros(sz);
AA = zeros(sz);

for ii = 1:length(pci_images)
    
    % Frequency space:
    w = PCI_rec.frequency_space(sz, pixelsize, energy, r1, r2{ii});  
    
    s = sin(w);
    c = cos(w); 

    A = A + s .* c;
    B = B + s.^2;
    C = C + c.^2;
    
    im_ft = fft2(pci_images{ii});
    AA = AA + im_ft .* c;
    CC = CC + im_ft .* s;
end

% Solution (phase image and attenuation image):

epsilon.phi = 1e-3;
epsilon.mu = 1e0;
phi = 1 ./ (2 * (B.*C - A.^2) + epsilon.phi) .* (C .* CC - A .* AA);
mu =  1 ./ (2 * (B.*C - A.^2) + epsilon.mu) .* (A .* CC - B .* AA);

wiener_recon.phi = real(ifft2(phi));
wiener_recon.mu = real(ifft2(mu));

% show:
show(wiener_recon.phi, [], 3, 'Wiener deconvolution: phase');
show(wiener_recon.mu, [], 4, 'Wiener deconvolution: intensity');

%clear C CC A B AA mu phi;
%% Construct the system of equations for multiple distances (use it in TV):

% Initialize the TV_min:
f = TV_min_2();

% Known variables (cell array):
f.Y = cell(1, length(pci_images));

% transformation matrixes for two variables system (phi and mu):
f.A.mu = cell(1, length(pci_images));
f.A_trans.mu = cell(1, length(pci_images));

f.A.phi = cell(1, length(pci_images));
f.A_trans.phi = cell(1, length(pci_images));

f.L.mu = 0;
f.L.phi = 0;

for ii = 1:length(pci_images)

    % Frequency space:
    w = PCI_rec.frequency_space(sz, pixelsize, energy, r1, r2{ii});
    s = 2*sin(w);
    c = -2*cos(w);
    
    % attenuation propagator:
    f.A.mu{ii} = @(x) c .* fft2(x);
    
    % phase propagator:
    f.A.phi{ii} = @(x) s .* fft2(x);
    
    % transposed propagators:
    f.A_trans.mu{ii} = @(x) real(ifft2(c .* x ));
    f.A_trans.phi{ii} = @(x) real(ifft2(s .* x ));
    
    f.Y{ii} = fft2(pci_images{ii} - 1);
       
    f.L.phi = 2 * max(max(abs(s(:)).^2), f.L.phi/2);
    f.L.mu = 2 * max(max(abs(c(:)).^2), f.L.mu/2);
end
%% Solve the system of equations using TV-minimization approach:

% Unknown variables (can be two: mu and phi), initial guess:
f.X.mu = zeros(sz);
f.X.phi = zeros(sz);

% Initialization parameters of the algorithm:
f.n_iter = 1000;

f.lambda.mu = f.L.mu / numel(wiener_recon.phi) * 300;
f.lambda.phi = f.L.phi / numel(wiener_recon.phi) * 300;

f.X_true = ground_truth;
 
f.bounds.mu = [-inf, inf];
f.bounds.phi = [-inf, inf];

% Calculate!
[tv_recon, history] = f.run();
 
% Set mean to zero:
tv_recon.mu = tv_recon.mu - mean(tv_recon.mu(:));
tv_recon.phi = tv_recon.phi - mean(tv_recon.phi(:));

% Display the result:
show(tv_recon.phi, [], 4 , 'TV-minimization: phi');
show(tv_recon.mu, [], 5 , 'TV-minimization: mu');

% Display the history of convergence:
figure(6); 
semilogy(history.L2.phi + history.TV.phi); title('Cost function: phi');

figure(7); 
semilogy(history.err.phi); title('Error: phi');

%% Compare the results in real and in frequency space:

% Real space error of the Wiener reconstruction:
wiener_err = wiener_recon.phi - ground_truth.phi;
wiener_err = wiener_err - mean(wiener_err(:));

% Real space error of the Wiener reconstruction:
tv_err = tv_recon.phi - ground_truth.phi;
tv_err = tv_err - mean(tv_err(:));

% Show error in real space:
show(wiener_err, [], 7, 'Real space error: Wiener deconvolution'); 
colorbar;
show(tv_err, [], 8, 'Real space error: TV-mimimization deconvolution'); 
colorbar;

% Calculate average Fourier spectrum of the images and the associated
% errors:
ft_image = ifftshift(fft2(ground_truth.phi));
image_spectrum = radial_average(abs(ft_image));

ft_image = ifftshift(fft2(wiener_err));
wiener_error_spectrum = radial_average(abs(ft_image));

ft_image = ifftshift(fft2(tv_err));
tv_error_spectrum = radial_average(abs(ft_image));

% 1D version of the propagator (to overlay it with the image spectra):
p_1d = sqrt(radial_average(abs(ifftshift(2 * (B.*C - A.^2)))));
p_1d = p_1d ./ max(p_1d);

% Show error in frequency space:
show(wiener_error_spectrum ./ image_spectrum, 'b', 9); hold on;
show(tv_error_spectrum ./ image_spectrum, 'r',9); 
show(p_1d, 'k:', 9); hold off;
ylim([0, 3]);

legend('Wiener error', 'TV error', 'Propagator envelope');
title('Error in frequency space');

%% Calculate the descripancy between Wiener and TV:
change_sp = radial_average(abs(ifftshift(fft2(wiener_recon.phi - tv_recon.phi))));
wiener_sp = radial_average(abs(ifftshift(fft2(wiener_recon.phi))));

%Show:
show(change_sp ./ wiener_sp, 'b', 10);hold on;
show(p_1d ./ max(p_1d), 'k:', 10); hold off;
xlim([2,128]);
title('Difference with Wiener deconvolution');